package acube.framework.captcha;

/**
 * CaptchaParameters
 * 
 * @date 20 dc. 07
 * @author EDS - bzgn4l
 */
public class CaptchaParameters {

    /**
     * <code>CAPTCHA_CONFIF_FILE</code> the CAPTCHA_CONFIF_FILE
     * refers the path of captcha configuration file. This file was
     * read on server start.
     */
    public static final String CAPTCHA_CONFIF_FILE = "/config/captcha.xml";

    /**
     * <code>WEB_INF_DIR</code> the WEB_INF_DIR refers WEB-INF
     * directory
     */
    public static final String WEB_INF_DIR = "/WEB-INF";

    /**
     * <code>CAPTCHA_IMAGE_CONTENT_TYPE</code> the
     * CAPTCHA_IMAGE_CONTENT_TYPE refers the type of http response. It
     * dependents of captcha's type.
     */
    public static final String CAPTCHA_IMAGE_CONTENT_TYPE = "image/jpeg";

    /**
     * <code>CAPTCHA_PARAM</code> the CAPTCHA_PARAM it use during
     * the treatment of the serveur answer. more precisely during the
     * compute response captcha call by a action. This parameter is
     * use by
     * {@link CaptchaManager#getCaptchaParam(javax.servlet.http.
     * HttpServletRequest, String)}
     * This parameter was sent by the html form, contaner of Acube's
     * ComposantCaptcha. It contains several parameters values
     * separate by separators expressions
     * {@link CaptchaParameters#CAPTCHA_PARAM_SPLIT_EXP} and
     * {@link CaptchaParameters#CAPTCHA_PARAM_SPLIT_EXP}
     * 
     * Example : parameter : "captcha" value :
     * "captcha_name=simple&captcha_user_key=userkey"
     * 
     * If you want some parameter in captcha's value use
     * {@link CaptchaManager#getCaptchaParam(javax.servlet.http.
     * HttpServletRequest, String)}.
     */
    public static final String CAPTCHA_PARAM = "captcha";

    /**
     * <code>CAPTCHA_PARAM_SPLIT_EXP</code> the
     * CAPTCHA_PARAM_SPLIT_EXP refers the expression use to separate
     * parameters in captcha parameter value.
     */
    public static final String CAPTCHA_PARAM_SPLIT_EXP = ";";

    /**
     * <code>CAPTCHA_PARAM_EQUAL_EXP</code> the
     * CAPTCHA_PARAM_EQUAL_EXP * <code>CAPTCHA_PARAM_SPLIT_EXP</code>
     * the CAPTCHA_PARAM_SPLIT_EXP refers the expression use to
     * separate parameter 's id and parameter 's value in captcha
     * parameter value.
     */
    public static final String CAPTCHA_PARAM_EQUAL_EXP = "=";

    /**
     * <code>CAPTCHA_NAME_PARAM</code> the CAPTCHA_NAME_PARAM refers
     * <code>captcha_name</code> parameter send by the form.
     */
    public static final String CAPTCHA_PARAM_NAME = "captcha_name";

    /**
     * <code>CAPTCHA_USER_KEY_PARAM</code> the
     * CAPTCHA_USER_KEY_PARAM refers
     * <code>captcha_user_key</code> parameter send send by the form.
     */
    public static final String CAPTCHA_PARAM_USER_KEY = "captcha_user_key";

    // ////////////////////////////////////////////////////////
    // XML PARAMETERS
    //
    // CAPTCHA_TAG
    // NAME_ATTRIBUT ID_ATTRIBUT CLASS_ATTRIBUT
    //
    // BEAN_TAG
    // ID_ATTRIBUT CLASS_ATTRIBUT
    //
    // CONSTRUCTOR_ARG_TAG
    // INDEX_ATTRIBUTE TYPE_ATTRIBUTE
    // VALUE_TAG LIST_TAG
    // ////////////////////////////////////////////////////////

    /**
     * <code>CAPTCHA_TAG</code> the CAPTCHA_TAG refers
     * <code>&lt;captcha&gt;</code> tag id in the captcha
     * configuration file.
     */
    public static final String CAPTCHA_TAG = "captcha";

    /**
     * <code>BEAN_TAG</code> the BEAN_TAG refers
     * <code>&lt;bean&gt;</code> tag id in the captcha configuration
     * file.
     */
    public static final String BEAN_TAG = "bean";

    /**
     * <code>ID_ATTRIBUT</code> the ID_ATTRIBUT refers id of
     * <code>id</code> attribute if <code>&lt;captcha&gt;</code> or 
     * <code>&lt;bean&gt;</code>. Remember that
     * every <code>id</code> attribute must be unique in the capcha
     * configuration file.
     */
    public static final String ID_ATTRIBUT = "id";

    /**
     * <code>CLASS_ATTRIBUT</code> the CLASS_ATTRIBUT refers the
     * <code>class</code> attribute in the
     * <code>&lt;captcha&gt;</code> or <code>&lt;bean&gt;</code>
     * tag.
     */
    public static final String CLASS_ATTRIBUT = "class";

    /**
     * <code>NAME_ATTRIBUT</code> the NAME_ATTRIBUT refers id of
     * <code>name</code> attribute of the
     * <code>&lt;captcha&gt;</code> tag. Remember that every
     * <code>name</code> attribute value must be unique in the
     * capcha configuration file.
     */
    public static final String NAME_ATTRIBUT = "name";

    /**
     * <code>CONSTRUCTOR_ARG_TAG</code> the CONSTRUCTOR_ARG_TAG
     * refers constructors arguments
     * <code>&lt;constructor-arg&gt;</code> tags in
     * <code>&lt;bean&gt;</code> tags.
     */
    public static final String CONSTRUCTOR_ARG_TAG = "constructor-arg";

    /**
     * <code>INDEX_ATTRIBUTE</code> the INDEX_ATTRIBUTE refers the
     * attribute <code>index</code> of the
     * <code>&lt;constructor-arg&gt;</code> tag.
     */
    public static final String INDEX_ATTRIBUTE = "index";

    /**
     * <code>TYPE_ATTRIBUTE</code> the TYPE_ATTRIBUTE refers the
     * attribute <code>type</code> of the
     * <code>&lt;constructor-arg&gt;</code> tag.
     */
    public static final String TYPE_ATTRIBUTE = "type";

    /**
     * <code>LIST_TAG</code> the LIST_TAG refers the tag
     * <code>&lt;list&gt;</code>, child of the
     * <code>&lt;constructor-arg&gt;</code> tag.
     */
    public static final String LIST_TAG = "list";

    /**
     * <code>VALUE_TAG</code> the VALUE_TAG refers the tag
     * <code>&lt;value&gt;</code>, child of the
     * <code>&lt;constructor-arg&gt;</code> tag.
     */
    public static final String VALUE_TAG = "value";

    /**
     * <code>SESSION_CAPTCHA_ID</code> the SESSION_CAPTCHA_ID
     * key in session attribute
     */
    public static final String SESSION_CAPTCHA_ID = "SessionCaptchaID";

    /**
     * <code>RANDOM_MAX_INT</code> the RANDOM_MAX_INT
     * range for generation id
     */
    public static final int RANDOM_MAX_INT = 100000;

    /**
     * <code>RANDOM_KEY_INT</code> the RANDOM_KEY_INT
     * key used in encryption generation for captcha id.
     */
    public static final int RANDOM_KEY_INT = 256;

    /**
     * <code>ALGORITHM_USE</code> the ALGORITHM_USE
     */
    public static final String ALGORITHM_USE = "MD5";

    /**
     * <code>ERROR_MESSAGE</code> the ERROR_MESSAGE
     *  
     */
    public static final String ERROR_MESSAGE =
        "delegateInstantiation() une erreur est survenue "
        + " lors de l'instanciation de la classe :";
}
