package acube.framework.captcha.captchaservices;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.jdom.Element;

import acube.framework.captcha.CaptchaAbstractService;
import acube.framework.captcha.CaptchaParameters;
import acube.framework.captcha.exception.CaptchaAcubeException;
import acube.framework.captcha.manager.CaptchaManager;

import com.octo.captcha.service.CaptchaServiceException;
import com.octo.captcha.service.image.ImageCaptchaService;
import com.sun.image.codec.jpeg.ImageFormatException;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

/**
 * Service for picture captcha types.
 * 
 * @date 20 dc. 07
 * @author EDS - bzgn4l
 * 
 */
public class CaptchaImageService extends CaptchaAbstractService {

    /**
     * <code>log</code> the log for CaptchaImageService class.
     */
    private static Logger log = Logger.getLogger(CaptchaImageService.class);

    /**
     * CaptchaImageService permet l'utilisation des captchas de type
     * image. {@link CaptchaAbstractService#CaptchaAbstractService()}
     */
    public CaptchaImageService() {

        super();
        this.setContentType(CaptchaParameters.CAPTCHA_IMAGE_CONTENT_TYPE);
    }

    /** {@inheritDoc} */
    public byte[] getOutStream(
          final String idCaptcha, final Locale LLocale)
                                    throws CaptchaAcubeException {

        // the output stream to render the captcha image as jpeg into
        ByteArrayOutputStream jpegOutputStream = new ByteArrayOutputStream();
        try {
            BufferedImage challenge =
                    ((ImageCaptchaService) this.getOctoCaptchaServiceInstance())
                            .getImageChallengeForID(
                                idCaptcha, LLocale);

            // a jpeg encoder
            JPEGImageEncoder jpegEncoder =
                    JPEGCodec.createJPEGEncoder(jpegOutputStream);
            jpegEncoder.encode(challenge);

        } catch (ImageFormatException e) {
            throw new CaptchaAcubeException(
                    e);
        } catch (IOException e) {
            throw new CaptchaAcubeException(
                    e);
        }
        return jpegOutputStream.toByteArray();
    }

    /** {@inheritDoc} */
    public void parseElement(
            final Element eCaptcha) throws CaptchaAcubeException {

        CaptchaImageService.log.info("Call of parseElement");
        Element beanService = eCaptcha.getChild(CaptchaParameters.BEAN_TAG);
        String sServiceId = beanService.getAttribute(
            CaptchaParameters.ID_ATTRIBUT).getValue();
        String sServiceClass = beanService.getAttribute(
            CaptchaParameters.CLASS_ATTRIBUT).getValue();
        CaptchaImageService.log.debug("service_id:" + sServiceId);
        CaptchaImageService.log.debug("service_class:" + sServiceClass);

        Object aCaptchaService =
                CaptchaManager.getInstance().delegateInstantiation(
                    sServiceClass);
        this.setOctoCaptchaServiceInstance(
            (ImageCaptchaService) aCaptchaService);
        this.setOctoCaptchaServiceId(sServiceId);
    }

    /** {@inheritDoc} */
    public boolean computeValidation(
            final String idCaptcha, final String response)
                                throws CaptchaAcubeException {

        try {
            return ((ImageCaptchaService) this
                            .getOctoCaptchaServiceInstance())
                            .validateResponseForID(
                                idCaptcha, response).booleanValue();
        } catch (CaptchaServiceException e) {
            CaptchaImageService.log.error(
                "Problme lors du computeValidation()", e);
            throw new CaptchaAcubeException(
                    e);
        }
    }
}
