package acube.framework.captcha.manager;

/**
 * CaptchaManager is the class for the management of Acube and
 * JCaptcha CaptchaService.
 * 
 * @date 19 dc. 07
 * @author EDS - bzgn4l
 */
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import acube.framework.captcha.CaptchaAbstractService;
import acube.framework.captcha.CaptchaParameters;
import acube.framework.captcha.captchaservices.CaptchaImageService;
import acube.framework.captcha.exception.CaptchaAcubeException;
import acube.framework.exception.BusinessException;


/**
 * Projet AppliTestLISE_J2EE.
 * 
 * @date 19 dc. 07
 * @author bzgn4l
 */
public class CaptchaManager {

    /**
     * <code>THIS_CLASS</code> - CaptchaManager's class name.
     */
    private static final String THIS_CLASS = "[CaptchaManager]";

    /**
     * <code>ERR_TEC_CAPTCHA_01</code> - the ERR_TEC_CAPTCHA_01 is
     * raised when this
     * {@link CaptchaParameters#CAPTCHA_PARAM parameter} is absent is
     * http servlet request.
     */
    private static final String ERR_TEC_CAPTCHA_01 = "ERR_TEC_CAPTCHA_01";

    /**
     * <code>ERR_TEC_CAPTCHA_02</code> - the ERR_TEC_CAPTCHA_02 is
     * raised when ERR_TEC_CAPTCHA_01 was raised, Acube Captcha
     * Service can't work without this parameter.
     */
    private static final String ERR_TEC_CAPTCHA_02 = "ERR_TEC_CAPTCHA_02";

    /**
     * <code>log</code> the log - CaptchaManager's logger.
     */
    private static Logger log = Logger.getLogger(CaptchaManager.class);

    /**
     * singleton.
     */
    private static CaptchaManager instance = new CaptchaManager();

    /**
     * <code>captchaMap</code> - the captchaMap is the Acube captcha
     * services map. It contains all Acube captcha services instances.
     */
    private HashMap<String, Object> captchaMap = null;

    /**
     * <code>ListIDs</code> the ListIDs refers ids from captcha tag
     */
    private ArrayList<String> aListIDs = null;

    /**
     * Initialize {@link CaptchaManager#captchaMap captchaMap} and
     * {@link CaptchaManager#hCaptchaNameSwitchMap hCaptchaNameSwitchMap}
     * attributes.
     */
    public CaptchaManager() {

        this.captchaMap = new HashMap<String, Object>();
        this.aListIDs = new ArrayList<String>();
    }

    /**
     * @return CaptchaManager's instance.
     */
    public static CaptchaManager getInstance() {

        return CaptchaManager.instance;
    }

    /**
     * delegateInstantiation method create a new instance for the
     * class name.
     * 
     * @param classPath
     *            class name
     * @return the instance
     * @throws CaptchaAcubeException 
     *  may be InstantiationException, IllegalAccessException
     *   and ClassNotFoundException
     *  
     */
    public Object delegateInstantiation(
            String classPath) throws CaptchaAcubeException {

        Object aCaptchaServiceInstance;
        try {
            aCaptchaServiceInstance = Class.forName(
                classPath).newInstance();

        } catch (InstantiationException e) {
            CaptchaManager.log.error(CaptchaParameters.ERROR_MESSAGE
                    + classPath + " de Type : [InstantiationException]");
            throw new CaptchaAcubeException(
                    e);
        } catch (IllegalAccessException e) {
            CaptchaManager.log.error(CaptchaParameters.ERROR_MESSAGE
                    + classPath + " de Type : [IllegalAccessException]");
            throw new CaptchaAcubeException(
                    e);
        } catch (ClassNotFoundException e) {
            CaptchaManager.log.error(CaptchaParameters.ERROR_MESSAGE
                    + classPath + " de Type : [ClassNotFoundException]");
            throw new CaptchaAcubeException(
                    e);
        }
        return aCaptchaServiceInstance;
    }

    /**
     * CaptchaSelection method select the specific Acube
     * CaptchaService by captcha name.
     * 
     * @param captchaName
     *            the name of the captcha to use.
     * @return the Acube CaptchaService selected.
     * @throws CaptchaAcubeException raise by 
     * {@link CaptchaManager#getCaptchaByName}
     */
    public CaptchaAbstractService doCaptchaSelection(
            String captchaName) throws CaptchaAcubeException {

        CaptchaManager.log.info("Call of CaptchaSelection");
        CaptchaAbstractService cs = null;
        CaptchaAbstractService myCaptcha =
                ((CaptchaAbstractService) CaptchaManager.getInstance()
                        .getCaptchaByName(
                            captchaName));
        if (myCaptcha != null) {
            Integer swithId = myCaptcha.getId();
            switch (swithId) {
                case 1:
                    cs = (CaptchaImageService) CaptchaManager.getInstance()
                                    .getCaptchaByName(
                                        captchaName);

                    break;
                default:
                    CaptchaManager.log.error("Wrong switchId :" + swithId
                            + "You have to implement your new captcha you "
                            + "describe in captcha configuration xml file.");
                    return null;

            }
        } else {
            throw new CaptchaAcubeException(
                    "wrong name ! " + captchaName + " doesn't exist");
        }

        return cs;
    }

    /**
     * this method create a new instance of specific Acube captcha
     * service and put this one into
     * {@link CaptchaManager#captchaMap CaptchaManager's attribute }
     * 
     * @param name captcha name
     * @param classPath path class
     * @throws CaptchaAcubeException raised by 
     * {@link CaptchaManager#delegateInstantiation()}
     */
    public void addCaptcha(
            String name, String classPath) throws CaptchaAcubeException {

        CaptchaManager.log.info("Call of CaptchaServiceLoader");
        this.captchaMap.put(
            name, this.delegateInstantiation(classPath));

    }

    /**
     * This getter give you the Acube Captcha Service by the name.
     * 
     * @param name
     *            the name of your catpcha
     * @return specific Acube captcha service.
     */
    public CaptchaAbstractService getCaptchaByName(
            String name) {

        return ((CaptchaAbstractService) this.captchaMap.get(name));
    }

    /**
     * Recuperation d'un attribut en requete sous forme de Integer.
     * Rejet FunctionalException si absence de l'objet en requete ou
     * mauvais format.
     * 
     * @param request
     *            HttpServletRequest requete
     * @param cle
     *            String cle dont on veut avoir la valeur en session
     * @return Integer
     */
    public static String getCaptchaParam(
            HttpServletRequest request, String cle) {

        String valeur = null;
        valeur = request.getParameter(CaptchaParameters.CAPTCHA_PARAM);
        if (null == valeur) {
            CaptchaManager.log.error(CaptchaManager.THIS_CLASS
                + "Impossible de rcuprer le captcha en cours");
            throw new BusinessException(CaptchaManager.ERR_TEC_CAPTCHA_02, cle);
        } else {
            String paramNvalueTab[] =
                    valeur.split(CaptchaParameters.CAPTCHA_PARAM_SPLIT_EXP);
            for (String indice : paramNvalueTab) {
                String paramsTab[] =
                        indice.split(CaptchaParameters.CAPTCHA_PARAM_EQUAL_EXP);
                String paramid = paramsTab[0];
                String paramValue = paramsTab[1];

                if ((paramid != null) && (paramid.equals(cle))) {
                    return paramValue;
                }
            }
        }
        return valeur;
    }

    /**
     * Check method in purpose to validate the captcha configuration.
     *
     * @return return true if all captcha id are unique.
     * @throws CaptchaAcubeException raised by
     *  {@link CaptchaManager#checkTools(Collection)}
     */
    public boolean checkIds() throws CaptchaAcubeException {

        return this.checkTools(CaptchaManager.getListIDs());
    }

    /**
     * Check method in purpose to validate the captcha configuration.
     *
     * @return return true if all captcha name are unique.
     */
    public boolean checkNames() {

        return (this.captchaMap.keySet().size() == CaptchaManager.getListIDs()
                .size());
    }

    /**
     * Check method in purpose to validate the captcha configuration.
     *
     * @param myCollection collection of values
     * @return return true if all captcha id are unique.
     * @throws CaptchaAcubeException raised if 'id' is not unique  
     */
    public boolean checkTools(
            Collection<String> myCollection) throws CaptchaAcubeException {

        boolean res = false;

        Iterator<String> iter = myCollection.iterator();
        ArrayList<String> checkedIds = new ArrayList<String>();
        while (iter.hasNext()) {
            String id = (String) iter.next();
            if (checkedIds.indexOf(id) >= 0) {
                throw new CaptchaAcubeException(
                        id + " must be unique in the configuration file.");
            } else {
                checkedIds.add(id);
            }
        }
        return !res;
    }

    /**
     * 
     * @return the captchamanager list of id
     */
    public static ArrayList<String> getListIDs() {

        return CaptchaManager.getInstance().aListIDs;
    }
}
