package acube.framework.captcha.manager;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import acube.framework.captcha.CaptchaParameters;
import acube.framework.captcha.exception.CaptchaAcubeException;

/**
 * This class initialize the {@link CaptchaManager CaptchaManager}
 * class and parse the captcha configuration file defined
 * {@link CaptchaParameters#CAPTCHA_CONFIF_FILE here}.
 * 
 * @date 20 dc. 07
 * @author EDS - bzgn4l
 */
public class CaptchaManagerLoader {

    /**
     * <code>log</code> the log for CaptchaManagerLoader class.
     */
    private static Logger log = Logger.getLogger(CaptchaManagerLoader.class);

    /**
     * 
     */
    private CaptchaManagerLoader() {

    }

    /**
     * Call on application starting.<br/> Create an instance of
     * captcha manager.<br/> Call the
     * {@link CaptchaManagerLoader#parseCaptcha(Element)} method.
     * 
     * @param inputStream
     *            the configuration file of captcha.
     * @throws CaptchaAcubeException
     *             is raised by the
     *             {@link CaptchaManagerLoader#parseCaptcha(Element)}
     *             method.
     */
    public static void initCaptchaService(
            InputStream inputStream) throws CaptchaAcubeException {

        CaptchaManagerLoader.log.info("Call of CaptchaServiceLoader");
        SAXBuilder sxb = new SAXBuilder();
        Document document;
        Element racine;
        try {

            CaptchaManagerLoader.log
              .debug("lecture fichier de configuration du service captcha.");
            document = sxb.build(inputStream);
            racine = document.getRootElement();
            Iterator< ? >  iter = racine.getChildren(
                CaptchaParameters.CAPTCHA_TAG).iterator();

            while (iter.hasNext()) {
                CaptchaManagerLoader.parseCaptcha((Element) iter.next());
            }

            if (!CaptchaManager.getInstance().checkIds()) {
                throw new CaptchaAcubeException(
                   "Bad configuration file. Yours 'id' attribute of <captcha>"
                                + " tag must unique. "
                                + "please check your configuration file here :"
                                + CaptchaParameters.WEB_INF_DIR
                         .concat(CaptchaParameters.CAPTCHA_CONFIF_FILE));
            }
            if (!CaptchaManager.getInstance().checkNames()) {
                throw new CaptchaAcubeException(
                        "Bad configuration file. "
                                + "Yours 'name' attribute of <captcha>"
                                + " tag must unique. "
                                + "please check your configuration file here :"
                                + CaptchaParameters.WEB_INF_DIR
                            .concat(CaptchaParameters.CAPTCHA_CONFIF_FILE));
            }
        } catch (JDOMException e) {
            CaptchaManagerLoader.log
                    .error("initCaptchaService(); une erreur [JDOMException] "
                            + "est survenue lors de la lecture du fichier"
                            + CaptchaParameters.WEB_INF_DIR
                            .concat(CaptchaParameters.CAPTCHA_CONFIF_FILE)
                            + " par le parseur DOM");
            throw new CaptchaAcubeException(
                    e);
        } catch (IOException e) {
            CaptchaManagerLoader.log
                    .error("initCaptchaService(); une erreur [IOException]"
                         + " est survenue lors de la rcuperation du fichier :"
                            + CaptchaParameters.WEB_INF_DIR
                            .concat(CaptchaParameters.CAPTCHA_CONFIF_FILE));
            throw new CaptchaAcubeException(
                    e);
        }
    }

    /**
     * parseCaptcha method adds the new Acube captcha service in the
     * {@link CaptchaManager CaptchaManager} and parses the captcha
     * configuration xml file. Finally, it calls the specific
     * {@link CaptchaAbstractService#parseElement(Element)} method.
     * 
     * @param captcha :
     *            Element (jDom) recovered on reading of the captcha
     *            's configuration file. It is about the tag
     *            &lt;captcha&gt;.
     * @throws CaptchaAcubeException
     *             is raised by the
     *             {@link CaptchaManager#addSimpleCaptcha(String, String)}
     *             method.
     */
    private static void parseCaptcha(
            Element captcha) throws CaptchaAcubeException {

        // name attribut
        String sName, sId, sClass = null;
        if ((captcha.getAttribute(CaptchaParameters.NAME_ATTRIBUT) != null)
                && (captcha.getAttribute(
                    CaptchaParameters.NAME_ATTRIBUT).getValue().length() > 0)) {
            sName = captcha.getAttribute(
                CaptchaParameters.NAME_ATTRIBUT).getValue();
        } else {
            throw new CaptchaAcubeException(
                    "Bad configuration file. Your <captcha> tag must have a "
                     + "'name' attribute and its value must be not empty");
        }
        // id attribut
        if ((captcha.getAttribute(CaptchaParameters.ID_ATTRIBUT) != null)
                && (captcha.getAttribute(
                    CaptchaParameters.ID_ATTRIBUT).getValue().length() > 0)) {
            sId = captcha.getAttribute(
                CaptchaParameters.ID_ATTRIBUT).getValue();
        } else {
            throw new CaptchaAcubeException(
                    "Bad configuration file. Your <captcha> tag must have an "
                            + "'id' attribute and its value must be not empty");
        }
        // class attribut
        if ((captcha.getAttribute(CaptchaParameters.CLASS_ATTRIBUT) != null)
                && (captcha.getAttribute(
              CaptchaParameters.CLASS_ATTRIBUT).getValue().length() > 0)) {
            sClass = captcha.getAttribute(
                CaptchaParameters.CLASS_ATTRIBUT).getValue();
        } else {
            throw new CaptchaAcubeException(
                    "Bad configuration file. Your <captcha> tag must have a "
              + "'class' attribute and its value must be not empty");
        }
        // Rferencement
        CaptchaManager.getInstance().addCaptcha(
            sName, sClass);
        CaptchaManager.getInstance().getCaptchaByName(
            sName).setId(
            Integer.valueOf(sId));
        CaptchaManager.getListIDs().add(
            sId);
        CaptchaManager.getInstance().getCaptchaByName(
            sName).parseElement(
            captcha);
    }
}
