package acube.framework.captcha.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import acube.framework.captcha.CaptchaAbstractService;
import acube.framework.captcha.CaptchaParameters;
import acube.framework.captcha.CaptchaTools;
import acube.framework.captcha.exception.CaptchaAcubeException;
import acube.framework.captcha.manager.CaptchaManager;
import acube.framework.captcha.manager.CaptchaManagerLoader;

import com.octo.captcha.service.CaptchaServiceException;

/**
 * Captcha servlet controller.<br/> It recieves http request from
 * user navigator and calls
 * {@link CaptchaAbstractService#getOutStream(String, java.util.Locale)}
 * method of the specific captcha by the http post parameter "name".
 * 
 * @date 19 dc. 07
 * @author bzgn4l
 */
public class CaptchaServlet extends HttpServlet {

    /**
     * serialVersionUID.
     */
    private static final long serialVersionUID = 1L;

    /**
     * <code>log</code> the log for CaptchaServlet class.
     */
    private static Logger log = Logger.getLogger(CaptchaServlet.class);

    /** {@inheritDoc} */
    public void init(
            ServletConfig servletConfig) throws ServletException {

        super.init(servletConfig);
        try {

            InputStream inputStream =
                    servletConfig.getServletContext().getResourceAsStream(
                        CaptchaParameters.WEB_INF_DIR
                                .concat(CaptchaParameters.CAPTCHA_CONFIF_FILE));

            if (inputStream != null) {
                CaptchaManagerLoader.initCaptchaService(inputStream);
            } else {
                throw new ServletException(
                        "Path not found");
            }
        } catch (CaptchaAcubeException e) {
            String message =
                    "\n*****************************\n"
                            + "CaptchaServlet.init() -- starting Error : \n "
                            + e + "\n*****************************\n";
            CaptchaServlet.log.error(message);
            System.out.println(message);
            throw new ServletException(
                    e);
        }
    }

    /** {@inheritDoc} */
    protected void doGet(
            HttpServletRequest httpServletRequest,
            HttpServletResponse httpServletResponse) throws ServletException,
        IOException {

        String captchaName = httpServletRequest.getParameter("name");
        byte captchaChallengeByte[] = null;
        if (captchaName != null && captchaName.length() > 0) {

            CaptchaAbstractService cs = null;
            try {

                cs = CaptchaManager.getInstance().doCaptchaSelection(
                    captchaName);
                // gnration de l'id
                String sId = CaptchaTools.getInstance().getSessionId();
                if (sId.length() > 0) {

                    httpServletRequest.getSession().setAttribute(
                        CaptchaParameters.SESSION_CAPTCHA_ID, sId);
                    captchaChallengeByte = cs.getOutStream(
                        sId, httpServletResponse.getLocale());
                    // flush it in the response
                    httpServletResponse.setHeader(
                        "Cache-Control", "no-store");
                    httpServletResponse.setHeader(
                        "Pragma", "no-cache");
                    httpServletResponse.setDateHeader(
                        "Expires", 0);
                    httpServletResponse.setContentType(cs.getContentType());
                    ServletOutputStream responseOutputStream =
                            httpServletResponse.getOutputStream();
                    responseOutputStream.write(captchaChallengeByte);
                    responseOutputStream.flush();
                    responseOutputStream.close();
                } else {
                    throw new CaptchaAcubeException(
                            "Prob CaptchaTools.getSessionId() return nothing");
                }
            } catch (IllegalArgumentException e) {
                httpServletResponse.sendError(HttpServletResponse.SC_NOT_FOUND);
                return;
            } catch (CaptchaServiceException e) {
                httpServletResponse
                        .sendError(
                            HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                return;
            } catch (CaptchaAcubeException e) {
                CaptchaServlet.log.error(
                    "Error Catpcha Name :" + captchaName, e);
                httpServletResponse.sendError(HttpServletResponse.SC_NOT_FOUND);
            } catch (NoSuchAlgorithmException e) {
                CaptchaServlet.log.error(
                    "MD5 not supported", e);
            }
        } else {
            CaptchaServlet.log.error("You must specify a 'name'"
                    + " parameter in your HttpRequest");
        }
    }

    /** {@inheritDoc} */
    public void doPost(
            HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {

        this.doGet(
            request, response);
    }
}
