package acube.framework.web.action;

import java.lang.management.ManagementFactory;
import java.security.SecureRandom;
import java.text.MessageFormat;
import java.util.Date;

import org.apache.log4j.Logger;

import acube.framework.exception.BusinessException;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionContext;

public class ErrorReportAction implements Action {

    private static Logger logger = Logger.getLogger(ErrorReportAction.class);

    private BusinessException exception;

    private String reportId;


    private static BusinessException DEFAULT_EXCEPTION = new BusinessException(
            "acube.framework.exception");

    /**
     * Gnration d'un rapport d'id
     * 
     * @return
     */
    private static String genReportId() {

        MessageFormat mf = new MessageFormat(
                "{0}-{1,date,MMddHHmmssSSS}-{2,number,000}");

        return mf.format(new Object[]{nodeId, new Date(), gen.nextInt(1000)});
    }

    /** */
    private static final SecureRandom gen = new SecureRandom();

    /** */
    private static String nodeId;

    static {
        // l'id du noeud = nom du process java@host
        try {
            nodeId = ManagementFactory.getRuntimeMXBean().getName();
        } catch (Throwable t) {
            nodeId = "???";
        }
    }

    public String execute() {

        Throwable throwable =
                (Throwable) ActionContext.getContext().getValueStack()
                        .findValue(
                            "exception");
        String id = ErrorReportAction.genReportId();
        ErrorReportAction.logger.error(
            "Erreur technique [" + id + "]", throwable);

        if (throwable instanceof BusinessException) {
            this.setException((BusinessException) throwable);
        } else {
            this.setException(DEFAULT_EXCEPTION);
        }
        this.setReportId(id);
        ActionContext.getContext().getValueStack().push(exception);
        // ActionContext.getContext().getValueStack().set(
        // "reportId", id);
        return ERROR;
    }

    /**
     * Getter of the exception
     * 
     * @return Returns the exception.
     */
    protected BusinessException getException() {

        return this.exception;
    }

    /**
     * Setter of the exception
     * 
     * @param exception
     *            The exception to set.
     */
    protected void setException(
            BusinessException exception) {

        this.exception = exception;
    }

    /**
     * Getter of the reportId
     * 
     * @return Returns the reportId.
     */
    protected String getReportId() {

        return this.reportId;
    }

    /**
     * Setter of the reportId
     * 
     * @param reportId
     *            The reportId to set.
     */
    protected void setReportId(
            String reportId) {

        this.reportId = reportId;
    }


}
