/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, EDS (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.action;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.ServletActionContext;

import acube.framework.exception.AuthentificationException;
import acube.framework.exception.ConfigurationException;
import acube.framework.exception.DAOException;
import acube.framework.exception.FileWrapperException;
import acube.framework.exception.FunctionalException;
import acube.framework.exception.JDBCWrapperException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Constants;
import acube.framework.technical.jdbc.TransactionManager;
import acube.framework.technical.log.Chrono;
import acube.framework.technical.log.level.Performance;
import acube.framework.vo.ErrorVO;
import acube.framework.vo.FileVO;

import com.opensymphony.xwork2.Action;

/**
 * Warning : This class is not migrate to the LISE 3.0
 * 
 * Traitement des actions de download de fichier les mthodes
 * abstraites de cette classe doivent tre surcharges
 * 
 * @author Aubay
 * @version 2.4
 * @memberof General
 */

abstract public class BaseActionFile extends BaseAction {

    /**
     * les sous-classes devront surcharger cette mthode
     * 
     * @param request
     *            la requte HTTP en cours.
     * @return un objet FileVO contenant le fichier a tlcharger
     */

    abstract protected FileVO getFileActions(
            HttpServletRequest request) throws DAOException,
        AuthentificationException, FunctionalException, TechnicalException,
        JDBCWrapperException, FileWrapperException;

    /**
     * Mthode appele pour permettre le tlchargement d'un fichier
     * 
     * @param mapping
     *            ActionMapping
     * @param form
     *            ActionForm
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @return ActionForward forward
     * @throws java.io.IOException
     * @throws javax.servlet.ServletException
     * @throws JDBCWrapperException
     *             exception MAE
     * @throws AuthentificationException
     *             exception d'authentification
     * @throws DAOException
     *             exception DAO
     * @throws TechnicalException
     *             exception technique
     */
    @Override
    public String execute() throws java.io.IOException,
        javax.servlet.ServletException, DAOException, TechnicalException,
        FunctionalException {

        HttpServletRequest request;
        HttpServletResponse response;
        ArrayList<Serializable> errors = new ArrayList<Serializable>();
        FileVO file = null;

        request = getRequest();
        response = getResponse();
        try {
            this.executeAtStart(
                request, response, getActivite());
            // On retourne forcment un html en cas d'erreur sinon le
            // ContentType est cras par celui dduit du fichier 
            // tlcharger
            response.setContentType("text/html");
        } catch (TechnicalException e) {
            logger.error("TechnicalException : " + e.getMessage());
            errors.add(e.getError());
        }

        // Demarrage du Timer pour la mesure de PERF
        Chrono chrono = new Chrono();
        chrono.start();

        // vrification du droit d'execution de cette action
        try {
            this.verifierDroitAcces(
                request, ServletActionContext.getServletContext(), this
                        .getClass().getName());
        } catch (AuthentificationException e) {
            this.logger.error("AuthentificationException : " + e.getMessage());
            errors.add(e.getError());
        } catch (TechnicalException e) {
            this.logger.error("TechnicalException : " + e.getMessage());
            errors.add(e.getError());
        }

        // Gestion des Paramtres de la requte et plus
        // particulirement dans le cas d'une requte mutilpart
        AnalyseRequest(request);

        try {
            file = getFileActions(request);
        } catch (ConfigurationException e) {
            logger.error("ConfigurationException : " + e.getMessage());
            TechnicalException tech =
                    new TechnicalException(
                            new ErrorVO(
                                    "ERR-TEC-00", "erreur technique", e
                                            .getMessage(), true, false, false));
            errors.add(tech.getError());
        } catch (JDBCWrapperException e) {
            logger.error("JDBCWrapperException : " + e.getMessage());
            TechnicalException tech =
                    new TechnicalException(
                            new ErrorVO(
                                    "ERR-TEC-00", "erreur technique", e
                                            .getMessage(), true, false, false));
            errors.add(tech.getError());
        } catch (FileWrapperException e) {
            logger.error("FileWrapperException : " + e.getMessage());
            TechnicalException tech =
                    new TechnicalException(
                            new ErrorVO(
                                    "ERR-TEC-00", "erreur technique", e
                                            .getMessage(), true, false, false));
            errors.add(tech.getError());
        } catch (DAOException e) {
            logger.error("DAOException : " + e.getMessage());
            errors.add(e.getError());
        } catch (TechnicalException e) {
            logger.error("TechnicalException : " + e.getMessage());
            errors.add(e.getError());
        } catch (FunctionalException e) {
            logger.error("FunctionalException : " + e.getMessage());
            errors.add(e.getError());
        } catch (AuthentificationException e) {
            logger.error("AuthentificationException : " + e.getMessage());
            errors.add(e.getError());
        }

        // dfinit les paramtres du forward
        String forward = Action.SUCCESS;
        forward = executeAtEnd(
            forward, request, response, file, errors, getTypeOfFlux());

        // Force d'init du transaciton manager
        TransactionManager tm = TransactionManager.getInstance();
        tm.close();

        // supprime le champ param qui est une Map des requtes
        request
                .removeAttribute(acube.framework.technical.Constants.PARAMS_REQUEST);

        // --------------------------------------------------
        // Log de performance
        // --------------------------------------------------
        chrono.stop();
        if (logger.isEnabledFor(Performance.TRACE)) {
            logger.log(
                Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                        + chrono.time() + " ms");
        }
        return forward;
    }

    /**
     * Mthode appele en fin de traitement: charge la bonne liste de
     * bean au niveau de la servlet (pour la transmettre  strutscx)
     * et choisit la bonne feuille xsl
     * 
     * @param mapping
     *            ActionMapping
     * @param form
     *            ActionForm
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param forward
     *            ActionForward
     * @param beans
     *            ArrayList
     * @param erreurs
     *            ArrayList
     * @param typeFluxAttendu
     *            String le type de flux attendu par le client (XML /
     *            PDF / ...)
     * @return ActionForward the ActionForward
     */
    protected String executeAtEnd(
            String forward, HttpServletRequest request,
            HttpServletResponse response, FileVO file,
            ArrayList<Serializable> erreurs, int typeFluxAttendu) {

        if (logger.isDebugEnabled()) {
            logger.debug(THIS_CLASS + "BASE_ACTIONFILE executeAtEnd");
        }

        Date datePassee = getCalandar();

        FileVO fvo = file;
        if (fvo == null || fvo.getFluxDonnees() == null) {
            TechnicalException tech =
                    new TechnicalException(
                            new ErrorVO(
                                    "ERR-TEC-00", "erreur technique",
                                    "fichier vide", true, false, false));
            erreurs.add(tech.getError());
        } else {
            InputStream data = fvo.getFluxDonnees();
            int length = 0;

            ServletOutputStream out = null;
            try {
                out = response.getOutputStream();
                byte[] responseBody = new byte[8192];
                ByteArrayOutputStream by = new ByteArrayOutputStream();

                while ((length = data.read(
                    responseBody, 0, 8192)) != -1)
                    by.write(
                        responseBody, 0, length);

                response.setHeader(
                    "Content-Disposition", (fvo.getInline() ? "inline"
                            : "attachment")
                            + "; filename=\"" + fvo.getFichier() + "\";");
                response.setContentType(fvo.getContentType());
                out.write(by.toByteArray());
                out.flush();
                out.close();
                length = by.size();
                response.setContentLength(length);

                forward = Constants.FORWARD_FILE;

                if (logger.isDebugEnabled()) {
                    logger.debug(THIS_CLASS + "Envoi du fichier downloade");
                }
            } catch (IOException e) {
                e.printStackTrace();
                logger.error("IOException : " + e.getMessage());
                TechnicalException tech =
                        new TechnicalException(
                                new ErrorVO(
                                        "ERR-TEC-00", "erreur technique", e
                                                .getMessage(), true, false,
                                        false));
                erreurs.add(tech.getError());
            }
        }

        // gestion des erreurs
        setError(
            erreurs, request, response, datePassee, typeFluxAttendu);

        return (forward);
    }

    /**
     * On override cette mthode pour pas qu'elle ne pose de problme
     * dans les objets hritant de BaseActionFile
     */
    @Override
    protected ArrayList<Serializable> getBeansActions(
            HttpServletRequest request) throws DAOException,
        AuthentificationException, FunctionalException, TechnicalException,
        JDBCWrapperException, FileWrapperException {

        return new ArrayList<Serializable>();
    }

}
