/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.Date;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

/**
 * Projet Clavier Vituel. Implment le blocage d'un utilisateur aprs
 * un certaion nombre de tentative.
 * 
 * @version 2.6.1
 * @date 27 avr. 2006
 */
public class ControleBlocage {

    /**
     * le nombre de tentatives permises avant le blocage du login
     */
    private static final int NB_MAX_TENTATIVES = 3;

    /**
     * <code>delaiBlocageHeuresDef</code> le delai de bloquage en heure
     */
    private static final int DELAI_DEBLOQUAGE = 48;

    /**
     * <code>connection</code> la connection  la base
     */
    private Connection connection;

    /**
     * le nombre de tentatives permises avant le blocage du login
     */
    private int nombreMaxTentatives;

    /**
     * <code>delaiBlocageHeures</code> le dlai de bloquage en heure
     */
    private int delaiBlocageHeures;

    /**
     * <code>dbCompteurTentatives</code> le nombre de tentatives effectives
     */
    private int dbCompteurTentatives;

    /**
     * <code>bloque</code> si le compte est bloqu
     */
    private boolean bloque;

    /**
     * Mthode mettant  jour la variable this.bloque indiquant si un
     * utilisateur est bloqu.
     * 
     * @param connection
     *            Connection
     * @param dbDateBlocage
     *            Date
     * @param dbCompteurTentatives
     *            int
     * @param username
     *            String
     * @throws Exception
     *             en cas d'erreur.
     */
    public ControleBlocage(
            Connection connection, Date dbDateBlocage,
            int dbCompteurTentatives, String username) throws Exception {

        Context ctx = new InitialContext();
        if (connection == null) {
            DataSource ds =
                    (DataSource) ctx.lookup("java:comp/env/jdbc/ElectisDS");
            connection = ds.getConnection();
        }

        try {
            this.nombreMaxTentatives =
                    ((Integer) ctx.lookup("java:comp/env/nombreMaxTentatives"))
                            .intValue();
            this.delaiBlocageHeures =
                    ((Integer) ctx.lookup("java:comp/env/delaiBlocageHeures"))
                            .intValue();
        } catch (Exception e) {
            // valeurs non renseignes...
            this.nombreMaxTentatives = ControleBlocage.NB_MAX_TENTATIVES;
            this.delaiBlocageHeures = ControleBlocage.DELAI_DEBLOQUAGE;
        }

        PreparedStatement stmt = null;
        ResultSet rs = null;

        if (username != null && !"".equals(username)) {
            try {
                String sql =
                        "select * " + "from AUTHENTIFICATION_INTERNET "
                                + "where AI_LOGIN = ?";

                stmt = connection.prepareStatement(sql);
                stmt.setString(
                    1, username);
                rs = stmt.executeQuery();
                if (rs.next()) {
                    dbDateBlocage = rs.getTimestamp("AI_DATE_BLOCAGE");
                    dbCompteurTentatives = rs.getInt("AI_TENTATIVES");
                }
                rs.close();
                rs = null;
                stmt.close();
            } catch (Exception e) {

            } finally {
                if (rs != null) {
                    rs.close();
                }
                if (stmt != null) {
                    stmt.close();
                }
            }
        }

        this.connection = connection;
        this.dbCompteurTentatives = dbCompteurTentatives;

        if (this.nombreMaxTentatives <= 0 || this.delaiBlocageHeures <= 0) {
            this.dbCompteurTentatives = -1;
        }

        long deltaBlocage = Long.MAX_VALUE;
        // long deltaBlocage = 0;
        if (dbDateBlocage != null) {
            deltaBlocage = System.currentTimeMillis() - dbDateBlocage.getTime();
        }

        if (deltaBlocage < this.delaiBlocageHeures * 3600 * 1000) {
            // le compte est encore bloqu
            this.bloque = true;
        } else {
            this.bloque = false;
        }

    }

    /**
     * Mthode mettant  jour la variable this.bloque indiquant si un
     * utilisateur est bloqu.
     * 
     * @param connection
     *            Connection
     * @param dbDateBlocage
     *            Date
     * @param dbCompteurTentatives
     *            int
     * @throws Exception
     *             en cas d'erreur.
     */
    public ControleBlocage(
            Connection connection, Date dbDateBlocage, int dbCompteurTentatives)
        throws Exception {

        this(
                connection, dbDateBlocage, dbCompteurTentatives, null);

    }

    /**
     * 
     * @param dbDateBlocage
     *            Date
     * @param compteurTentatives
     *            int
     * @throws Exception
     *             en cas d'erreur
     */
    public ControleBlocage(
            Date dbDateBlocage, int compteurTentatives) throws Exception {

        this(
                null, dbDateBlocage, compteurTentatives);
    }

    /**
     * Methode incrementant le nombre de tentative de connexion
     * echouees. Elle marque la date de blocage quand le nombre max de
     * tentatives est atteint.
     * 
     * @param username
     *            String
     * @return true si le compte est bloqu
     * @throws SQLException
     *             en cas d'echec d'acces a la base.
     */
    public boolean gererBlocageCompte(
            String username) throws SQLException {

        if (this.dbCompteurTentatives == -1) {
            return false;
        }

        PreparedStatement stmt = null;
        try {
            if (this.dbCompteurTentatives >= this.nombreMaxTentatives - 1) {
                String sql =
                        "update AUTHENTIFICATION_INTERNET "
                                + "set AI_DATE_BLOCAGE = ? "
                                + "where AI_LOGIN = ?";
                stmt = this.connection.prepareStatement(sql);
                stmt.setTimestamp(
                    1, new Timestamp(
                            System.currentTimeMillis()));
                stmt.setString(
                    2, username);
                stmt.executeUpdate();

                this.bloque = true;
                return true;
            } else {
                if (this.dbCompteurTentatives >= 0) {
                    String sql =
                            "update AUTHENTIFICATION_INTERNET "
                                    + "set AI_TENTATIVES = ? "
                                    + "where AI_LOGIN = ?";
                    stmt = this.connection.prepareStatement(sql);
                    stmt.setInt(
                        1, this.dbCompteurTentatives + 1);
                    stmt.setString(
                        2, username);
                    stmt.executeUpdate();

                    return false;
                }
            }
        } finally {
            if (stmt != null) {
                stmt.close();
            }
        }
        return false;
    }

    /**
     * Methode permettant de reinitialiser un compte bloque
     * 
     * @param username
     *            String
     * @return true si le compte est dbloqu
     * @throws SQLException
     *             en cas d'echec d'acces a la base.
     */
    public boolean gererDeblocageCompte(
            String username) throws SQLException {

        if (this.dbCompteurTentatives <= 0) {
            return false;
        }

        PreparedStatement stmt = null;
        try {
            String sql =
                    "update AUTHENTIFICATION_INTERNET "
                            + "set AI_TENTATIVES = 0, "
                            + "AI_DATE_BLOCAGE = null " + "where AI_LOGIN = ?";
            stmt = this.connection.prepareStatement(sql);
            stmt.setString(
                1, username);
            stmt.executeUpdate();
        } finally {
            if (stmt != null) {
                stmt.close();
            }
        }
        return true;
    }

    /**
     * Getter de la variable bloque
     * 
     * @return Returns the bloque.
     */
    public boolean isBloque() {

        return this.bloque;
    }

    /**
     * Getter de la variable delaiBlocageHeures
     * 
     * @return Returns the delaiBlocageHeures.
     */
    public int getDelaiBlocageHeures() {

        return this.delaiBlocageHeures;
    }

    /**
     * Getter de la variable nombreMaxTentatives
     * 
     * @return Returns the nombreMaxTentatives.
     */
    public int getNombreMaxTentatives() {

        return this.nombreMaxTentatives;
    }

}
