/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.imageio.ImageIO;

/**
 * Projet Clavier Vituel. Classe utilise pour la gnration du
 * clavier virtuel.
 * 
 * 
 * @version 2.6.1
 * @date 07/12/2007
 */
public class GestionnaireImage {

    /**
     * Style par dfaut
     */
    public static final int STYLE_CIRCLE = 1;

    /**
     * <code>CIRCLE_WIDTH</code> width of keypad for the CIRCLE style
     */
    public static final int CIRCLE_WIDTH = 148;

    /**
     * <code>CIRCLE_HEIGHT</code> height of keypad for the CIRCLE
     * style
     */
    public static final int CIRCLE_HEIGHT = 128;

    /**
     * <code>CIRCLE_DIGIT_WIDTH</code> width of keypad's digit for
     * the CIRCLE style
     */
    public static final int CIRCLE_DIGIT_WIDTH = 28;

    /**
     * <code>CIRCLE_DIGIT_HEIGHT</code> height of keypad's digit for
     * the CIRCLE style
     */
    public static final int CIRCLE_DIGIT_HEIGHT = 29;

    /**
     * <code>STYLE_RECT</code> the STYLE_RECT
     */
    public static final int STYLE_RECT = 2;

    /**
     * <code>RECT_WIDTH</code> width of keypad for the RECT style
     */
    public static final int RECT_WIDTH = 148;

    /**
     * <code>RECT_HEIGHT</code> height of keypad for the RECT style
     */
    public static final int RECT_HEIGHT = 128;

    /**
     * <code>RECT_DIGIT_WIDTH</code> width of keypad's digit for the
     * RECT style
     */
    public static final int RECT_DIGIT_WIDTH = 28;

    /**
     * <code>RECT_DIGIT_HEIGHT</code> height of keypad's digit for the
     * RECT style
     */
    public static final int RECT_DIGIT_HEIGHT = 29;

    /**
     * <code>IMAGES_REPOSITORY</code> the image repository
     */
    private static final String IMAGES_REPOSITORY = "images/";

    /**
     * <code>LIST_ICONS</code> the list of icons available
     */
    private static final String LIST_ICONS[] =
            {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "a", "b", "c",
                    "d", "e", "f"};

    /**
     * <code>SUFFIX_BACKGROUND</code> the background image for the
     * RECT style
     */
    private static final String SUFFIX_BACKGROUND = "bg";

    /**
     * <code>EXTENTION_FILE</code> the extention of pictures files
     */
    private static final String EXTENTION_FILE = ".gif";

    /**
     * <code>CIRCLE_PREFIX</code> the prefix for the CIRCLE style
     * pictures
     */
    private static final String CIRCLE_PREFIX = "circle_";

    /**
     * <code>RECT_PREFIX</code> the prefix for the RECT style pictures
     */
    private static final String RECT_PREFIX = "rect_";

    /**
     * Create an image from the key the style and the number of icons.
     * 
     * @param key
     *            key of the keypad
     * @param style
     *            style of the keypad
     * @param nbIcons
     *            number of icons in the keypad
     * @return image
     * @throws IOException
     *             if one picture is not found
     */
    public byte[] createImage(
            String key, int style, int nbIcons) throws IOException {

        Image img;
        int width;
        int heigh;
        int toucheW = 0;
        int toucheH = 0;
        int nbColumns = GestionnaireImage.getNumberColumnsKeyPad(nbIcons);

        // We get the prefix for the style
        String prefix = GestionnaireImage.getPrefixFromStyle(style);

        List<String> list = GestionnaireRefCles.getListIcons(key);

        BufferedImage images[] = new BufferedImage[list.size() + 1];

        // We initialize parameters of image.
        switch (style) {
            case GestionnaireImage.STYLE_CIRCLE:
                width = GestionnaireImage.CIRCLE_WIDTH;
                heigh = GestionnaireImage.CIRCLE_HEIGHT;
                toucheW = GestionnaireImage.CIRCLE_DIGIT_WIDTH;
                toucheH = GestionnaireImage.CIRCLE_DIGIT_HEIGHT;
                break;
            default:
                width = GestionnaireImage.RECT_WIDTH;
                heigh = GestionnaireImage.RECT_HEIGHT;
                toucheW = GestionnaireImage.RECT_DIGIT_WIDTH;
                toucheH = GestionnaireImage.RECT_DIGIT_HEIGHT;
                break;
        }

        // We initialize the background
        StringBuffer urlBackground = new StringBuffer(
                prefix);
        urlBackground.append(
            GestionnaireImage.SUFFIX_BACKGROUND).append(
            GestionnaireImage.EXTENTION_FILE);
        images[0] =
                ImageIO.read((new GestionnaireImage()).getClass()
                        .getResourceAsStream(
                            urlBackground.toString()));
        // We load all the pictures in the table
        int j = 1;
        for (String icons : list) {

            // If it's not an empty place whe load the
            // picture
            if (!"".equals(icons)) {

                String urlImage = new StringBuffer(
                        prefix).append(
                    icons).append(
                    GestionnaireImage.EXTENTION_FILE).toString();

                BufferedImage image =
                        ImageIO.read((new GestionnaireImage()).getClass()
                                .getResourceAsStream(
                                    urlImage));
                images[j] = image;
            }
            j++;
        }

        // Cration de l'image de sortie
        BufferedImage total = new BufferedImage(
                width, heigh, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graph = total.createGraphics();

        // Insertion de l'image de fond
        graph.drawImage(
            images[0], 0, 0, null);

        int margeX = GestionnaireImage.getMarge(
            width, nbColumns, toucheW);
        int margeY = GestionnaireImage.getMarge(
            heigh, nbColumns, toucheH);

        // Insertion des images
        for (int i = 0; i < list.size(); i++) {
            // Parcours des touches et calcul de leur emplacement
            int celX = i % nbColumns;
            int celY = i / nbColumns;
            int posX = (margeX + toucheW) * celX + margeX;
            int posY = (margeY + toucheH) * celY + margeY;

            if (images[i + 1] != null) {
                // Redimentionner l'image images[i].getGraphics()
                img = images[i + 1].getScaledInstance(
                    toucheW, toucheH, Image.SCALE_DEFAULT);

                // Insrer l'image dans le graphique
                graph.drawImage(
                    img, posX, posY, null);
            }
        }

        // Cration du fichier PNG
        // this.appliqueTransparence(total);
        PngEncoder encodeur = new PngEncoder(
                total);
        encodeur.setEncodeAlpha(true);
        return encodeur.pngEncode();
    }

    /**
     * Return the prefix of picture for the style in parameters
     * 
     * @param style
     *            the style of the keypad
     * @return the prefix of pictures
     */
    public static String getPrefixFromStyle(
            int style) {

        StringBuffer prefix = new StringBuffer(
                GestionnaireImage.IMAGES_REPOSITORY);

        switch (style) {
            case GestionnaireImage.STYLE_CIRCLE:
                prefix.append(GestionnaireImage.CIRCLE_PREFIX);
                break;

            default:
                prefix.append(GestionnaireImage.RECT_PREFIX);
                break;
        }

        return prefix.toString();
    }

    /**
     * Return the tab contains the icon that will be use in the
     * virtual keypad. This tab is intialised with the number of icons
     * to display in the keypad. The order of the icons is random.
     * 
     * @param nbIcons
     *            the number of digit to display in the keypad
     * @param width
     *            width of the keypad
     * @return List of icons to use
     */
    public static List<String> getListIcons(
            int nbIcons, int width) {

        // We find the number of place in the keypad
        int nbPlace = width * width;

        // We fill the list with icons
        List<String> list = new ArrayList<String>();
        for (int i = 0; i < nbIcons; i++) {

            list.add(GestionnaireImage.LIST_ICONS[i]);
        }

        // We complete the list with empty places
        for (int i = list.size(); i < nbPlace; i++) {

            list.add("");
        }

        // We do the random permutation of the list
        Collections.shuffle(
            list, new SecureRandom());

        return list;
    }

    /**
     * Calcul of the number of columns of the keypad
     * 
     * @param nbIcons
     *            the number of icons to display
     * @return the number of columns of the keypad
     */
    public static int getNumberColumnsKeyPad(
            int nbIcons) {

        // We find the width of the composant
        double floor = Math.sqrt(nbIcons);

        int width = (int) Math.ceil(floor);

        return width;
    }

    /**
     * Return the marge of the picture. It return the width of the
     * empty space between 2 digit.
     * 
     * @param width
     *            width of the picture
     * @param nbColumns
     *            number of column in the picture
     * @param digitWidth
     *            width of the digit
     * @return the marge
     */
    public static int getMarge(
            int width, int nbColumns, int digitWidth) {

        int marge = (width - nbColumns * digitWidth) / (nbColumns + 1);

        return marge;
    }

    /**
     * Return a list that contains all position of empty places.
     * 
     * @param listIcons
     *            list of icons to display
     * @param nbIcons
     *            number of icons to display
     * @return positions of empty places
     */
    public static int[] genererMapTouchesLibres(
            List<String> listIcons, int nbIcons) {

        int mappageLibre[] = new int[listIcons.size() - nbIcons];

        int nbEmptyPlace = 0;
        for (int i = 0; i < listIcons.size(); i++) {

            if (listIcons.get(
                i).equals(
                "")) {
                mappageLibre[nbEmptyPlace] = i;
                nbEmptyPlace++;
            }
        }

        return mappageLibre;
    }
}
