/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel;

import java.security.SecureRandom;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Implementation de la gestion des rfrences client pour le clavier
 * virtuel
 * 
 * Les accs aux deux HashMap seront synchronises sur l'objet tabRef
 * pour permettre de garder une cohrence entre les deux HashMap
 * 
 * @author Florent RAYMONDAUD
 * @version 2.6.1
 * @date 14/11/06
 */
public class GestionnaireRefCles {

    /**
     * Length of the key
     */
    private static final int LG_CLE = 15;

    /**
     * Table that contains all key and the list of icons assoociate
     */
    private static Map<String, List<String>> tabKey =
            new HashMap<String, List<String>>();

    /**
     * Random number generator
     */
    private static SecureRandom generator = new SecureRandom();

    /**
     * Private constructor
     */
    private GestionnaireRefCles() {

    }

    /**
     * Thie method will generate a new key and will put the key and
     * lthe list associate to the global list.
     * 
     * @param listIcons
     *            the list of icons that will be use
     * @return String a new key
     */
    public static String generateKey(
            List<String> listIcons) {

        String key =
                GestionnaireRefCles
                        .generateRandomNumber(GestionnaireRefCles.LG_CLE);

        // We put the key and the List in the table
        GestionnaireRefCles.tabKey.put(
            key, listIcons);

        return key;
    }

    /**
     * Return the tab contains the icon that will be use in the
     * virtual keypad. This tab is find by the key in parameters.
     * 
     * @param key
     *            key to find the list of icons
     * @return list of icons
     */
    public static List<String> getListIcons(
            String key) {

        return GestionnaireRefCles.tabKey.get(key);
    }

    /**
     * This method will generate a random number with a define length.
     * 
     * @param length
     *            length of the number to generate
     * @return A random number
     */
    private static String generateRandomNumber(
            int length) {

        int random;
        String key = "";
        while (key.length() < length) {
            random = GestionnaireRefCles.generator.nextInt();
            if (random < 0) {
                random = random * -1;
            }
            if (Integer.toString(
                random).length() > length - key.length()) {
                key += Integer.toString(
                    random).substring(
                    0, length - key.length());
            } else {
                key += Integer.toString(random);
            }
        }

        return key;
    }

    /**
     * Return the password of user from the credential. Return null if
     * the key is not valid.
     * 
     * @param credential
     *            the encrypt password
     * @return the password of the user
     */
    public static String getPassword(
            String credential) {

        // Decrypt password
        String password = null;

        if (credential != null) {

            String key = credential.substring(
                0, GestionnaireRefCles.LG_CLE);

            // First we get the list of icons
            List<String> list = GestionnaireRefCles.getListIcons(key);

            // If the list is empty that because the key is not
            // valid
            if (list != null) {

                // Test if the password is no empty
                if (credential.length() > GestionnaireRefCles.LG_CLE) {
                    String pass =
                            credential
                                    .substring(GestionnaireRefCles.LG_CLE + 1);

                    String split[] = pass.split("\\|");

                    StringBuffer buffer = new StringBuffer();

                    for (int i = 0; i < split.length; i++) {

                        int position = Integer.parseInt(split[i]);

                        buffer.append(list.get(position));
                    }

                    password = buffer.toString();
                } else {

                    // We return an empty password
                    password = new String();
                }

                // We remove the key of the table so the key will
                // be
                // lost
                GestionnaireRefCles.removeKey(key);

            }
        }

        return password;
    }

    /**
     * This method will remove a key of the table
     * 
     * @param key
     *            the key to remove
     */
    private static void removeKey(
            String key) {

        GestionnaireRefCles.tabKey.remove(key);
    }
}
