/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.security.Principal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.naming.Context;
import javax.sql.DataSource;

import org.apache.catalina.LifecycleException;
import org.apache.catalina.ServerFactory;
import org.apache.catalina.core.StandardServer;
import org.apache.catalina.realm.Constants;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.util.StringManager;
import org.apache.naming.ContextBindings;

import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * 
 * Implmentation of <b>Realm</b> that works with any JDBC JNDI
 * DataSource. See the JDBCRealm.howto for more details on how to set
 * up the database and for configuration options.
 * 
 * @author Glenn L. Nielsen
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
 * @version $Revision: 1.1 $
 */

public class DataSourceRealm extends RealmBaseAcube {

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * Descriptive information about this Realm implementation.
     */
    private static final String NAME = "DataSourceRealm";

    /**
     * The string manager for this package.
     */
    private static final StringManager SM =
            StringManager.getManager(Constants.Package);

    /**
     * The NAME of the JNDI JDBC DataSource
     */
    private String dataSourceName = null;

    /**
     * Context local datasource.
     */
    private boolean localDataSource = false;

    /**
     * The column in the user role table that names a role
     */
    private String roleNameCol = null;

    /**
     * The column in the user table that holds the user's credintials
     */
    private String userCredCol = null;

    /**
     * The column in the user table that holds the user's NAME
     */
    private String userNameCol = null;

    /**
     * The table that holds the relation between user's and roles
     */
    private String userRoleTable = null;

    /**
     * The table that holds user data.
     */
    private String userTable = null;

    /**
     * The generated string for the roles PreparedStatement
     */
    private String preparedRoles = null;

    /**
     * The generated string for the credentials PreparedStatement
     */
    private String preparedCredentials = null;

    // -------------------------------------------------------------
    // Properties

    /**
     * Return the NAME of the JNDI JDBC DataSource.
     * 
     * @return the NAME of the JNDI JDBC DataSource.
     */
    public String getDataSourceName() {

        return this.dataSourceName;
    }

    /**
     * Set the NAME of the JNDI JDBC DataSource.
     * 
     * @param dataSourceName
     *            the NAME of the JNDI JDBC DataSource
     */
    public void setDataSourceName(
            String dataSourceName) {

        this.dataSourceName = dataSourceName;
    }

    /**
     * Return if the datasource will be looked up in the webapp JNDI
     * Context.
     * 
     * @return if the datasource will be looked up in the webapp JNDI
     *         Context.
     */
    public boolean getLocalDataSource() {

        return this.localDataSource;
    }

    /**
     * Set to true to cause the datasource to be looked up in the
     * webapp JNDI Context.
     * 
     * @param localDataSource
     *            the new flag value
     */
    public void setLocalDataSource(
            boolean localDataSource) {

        this.localDataSource = localDataSource;
    }

    /**
     * Return the column in the user role table that names a role.
     * 
     * @return the column in the user role table that names a role.
     */
    public String getRoleNameCol() {

        return this.roleNameCol;
    }

    /**
     * Set the column in the user role table that names a role.
     * 
     * @param roleNameCol
     *            The column NAME
     */
    public void setRoleNameCol(
            String roleNameCol) {

        this.roleNameCol = roleNameCol;
    }

    /**
     * Return the column in the user table that holds the user's
     * credentials.
     * 
     * @return the column in the user table that holds the user's
     * 
     */
    public String getUserCredCol() {

        return this.userCredCol;
    }

    /**
     * Set the column in the user table that holds the user's
     * credentials.
     * 
     * @param userCredCol
     *            The column NAME
     */
    public void setUserCredCol(
            String userCredCol) {

        this.userCredCol = userCredCol;
    }

    /**
     * Return the column in the user table that holds the user's NAME.
     * 
     * @return the column in the user table that holds the user's
     *         NAME.
     */
    public String getUserNameCol() {

        return this.userNameCol;
    }

    /**
     * Set the column in the user table that holds the user's NAME.
     * 
     * @param userNameCol
     *            The column NAME
     */
    public void setUserNameCol(
            String userNameCol) {

        this.userNameCol = userNameCol;
    }

    /**
     * Return the table that holds the relation between user's and
     * roles.
     * 
     * @return the table that holds the relation between user's and
     *         roles.
     */
    public String getUserRoleTable() {

        return this.userRoleTable;
    }

    /**
     * Set the table that holds the relation between user's and roles.
     * 
     * @param userRoleTable
     *            The table NAME
     */
    public void setUserRoleTable(
            String userRoleTable) {

        this.userRoleTable = userRoleTable;
    }

    /**
     * Return the table that holds user data..
     * 
     * @return the table that holds user data.
     */
    public String getUserTable() {

        return this.userTable;
    }

    /**
     * Set the table that holds user data.
     * 
     * @param userTable
     *            The table NAME
     */
    public void setUserTable(
            String userTable) {

        this.userTable = userTable;
    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /** {@inheritDoc} */
    public Principal authenticate(
            String username, String credentials) {

        // Prise en compte du mot de passe saisie par le clavier
        // virtuel
        String password = GestionnaireRefCles.getPassword(credentials);

        // No user - can't possibly authenticate, don't bother the
        // database
        // then
        if (username == null) {
            return null;
        }

        Connection dbConnection = null;

        try {

            // Ensure that we have an open database connection
            dbConnection = this.open();
            if (dbConnection == null) {
                // If the db connection open fails, return "not
                // authenticated"
                return null;
            }

            // Acquire a Principal object for this user
            Principal principal = this.authenticate(
                dbConnection, username, password);

            return principal;

        } catch (SQLException e) {
            // Log the problem for posterity
            containerLog.error(
                DataSourceRealm.SM.getString("dataSourceRealm.exception"), e);

            // Return "not authenticated" for this request
            return (null);

        } finally {
            this.close(dbConnection);
        }

    }

    // --------------------------------------------------------
    // Package
    // Methods

    // ------------------------------------------------------
    // Protected
    // Methods

    /**
     * Return the Principal associated with the specified username and
     * credentials, if there is one; otherwise return
     * <code>null</code>.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username of the Principal to look up
     * @param credentials
     *            Password or other credentials to use in
     *            authenticating this username
     * @return the Principal associated with the specified username
     *         and credentials
     * @throws SQLException
     *             if it had connection errors
     */
    protected Principal authenticate(
            Connection dbConnection, String username, String credentials)
        throws SQLException {

        String dbCredentials = this.getPassword(
            dbConnection, username);

        // Validate the user's credentials
        boolean validated = false;
        if (this.hasMessageDigest()) {
            // Hex hashes should be compared case-insensitive
            validated =
                    (this.digest(credentials).equalsIgnoreCase(dbCredentials));
        } else {
            validated = (this.digest(credentials).equals(dbCredentials));
        }

        if (validated) {
            if (containerLog.isTraceEnabled()) {
                containerLog.trace(DataSourceRealm.SM.getString(
                    "dataSourceRealm.authenticateSuccess", username));
            }
        } else {
            if (containerLog.isTraceEnabled()) {
                containerLog.trace(DataSourceRealm.SM.getString(
                    "dataSourceRealm.authenticateFailure", username));
            }
            return (null);
        }

        ArrayList<String> list = this.getRoles(
            dbConnection, username);

        // Create and return a suitable Principal for this user
        return (new GenericPrincipal(
                this, username, credentials, list));

    }

    /**
     * Close the specified database connection.
     * 
     * @param dbConnection
     *            The connection to be closed
     */
    protected void close(
            Connection dbConnection) {

        // Do nothing if the database connection is already closed
        if (dbConnection == null) {
            return;
        }

        // Commit if not auto committed
        try {
            if (!dbConnection.getAutoCommit()) {
                dbConnection.commit();
            }
        } catch (SQLException e) {
            containerLog.error(
                "Exception committing connection before closing:", e);
        }

        // Close this database connection, and log any errors
        try {
            dbConnection.close();
        } catch (SQLException e) {
            // Just log it here
            containerLog.error(
                DataSourceRealm.SM.getString("dataSourceRealm.close"), e);
        }

    }

    /**
     * Open the specified database connection.
     * 
     * @return Connection to the database
     */
    protected Connection open() {

        try {
            Context context = null;
            if (this.localDataSource) {
                context = ContextBindings.getClassLoader();
                context = (Context) context.lookup("comp/env");
            } else {
                StandardServer server =
                        (StandardServer) ServerFactory.getServer();
                context = server.getGlobalNamingContext();
            }
            DataSource dataSource =
                    (DataSource) context.lookup(this.dataSourceName);
            return dataSource.getConnection();
        } catch (Exception e) {
            // Log the problem for posterity
            containerLog.error(
                DataSourceRealm.SM.getString("dataSourceRealm.exception"), e);
        }
        return null;
    }

    /** {@inheritDoc} */
    protected String getName() {

        return (DataSourceRealm.NAME);

    }

    /** {@inheritDoc} */
    protected String getPassword(
            String username) {

        Connection dbConnection = null;

        // Ensure that we have an open database connection
        dbConnection = this.open();
        if (dbConnection == null) {
            return null;
        }

        try {
            return this.getPassword(
                dbConnection, username);
        } finally {
            this.close(dbConnection);
        }
    }

    /**
     * Return the password associated with the given principal's user
     * NAME.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which password should be retrieved
     * @return the password associated with the given principal's user
     *         NAME.
     */
    protected String getPassword(
            Connection dbConnection, String username) {

        ResultSet rs = null;
        PreparedStatement stmt = null;
        String dbCredentials = null;

        try {
            stmt = this.credentials(
                dbConnection, username);
            rs = stmt.executeQuery();
            if (rs.next()) {
                dbCredentials = rs.getString(1);
            }

            if (dbCredentials != null) {
                return dbCredentials.trim();
            } else {
                return null;
            }

        } catch (SQLException e) {
            containerLog.error(DataSourceRealm.SM.getString(
                "dataSourceRealm.getPassword.exception", username));
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (stmt != null) {
                    stmt.close();
                }
            } catch (SQLException e) {
                containerLog.error(DataSourceRealm.SM.getString(
                    "dataSourceRealm.getPassword.exception", username));

            }
        }

        return null;
    }

    /** {@inheritDoc} */
    protected Principal getPrincipal(
            String username) {

        Connection dbConnection = this.open();
        if (dbConnection == null) {
            return new GenericPrincipal(
                    this, username, null, null);
        }
        try {
            return (new GenericPrincipal(
                    this, username, this.getPassword(
                        dbConnection, username), this.getRoles(
                        dbConnection, username)));
        } finally {
            this.close(dbConnection);
        }

    }

    /**
     * Return the roles associated with the given user NAME.
     * 
     * @param username
     *            Username for which roles should be retrieved
     * @return the roles associated with the given user NAME.
     */
    protected ArrayList<String> getRoles(
            String username) {

        Connection dbConnection = null;

        // Ensure that we have an open database connection
        dbConnection = this.open();
        if (dbConnection == null) {
            return null;
        }

        try {
            return this.getRoles(
                dbConnection, username);
        } finally {
            this.close(dbConnection);
        }
    }

    /**
     * Return the roles associated with the given user NAME
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which roles should be retrieved
     * @return the roles associated with the given user NAME
     */
    protected ArrayList<String> getRoles(
            Connection dbConnection, String username) {

        ResultSet rs = null;
        PreparedStatement stmt = null;
        ArrayList<String> list = null;

        try {
            stmt = this.roles(
                dbConnection, username);
            rs = stmt.executeQuery();
            list = new ArrayList<String>();

            while (rs.next()) {
                String role = rs.getString(1);
                if (role != null) {
                    list.add(role.trim());
                }
            }
            return list;
        } catch (SQLException e) {
            containerLog.error(DataSourceRealm.SM.getString(
                "dataSourceRealm.getRoles.exception", username));
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (stmt != null) {
                    stmt.close();
                }
            } catch (SQLException e) {
                containerLog.error(DataSourceRealm.SM.getString(
                    "dataSourceRealm.getRoles.exception", username));
            }
        }

        return null;
    }

    /**
     * Return a PreparedStatement configured to perform the SELECT
     * required to retrieve user credentials for the specified
     * username.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which credentials should be retrieved
     * 
     * @return a PreparedStatement configured to perform the SELECT
     *         required to retrieve user credentials for the specified
     *         username.
     * @exception SQLException
     *                if a database error occurs
     */
    private PreparedStatement credentials(
            Connection dbConnection, String username) throws SQLException {

        PreparedStatement credentials =
                dbConnection.prepareStatement(this.preparedCredentials);

        credentials.setString(
            1, username);
        return (credentials);

    }

    /**
     * Return a PreparedStatement configured to perform the SELECT
     * required to retrieve user roles for the specified username.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which roles should be retrieved
     * @return a PreparedStatement configured to perform the SELECT
     *         required to retrieve user roles for the specified
     *         username.
     * @exception SQLException
     *                if a database error occurs
     */
    private PreparedStatement roles(
            Connection dbConnection, String username) throws SQLException {

        PreparedStatement roles =
                dbConnection.prepareStatement(this.preparedRoles);

        roles.setString(
            1, username);
        return (roles);

    }

    // ------------------------------------------------------
    // Lifecycle
    // Methods

    /**
     * 
     * Prepare for active use of the public methods of this Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                prevents it from being started
     */
    public void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

        // Create the roles PreparedStatement string
        StringBuffer temp = new StringBuffer(
                "SELECT ");
        temp.append(this.roleNameCol);
        temp.append(" FROM ");
        temp.append(this.userRoleTable);
        temp.append(" WHERE ");
        temp.append(this.userNameCol);
        temp.append(" = ?");
        this.preparedRoles = temp.toString();

        // Create the credentials PreparedStatement string
        temp = new StringBuffer(
                "SELECT ");
        temp.append(this.userCredCol);
        temp.append(" FROM ");
        temp.append(this.userTable);
        temp.append(" WHERE ");
        temp.append(this.userNameCol);
        temp.append(" = ?");
        this.preparedCredentials = temp.toString();
    }

    /**
     * Gracefully shut down active use of the public methods of this
     * Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                needs to be reported
     */
    public void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

    }

}
