/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.security.Principal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;

import javax.naming.Context;
import javax.sql.DataSource;

import org.apache.catalina.LifecycleException;
import org.apache.catalina.ServerFactory;
import org.apache.catalina.core.StandardServer;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.naming.ContextBindings;

import acube.framework.clavierVirtuel.ControleBlocage;
import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * Classe d'authentification J2EE Realm.<br/>
 * <p>
 * Permet d'authentifier un utilisateur pour l'application Electis
 * avec politique de blocage.<br/> La politique de blocage consiste 
 * refuser l'accs  tout utilisateur dont le compte est bloqu. Un
 * compte est bloqu si un ou plusieurs utilisateurs dpasse un nombre
 * limite de tentatives de connexion  ce compte. Le compte reste
 * bloqu pour un certain temps durant lequel aucun test
 * d'authentification n'est ralis.<br/> Le dblocage s'effectue si
 * le temps d'attente est dpass et qu'un utilisateur est
 * authentifi. Son compteur de tentatives de connexions est alors
 * remis  zro.
 * </p>
 * 
 * Prrequis : Utilise une datasource dfinie en tant que resource
 * serveur et accessible avec un nom JNDI.
 * 
 * Projet ELECTIS.
 * 
 * @date 26 avr. 2006
 * @author eds4
 * 
 */
public class ElectisRealm extends RealmBaseAcube {

    /**
     * Descriptive information about this Realm implementation.
     */
    protected static final String INFO =
            "mae.projet.tomcat.realm.ElectisRealm/1.0";

    /**
     * Descriptive information about this Realm implementation.
     */
    private static final String NAME = "ElectisRealm";

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * Le nom JNDI de la datasource utilise.
     */
    private String dataSourceName = null;

    /**
     * Context local datasource.
     */
    private boolean localDataSource = false;

    // -------------------------------------------------------------
    // Properties

    /**
     * @return Returns the dataSourceName.
     */
    public String getDataSourceName() {

        return this.dataSourceName;
    }

    /**
     * @param argDataSourceName
     *            The dataSourceName to set.
     */
    public void setDataSourceName(
            String argDataSourceName) {

        this.dataSourceName = argDataSourceName;
    }

    /**
     * @return Returns the localDataSource.
     */
    public boolean isLocalDataSource() {

        return this.localDataSource;
    }

    /**
     * @param argLocalDataSource
     *            The localDataSource to set.
     */
    public void setLocalDataSource(
            boolean argLocalDataSource) {

        this.localDataSource = argLocalDataSource;
    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /**
     * Return the Principal associated with the specified username and
     * credentials, if there is one; otherwise return
     * <code>null</code>.
     * 
     * If there are any errors with the JDBC connection, executing the
     * query or anything we return null (don't authenticate). This
     * event is also logged, and the connection will be closed so that
     * a subsequent request will automatically re-open it.
     * 
     * @param username
     *            Username of the Principal to look up
     * @param credentials
     *            Password or other credentials to use in
     *            authenticating this username
     * @return Principal
     */
    public Principal authenticate(
            String username, String credentials) {

        // Prise en compte du mot de passe saisie par le clavier
        // virtuel
        String password = GestionnaireRefCles.getPassword(credentials);

        Connection dbConnection = null;

        try {

            // Ensure that we have an open database connection
            dbConnection = this.open();
            if (dbConnection == null) {
                // If the db connection open fails, return "not
                // authenticated"
                return null;
            }

            // Acquire a Principal object for this user
            Principal principal = this.authenticate(
                dbConnection, username, password);

            if (!dbConnection.getAutoCommit()) {
                dbConnection.commit();
            }

            // Release the database connection we just used
            this.close(dbConnection);
            dbConnection = null;

            // Return the Principal (if any)
            return (principal);

        } catch (Exception e) {

            // Log the problem for posterity
            container.getLogger().error(
                sm.getString("dataSourceRealm.exception"), e);

            // Close the connection so that it gets reopened next time
            if (dbConnection != null) {
                this.close(dbConnection);
            }

            // Return "not authenticated" for this request
            return (null);

        }

    }

    // --------------------------------------------------------
    // Package
    // Methods

    // ------------------------------------------------------
    // Protected
    // Methods

    /**
     * Return the Principal associated with the specified username and
     * credentials, if there is one; otherwise return
     * <code>null</code>.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username of the Principal to look up
     * @param credentials
     *            Password or other credentials to use in
     *            authenticating this username
     * @return Principal
     * 
     * @exception Exception
     *                if a database error occurs
     */
    protected Principal authenticate(
            Connection dbConnection, String username, String credentials)
        throws Exception {

        ResultSet rs = null;
        PreparedStatement stmt = null;
        ArrayList<String> list = null;

        try {
            String sql =
                    "select * " + "from AUTHENTIFICATION_INTERNET "
                            + "where AI_LOGIN = ?";

            // valeur par dfaut au cas o le compte n'existe pas
            // ou qu'on a pas dfini de politique de blocage
            String dbCredentials = null;
            String dbRole = null;
            Date dbDateBlocage = null;
            int dbNbTentatives = -1;

            stmt = dbConnection.prepareStatement(sql);
            stmt.setString(
                1, username);
            rs = stmt.executeQuery();
            if (rs.next()) {
                // si la politique de blocage est bien dfinie, on
                // rcupre l'tat de blocage du compte
                dbDateBlocage = rs.getTimestamp("AI_DATE_BLOCAGE");
                dbNbTentatives = rs.getInt("AI_TENTATIVES");
                dbCredentials = rs.getString("AI_MOT_DE_PASSE");
                dbRole = rs.getString("AI_ROLE");
            }
            rs.close();
            rs = null;
            stmt.close();
            stmt = null;

            ControleBlocage controleBlocage = new ControleBlocage(
                    dbConnection, dbDateBlocage, dbNbTentatives);

            /*
             * if (controleBlocage.isBloque()) { //le compte est
             * encore bloqu container.getLogger().info( "Le compte de
             * login " + username + " est bloqu" ); return null; }
             */

            if (dbCredentials == null) {
                // le mot de passe n'est jamais nul (contrainte bdd)
                // donc le compte n'existe pas, donc on sort sans
                // grer le blocage...
                return null;
            }
            dbCredentials = dbCredentials.trim();

            // Validate the user's credentials
            boolean validated = false;
            if (this.hasMessageDigest()) {
                // Hex hashes should be compared case-insensitive
                validated =
                        (this.digest(credentials)
                                .equalsIgnoreCase(dbCredentials));
            } else {
                validated = (this.digest(credentials).equals(dbCredentials));
            }

            if (validated) {
                // login/mdp ok et compte bloque ou pas (vu que le if
                // deja
                // fait sur cette condition)
                if (container.getLogger().isTraceEnabled()) {
                    container.getLogger().trace(
                        sm.getString(
                            "dataSourceRealm.authenticateSuccess", username));
                }

                // si compte pas bloque, on dbloque le compte
                if (!controleBlocage.isBloque()) {
                    if (controleBlocage.gererDeblocageCompte(username)) {
                        container.getLogger().info(
                            "dblocage du compte de login " + username);
                    }
                }

                // Accumulate the user's roles
                list = new ArrayList<String>();
                list.add(dbRole);
            } else {
                // login/mdp nok
                if (container.getLogger().isDebugEnabled()) {
                    container.getLogger().trace(
                        sm.getString(
                            "dataSourceRealm.authenticateFailure", username));
                }
                if (!controleBlocage.isBloque()) {
                    // on met  jour le champ de blocage ou
                    // on incrmente le compteur
                    // uniquement si aucun blocage n'existe dj
                    // afin d'viter de reconduire le dlai d'attente
                    boolean res = controleBlocage.gererBlocageCompte(username);

                    if (res) {
                        container.getLogger().info(
                            "blocage du compte de login " + username);
                    } else {
                        container.getLogger().info(
                            "Le login "
                                    + username
                                    + " possde encore "
                                    + (controleBlocage.getNombreMaxTentatives()
                                            - dbNbTentatives - 1)
                                    + " tentatives de connexion");
                    }
                    return null; // retour page de connexion avec
                    // msg
                    // type 'login/mdp pas bon'
                } else {
                    // login/mdp nok et compte bloqu => il y aura
                    // contrle
                    // dans le baseAction
                    // pour gnrer msg type 'compte bloqu'
                    list = new ArrayList<String>();
                    list.add("Utilisateur");
                    return new GenericPrincipal(
                            this, username, credentials, list);
                }

            }

        } finally {
            if (rs != null) {
                rs.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }

        // Create and return a suitable Principal for this user
        return new GenericPrincipal(
                this, username, credentials, list);
    }

    /**
     * Close the specified database connection.
     * 
     * @param dbConnection
     *            The connection to be closed
     */
    private void close(
            Connection dbConnection) {

        // Do nothing if the database connection is already closed
        if (dbConnection == null) {
            return;
        }

        // Close this database connection, and log any errors
        try {
            dbConnection.close();
        } catch (SQLException e) {
            // Just log it here
            container.getLogger().error(
                sm.getString("dataSourceRealm.close"), e);
        }

    }

    /**
     * Open the specified database connection.
     * 
     * @return Connection to the database
     */
    private Connection open() {

        try {
            Context context = null;
            if (this.localDataSource) {
                context = ContextBindings.getClassLoader();
                context = (Context) context.lookup("comp/env");
            } else {
                StandardServer server =
                        (StandardServer) ServerFactory.getServer();
                context = server.getGlobalNamingContext();
            }
            DataSource dataSource =
                    (DataSource) context.lookup(this.dataSourceName);
            return dataSource.getConnection();
        } catch (Exception e) {
            // Log the problem for posterity
            container.getLogger().error(
                sm.getString("dataSourceRealm.exception"), e);
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    protected String getName() {

        return ElectisRealm.NAME;
    }

    /** {@inheritDoc} */
    public String getInfo() {

        return ElectisRealm.INFO;
    }

    /**
     * {@inheritDoc}
     */
    protected String getPassword(
            String username) {

        return (null);
    }

    /**
     * {@inheritDoc}
     */
    protected Principal getPrincipal(
            String username) {

        return (null);
    }

    // ------------------------------------------------------
    // Lifecycle
    // Methods

    /**
     * 
     * Prepare for active use of the public methods of this Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                prevents it from being started
     */
    public void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

    }

    /**
     * Gracefully shut down active use of the public methods of this
     * Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                needs to be reported
     */
    public void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

    }
}
