/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.io.IOException;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

import org.apache.catalina.realm.Constants;
import org.apache.catalina.util.StringManager;

/**
 * <p>
 * Implementation of the JAAS <code>CallbackHandler</code>
 * interface, used to negotiate delivery of the username and
 * credentials that were specified to our constructor. No interaction
 * with the user is required (or possible).
 * </p>
 * 
 * <p>
 * This <code>CallbackHandler</code> will pre-digest the supplied
 * password, if required by the <code>&lt;Realm&gt;</code> element
 * in <code>server.xml</code>.
 * </p>
 * <p>
 * At present, <code>JAASCallbackHandler</code> knows how to handle
 * callbacks of type
 * <code>javax.security.auth.callback.NameCallback</code> and
 * <code>javax.security.auth.callback.PasswordCallback</code>.
 * </p>
 * 
 * @author Craig R. McClanahan
 * @author Andrew R. Jaquith
 * @version $Revision: 1.1 $ $Date: 2007-03-26 08:14:19 $
 */

public class JAASCallbackHandler implements CallbackHandler {

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * The string manager for this package.
     */
    private static final StringManager SM =
            StringManager.getManager(Constants.Package);

    /**
     * The password to be authenticated with.
     */
    private String password = null;

    /**
     * The associated <code>JAASRealm</code> instance.
     */
    private JAASRealm realm = null;

    /**
     * The username to be authenticated with.
     */
    private String username = null;

    /**
     * Construct a callback handler configured with the specified
     * values. Note that if the <code>JAASRealm</code> instance
     * specifies digested passwords, the <code>password</code>
     * parameter will be pre-digested here.
     * 
     * @param realm
     *            Our associated JAASRealm instance
     * @param username
     *            Username to be authenticated with
     * @param password
     *            Password to be authenticated with
     */
    public JAASCallbackHandler(
            JAASRealm realm, String username, String password) {

        super();
        this.realm = realm;
        this.username = username;

        if (realm.hasMessageDigest()) {
            this.password = realm.digest(password);
        } else {
            this.password = password;
        }
    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /**
     * Retrieve the information requested in the provided
     * <code>Callbacks</code>. This implementation only recognizes
     * <code>NameCallback</code> and <code>PasswordCallback</code>
     * instances.
     * 
     * @param callbacks
     *            The set of <code>Callback</code>s to be processed
     * 
     * @exception IOException
     *                if an input/output error occurs
     * @exception UnsupportedCallbackException
     *                if the login method requests an unsupported
     *                callback type
     */
    public void handle(
            Callback callbacks[]) throws IOException,
        UnsupportedCallbackException {

        for (int i = 0; i < callbacks.length; i++) {

            if (callbacks[i] instanceof NameCallback) {
                if (this.realm.getContainer().getLogger().isTraceEnabled()) {
                    this.realm.getContainer().getLogger().trace(
                        JAASCallbackHandler.SM.getString(
                            "jaasCallback.username", this.username));
                }
                ((NameCallback) callbacks[i]).setName(this.username);
            } else if (callbacks[i] instanceof PasswordCallback) {
                final char passwordcontents[];
                if (this.password != null) {
                    passwordcontents = this.password.toCharArray();
                } else {
                    passwordcontents = new char[0];
                }
                ((PasswordCallback) callbacks[i]).setPassword(passwordcontents);
            } else {
                throw new UnsupportedCallbackException(
                        callbacks[i]);
            }
        }
    }
}
