/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.security.auth.Subject;
import javax.security.auth.login.AccountExpiredException;
import javax.security.auth.login.CredentialExpiredException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.apache.catalina.Container;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.realm.Constants;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.util.StringManager;

import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * <p>
 * Implmentation of <b>Realm</b> that authenticates users via the
 * <em>Java
 * Authentication and Authorization Service</em> (JAAS).
 * JAAS support requires either JDK 1.4 (which includes it as part of
 * the standard platform) or JDK 1.3 (with the plug-in
 * <code>jaas.jar</code> file).
 * </p>
 * 
 * <p>
 * The value configured for the <code>appName</code> property is
 * passed to the <code>javax.security.auth.login.LoginContext</code>
 * constructor, to specify the <em>application NAME</em> used to
 * select the set of relevant <code>LoginModules</code> required.
 * </p>
 * 
 * <p>
 * The JAAS Specification describes the result of a successful login
 * as a <code>javax.security.auth.Subject</code> instance, which can
 * contain zero or more <code>java.security.Principal</code> objects
 * in the return value of the <code>Subject.getPrincipals()</code>
 * method. However, it provides no guidance on how to distinguish
 * Principals that describe the individual user (and are thus
 * appropriate to return as the value of request.getUserPrincipal() in
 * a web application) from the Principal(s) that describe the
 * authorized roles for this user. To maintain as much independence as
 * possible from the underlying <code>LoginMethod</code>
 * implementation executed by JAAS, the following policy is
 * implemented by this Realm:
 * </p>
 * <ul>
 * <li>The JAAS <code>LoginModule</code> is assumed to return a
 * <code>Subject</code> with at least one <code>Principal</code>
 * instance representing the user himself or herself, and zero or more
 * separate <code>Principals</code> representing the security roles
 * authorized for this user.</li>
 * <li>On the <code>Principal</code> representing the user, the
 * Principal NAME is an appropriate value to return via the Servlet
 * API method <code>HttpServletRequest.getRemoteUser()</code>.</li>
 * <li>On the <code>Principals</code> representing the security
 * roles, the NAME is the NAME of the authorized security role.</li>
 * <li>This Realm will be configured with two lists of fully
 * qualified Java class names of classes that implement
 * <code>java.security.Principal</code> - one that identifies
 * class(es) representing a user, and one that identifies class(es)
 * representing a security role.</li>
 * <li>As this Realm iterates over the <code>Principals</code>
 * returned by <code>Subject.getPrincipals()</code>, it will
 * identify the first <code>Principal</code> that matches the "user
 * classes" list as the <code>Principal</code> for this user.</li>
 * <li>As this Realm iterates over the <code>Princpals</code>
 * returned by <code>Subject.getPrincipals()</code>, it will
 * accumulate the set of all <code>Principals</code> matching the
 * "role classes" list as identifying the security roles for this
 * user.</li>
 * <li>It is a configuration error for the JAAS login method to
 * return a validated <code>Subject</code> without a
 * <code>Principal</code> that matches the "user classes" list.</li>
 * <li>By default, the enclosing Container's NAME serves as the
 * application NAME used to obtain the JAAS LoginContext ("Catalina"
 * in a default installation). Tomcat must be able to find an
 * application with this NAME in the JAAS configuration file. Here is
 * a hypothetical JAAS configuration file entry for a
 * database-oriented login module that uses a Tomcat-managed JNDI
 * database resource: <blockquote>
 * 
 * <pre>
 * Catalina {
 *  org.foobar.auth.DatabaseLoginModule REQUIRED
 *  JNDI_RESOURCE=jdbc/AuthDB
 *  USER_TABLE=users
 *  USER_ID_COLUMN=id
 *  USER_NAME_COLUMN=NAME
 *  USER_CREDENTIAL_COLUMN=password
 *  ROLE_TABLE=roles
 *  ROLE_NAME_COLUMN=NAME
 *  PRINCIPAL_FACTORY=org.foobar.auth.impl.SimplePrincipalFactory;
 *  };
 * </pre>
 * 
 * </blockquote></li>
 * <li>To set the JAAS configuration file location, set the
 * <code>CATALINA_OPTS</code> environment variable similar to the
 * following: <blockquote><code>CATALINA_OPTS="-Djava.security.auth.login.config=$CATALINA_HOME/conf/jaas.config"</code></blockquote>
 * </li>
 * <li>As part of the login process, JAASRealm registers its own
 * <code>CallbackHandler</code>, called (unsurprisingly)
 * <code>JAASCallbackHandler</code>. This handler supplies the HTTP
 * requests's username and credentials to the user-supplied
 * <code>LoginModule</code></li>
 * <li>As with other <code>Realm</code> implementations, digested
 * passwords are supported if the <code>&lt;Realm&gt;</code> element
 * in <code>server.xml</code> contains a <code>digest</code>
 * attribute; <code>JAASCallbackHandler</code> will digest the
 * password prior to passing it back to the <code>LoginModule</code></li>
 * </ul>
 * 
 * @author Craig R. McClanahan
 * @author Yoav Shapira
 * @version $Revision: 1.1 $ $Date: 2005-04-01 05:36:52 -0600 (Fri, 01
 *          Apr 2005) $
 */

public class JAASRealm extends RealmBaseAcube {

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * Descriptive information about this <code>Realm</code>
     * implementation.
     */
    protected static final String INFO =
            "org.apache.catalina.realm.JAASRealm/1.0";

    /**
     * Descriptive information about this <code>Realm</code>
     * implementation.
     */
    protected static final String NAME = "JAASRealm";

    /**
     * The string manager for this package.
     */
    protected static final StringManager SM =
            StringManager.getManager(Constants.Package);

    /**
     * The application NAME passed to the JAAS
     * <code>LoginContext</code>, which uses it to select the set
     * of relevant <code>LoginModule</code>s.
     */
    private String appName = null;

    /**
     * Comma-delimited list of <code>java.security.Principal</code>
     * classes that represent individual users.
     */
    private String userClassNames = null;

    /**
     * Comma-delimited list of <code>java.security.Principal</code>
     * classes that represent security roles.
     */
    private String roleClassNames = null;

    /**
     * The set of user class names, split out for easy processing.
     */
    private List<String> userClasses = new ArrayList<String>();

    /**
     * Whether to use context ClassLoader or default ClassLoader. True
     * means use context ClassLoader, and True is the default value.
     */
    private boolean useContextClassLoader = true;

    /**
     * The list of role class names, split out for easy processing.
     */
    private List<String> roleClasses = new ArrayList<String>();

    // -------------------------------------------------------------
    // Properties

    /**
     * setter for the <code>appName</code> member variable
     * 
     * @param applicationName
     *            the <code>appName</code> member variable
     * @deprecated JAAS should use the <code>Engine</code> (domain)
     *             NAME and webpp/host overrides
     */
    public void setAppName(
            String applicationName) {

        this.appName = applicationName;
    }

    /**
     * getter for the <code>appName</code> member variable
     * 
     * @return the <code>appName</code> member variable
     */
    public String getAppName() {

        return this.appName;
    }

    /**
     * Sets whether to use the context or default ClassLoader. True
     * means use context ClassLoader.
     * 
     * @param useContext
     *            True means use context ClassLoader
     */
    public void setUseContextClassLoader(
            boolean useContext) {

        this.useContextClassLoader = useContext;
        containerLog.info("Setting useContextClassLoader = " + useContext);
    }

    /**
     * Returns whether to use the context or default ClassLoader. True
     * means to use the context ClassLoader.
     * 
     * @return The value of useContextClassLoader
     */
    public boolean isUseContextClassLoader() {

        return this.useContextClassLoader;
    }

    /** {@inheritDoc} */
    public void setContainer(
            Container container) {

        super.setContainer(container);

        if (this.appName == null) {
            String applicationName = container.getName();
            applicationName = this.makeLegalForJAAS(applicationName);

            this.appName = applicationName;

            containerLog.info("Set JAAS app NAME " + this.appName);
        }
    }

    /**
     * @return the role class names
     */
    public String getRoleClassNames() {

        return (this.roleClassNames);
    }

    /**
     * Sets the list of comma-delimited classes that represent roles.
     * The classes in the list must implement
     * <code>java.security.Principal</code>. When this accessor is
     * called (for example, by a <code>Digester</code> instance
     * parsing the configuration file), it will parse the class names
     * and store the resulting string(s) into the
     * <code>ArrayList</code> field <code>roleClasses</code>.
     * 
     * @param roleClassNamesParameter
     *            the role class names
     */
    public void setRoleClassNames(
            String roleClassNamesParameter) {

        this.roleClassNames = roleClassNamesParameter;
        this.roleClasses.clear();
        String temp = this.roleClassNames;
        if (temp == null) {
            return;
        }
        while (true) {
            int comma = temp.indexOf(',');
            if (comma < 0) {
                break;
            }
            this.roleClasses.add(temp.substring(
                0, comma).trim());
            temp = temp.substring(comma + 1);
        }
        temp = temp.trim();
        if (temp.length() > 0) {
            this.roleClasses.add(temp);
        }
    }

    /**
     * @return the user class names
     */
    public String getUserClassNames() {

        return (this.userClassNames);
    }

    /**
     * Sets the list of comma-delimited classes that represent
     * individual users. The classes in the list must implement
     * <code>java.security.Principal</code>. When this accessor is
     * called (for example, by a <code>Digester</code> instance
     * parsing the configuration file), it will parse the class names
     * and store the resulting string(s) into the
     * <code>ArrayList</code> field <code>userClasses</code>.
     * 
     * @param userClassNames
     *            the user class names
     */
    public void setUserClassNames(
            String userClassNames) {

        this.userClassNames = userClassNames;
        this.userClasses.clear();
        String temp = this.userClassNames;
        if (temp == null) {
            return;
        }
        while (true) {
            int comma = temp.indexOf(',');
            if (comma < 0) {
                break;
            }
            this.userClasses.add(temp.substring(
                0, comma).trim());
            temp = temp.substring(comma + 1);
        }
        temp = temp.trim();
        if (temp.length() > 0) {
            this.userClasses.add(temp);
        }
    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /** {@inheritDoc} */
    public Principal authenticate(
            String username, String credentials) {

        // Prise en compte du mot de passe saisie par le clavier
        // virtuel
        String password = GestionnaireRefCles.getPassword(credentials);

        // Establish a LoginContext to use for authentication
        try {
            LoginContext loginContext = null;
            if (this.appName == null) {
                this.appName = "Tomcat";
            }

            if (containerLog.isDebugEnabled()) {
                containerLog.debug(JAASRealm.SM.getString(
                    "jaasRealm.beginLogin", username, this.appName));
            }

            // What if the LoginModule is in the container class
            // loader ?
            ClassLoader ocl = null;

            if (this.isUseContextClassLoader()) {
                ocl = Thread.currentThread().getContextClassLoader();
                Thread.currentThread().setContextClassLoader(
                    this.getClass().getClassLoader());
            }

            try {
                loginContext = new LoginContext(
                        this.appName, new JAASCallbackHandler(
                                this, username, password));
            } catch (Throwable e) {
                containerLog.error(
                    JAASRealm.SM.getString("jaasRealm.unexpectedError"), e);
                return (null);
            } finally {
                if (this.isUseContextClassLoader()) {
                    Thread.currentThread().setContextClassLoader(
                        ocl);
                }
            }

            if (containerLog.isDebugEnabled()) {
                containerLog.debug("Login context created " + username);
            }

            // Negotiate a login via this LoginContext
            Subject subject = null;
            try {
                loginContext.login();
                subject = loginContext.getSubject();
                if (subject == null) {
                    if (containerLog.isDebugEnabled()) {
                        containerLog.debug(JAASRealm.SM.getString(
                            "jaasRealm.failedLogin", username));
                    }
                    return (null);
                }
            } catch (AccountExpiredException e) {
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM.getString(
                        "jaasRealm.accountExpired", username));
                }
                return (null);
            } catch (CredentialExpiredException e) {
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM.getString(
                        "jaasRealm.credentialExpired", username));
                }
                return (null);
            } catch (FailedLoginException e) {
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM.getString(
                        "jaasRealm.failedLogin", username));
                }
                return (null);
            } catch (LoginException e) {
                containerLog.warn(
                    JAASRealm.SM.getString(
                        "jaasRealm.loginException", username), e);
                return (null);
            } catch (Throwable e) {
                containerLog.error(
                    JAASRealm.SM.getString("jaasRealm.unexpectedError"), e);
                return (null);
            }

            if (containerLog.isDebugEnabled()) {
                containerLog.debug(JAASRealm.SM.getString(
                    "jaasRealm.loginContextCreated", username));
            }

            // Return the appropriate Principal for this authenticated
            // Subject
            Principal principal = this.createPrincipal(
                username, subject);
            if (principal == null) {
                containerLog.debug(JAASRealm.SM.getString(
                    "jaasRealm.authenticateFailure", username));
                return (null);
            }
            if (containerLog.isDebugEnabled()) {
                containerLog.debug(JAASRealm.SM.getString(
                    "jaasRealm.authenticateSuccess", username));
            }

            return (principal);
        } catch (Throwable t) {
            containerLog.error(
                "error ", t);
            return null;
        }
    }

    // --------------------------------------------------------
    // Package
    // Methods

    // ------------------------------------------------------
    // Protected
    // Methods

    /** {@inheritDoc} */
    protected String getName() {

        return (JAASRealm.NAME);

    }

    /** {@inheritDoc} */
    protected String getPassword(
            String username) {

        return (null);

    }

    /** {@inheritDoc} */
    protected Principal getPrincipal(
            String username) {

        return (null);

    }

    /**
     * Identify and return a <code>java.security.Principal</code>
     * instance representing the authenticated user for the specified
     * <code>Subject</code>. The Principal is constructed by
     * scanning the list of Principals returned by the
     * JAASLoginModule. The first <code>Principal</code> object that
     * matches one of the class names supplied as a "user class" is
     * the user Principal. This object is returned to tha caller. Any
     * remaining principal objects returned by the LoginModules are
     * mapped to roles, but only if their respective classes match one
     * of the "role class" classes. If a user Principal cannot be
     * constructed, return <code>null</code>.
     * 
     * @param username
     *            the user NAME
     * @param subject
     *            The <code>Subject</code> representing the
     *            logged-in user
     * @return a <code>java.security.Principal</code> instance
     *         representing the authenticated user
     */
    protected Principal createPrincipal(
            String username, Subject subject) {

        List<String> roles = new ArrayList<String>();
        Principal userPrincipal = null;

        // Scan the Principals for this Subject
        Iterator<Principal> principals = subject.getPrincipals().iterator();
        while (principals.hasNext()) {
            Principal principal = principals.next();
            String principalClass = principal.getClass().getName();

            if (containerLog.isDebugEnabled()) {
                containerLog.debug(JAASRealm.SM.getString(
                    "jaasRealm.checkPrincipal", principal, principalClass));
            }

            if (userPrincipal == null
                    && this.userClasses.contains(principalClass)) {
                userPrincipal = principal;
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM.getString(
                        "jaasRealm.userPrincipalSuccess", principal.getName()));
                }
            }

            if (this.roleClasses.contains(principalClass)) {
                roles.add(principal.getName());
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM.getString(
                        "jaasRealm.rolePrincipalAdd", principal.getName()));
                }
            }
        }

        // Print failure message if needed
        if (userPrincipal == null) {
            if (containerLog.isDebugEnabled()) {
                containerLog.debug(JAASRealm.SM
                        .getString("jaasRealm.userPrincipalFailure"));
                containerLog.debug(JAASRealm.SM
                        .getString("jaasRealm.rolePrincipalFailure"));
            }
        } else {
            if (roles.size() == 0) {
                if (containerLog.isDebugEnabled()) {
                    containerLog.debug(JAASRealm.SM
                            .getString("jaasRealm.rolePrincipalFailure"));
                }
            }
        }

        // Return the resulting Principal for our authenticated user
        return new GenericPrincipal(
                this, username, null, roles, userPrincipal);
    }

    /**
     * Ensure the given NAME is legal for JAAS configuration. Added
     * for Bugzilla 30869, made protected for easy customization in
     * case my implementation is insufficient, which I think is very
     * likely.
     * 
     * @param src
     *            The NAME to validate
     * @return A string that's a valid JAAS realm NAME
     */
    protected String makeLegalForJAAS(
            final String src) {

        String result = src;

        // Default NAME is "other" per JAAS spec
        if (result == null) {
            result = "other";
        }

        // Strip leading slash if present, as Sun JAAS impl
        // barfs on it (see Bugzilla 30869 bug report).
        if (result.startsWith("/")) {
            result = result.substring(1);
        }

        return result;
    }

    // ------------------------------------------------------
    // Lifecycle
    // Methods

    /**
     * 
     * Prepare for active use of the public methods of this
     * <code>Component</code>.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                prevents it from being started
     */
    public void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

    }

    /**
     * Gracefully shut down active use of the public methods of this
     * <code>Component</code>.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                needs to be reported
     */
    public void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

    }

}
