/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.security.Principal;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.catalina.LifecycleException;
import org.apache.catalina.realm.Constants;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.util.StringManager;

import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * 
 * Implmentation of <b>Realm</b> that works with any JDBC supported
 * database. See the JDBCRealm.howto for more details on how to set up
 * the database and for configuration options.
 * 
 * <p>
 * <strong>TODO</strong> - Support connection pooling (including
 * message format objects) so that <code>authenticate()</code> does
 * not have to be synchronized and would fix the ugly connection
 * logic.
 * </p>
 * 
 * @author Craig R. McClanahan
 * @author Carson McDonald
 * @author Ignacio Ortega
 * @version $Revision: 1.1 $ $Date: 2007-03-26 08:14:20 $
 */

public class JDBCRealm extends RealmBaseAcube {

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * The string manager for this package.
     */
    protected static final StringManager SM =
            StringManager.getManager(Constants.Package);

    /**
     * Descriptive information about this Realm implementation.
     */
    protected static final String INFO =
            "org.apache.catalina.realm.JDBCRealm/1.0";

    /**
     * Descriptive information about this Realm implementation.
     */
    protected static final String NAME = "JDBCRealm";

    /**
     * The connection username to use when trying to connect to the
     * database.
     */
    private String connectionName = null;

    /**
     * The connection URL to use when trying to connect to the
     * database.
     */
    private String connectionPassword = null;

    /**
     * The connection URL to use when trying to connect to the
     * database.
     */
    private String connectionURL = null;

    /**
     * The connection to the database.
     */
    private Connection dbConnection = null;

    /**
     * Instance of the JDBC Driver class we use as a connection
     * factory.
     */
    private Driver driver = null;

    /**
     * The JDBC driver to use.
     */
    private String driverName = null;

    /**
     * The PreparedStatement to use for authenticating users.
     */
    private PreparedStatement preparedCredentials = null;

    /**
     * The PreparedStatement to use for identifying the roles for a
     * specified user.
     */
    private PreparedStatement preparedRoles = null;

    /**
     * The column in the user role table that names a role
     */
    private String roleNameCol = null;

    /**
     * The column in the user table that holds the user's credintials
     */
    private String userCredCol = null;

    /**
     * The column in the user table that holds the user's NAME
     */
    private String userNameCol = null;

    /**
     * The table that holds the relation between user's and roles
     */
    private String userRoleTable = null;

    /**
     * The table that holds user data.
     */
    private String userTable = null;

    // -------------------------------------------------------------
    // Properties

    /**
     * Return the username to use to connect to the database.
     * 
     * @return the username to use to connect to the database.
     * 
     */
    public String getConnectionName() {

        return this.connectionName;
    }

    /**
     * Set the username to use to connect to the database.
     * 
     * @param connectionName
     *            Username
     */
    public void setConnectionName(
            String connectionName) {

        this.connectionName = connectionName;
    }

    /**
     * Return the password to use to connect to the database.
     * 
     * @return the password to use to connect to the database.
     */
    public String getConnectionPassword() {

        return this.connectionPassword;
    }

    /**
     * Set the password to use to connect to the database.
     * 
     * @param connectionPassword
     *            User password
     */
    public void setConnectionPassword(
            String connectionPassword) {

        this.connectionPassword = connectionPassword;
    }

    /**
     * Return the URL to use to connect to the database.
     * 
     * @return the URL to use to connect to the database.
     */
    public String getConnectionURL() {

        return this.connectionURL;
    }

    /**
     * Set the URL to use to connect to the database.
     * 
     * @param connectionURL
     *            The new connection URL
     */
    public void setConnectionURL(
            String connectionURL) {

        this.connectionURL = connectionURL;
    }

    /**
     * Return the JDBC driver that will be used.
     * 
     * @return the JDBC driver that will be used.
     */
    public String getDriverName() {

        return this.driverName;
    }

    /**
     * Set the JDBC driver that will be used.
     * 
     * @param driverName
     *            The driver NAME
     */
    public void setDriverName(
            String driverName) {

        this.driverName = driverName;
    }

    /**
     * Return the column in the user role table that names a role.
     * 
     * @return the column in the user role table that names a role.
     */
    public String getRoleNameCol() {

        return this.roleNameCol;
    }

    /**
     * Set the column in the user role table that names a role.
     * 
     * @param roleNameCol
     *            The column NAME
     */
    public void setRoleNameCol(
            String roleNameCol) {

        this.roleNameCol = roleNameCol;
    }

    /**
     * Return the column in the user table that holds the user's
     * credentials.
     * 
     * @return the column in the user table that holds the user's
     *         credentials.
     */
    public String getUserCredCol() {

        return this.userCredCol;
    }

    /**
     * Set the column in the user table that holds the user's
     * credentials.
     * 
     * @param userCredCol
     *            The column NAME
     */
    public void setUserCredCol(
            String userCredCol) {

        this.userCredCol = userCredCol;
    }

    /**
     * Return the column in the user table that holds the user's NAME.
     * 
     * @return the column in the user table that holds the user's
     *         NAME.
     */
    public String getUserNameCol() {

        return this.userNameCol;
    }

    /**
     * Set the column in the user table that holds the user's NAME.
     * 
     * @param userNameCol
     *            The column NAME
     */
    public void setUserNameCol(
            String userNameCol) {

        this.userNameCol = userNameCol;
    }

    /**
     * Return the table that holds the relation between user's and
     * roles.
     * 
     * @return the table that holds the relation between user's and
     *         roles.
     */
    public String getUserRoleTable() {

        return this.userRoleTable;
    }

    /**
     * Set the table that holds the relation between user's and roles.
     * 
     * @param userRoleTable
     *            The table NAME
     */
    public void setUserRoleTable(
            String userRoleTable) {

        this.userRoleTable = userRoleTable;
    }

    /**
     * Return the table that holds user data..
     * 
     * @return the table that holds user data..
     */
    public String getUserTable() {

        return this.userTable;
    }

    /**
     * Set the table that holds user data.
     * 
     * @param userTable
     *            The table NAME
     */
    public void setUserTable(
            String userTable) {

        this.userTable = userTable;
    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /** {@inheritDoc} */
    public synchronized Principal authenticate(
            String username, String credentials) {

        // Number of tries is the numebr of attempts to connect to the
        // database
        // during this login attempt (if we need to open the database)
        // This needs rewritten wuth better pooling support, the
        // existing
        // code
        // needs signature changes since the Prepared statements needs
        // cached
        // with the connections.
        // The code below will try twice if there is a SQLException so
        // the
        // connection may try to be opened again. On normal conditions
        // (including
        // invalid login - the above is only used once.
        int numberOfTries = 2;

        // Prise en compte du mot de passe saisie par le clavier
        // virtuel
        String password = GestionnaireRefCles.getPassword(credentials);

        while (numberOfTries > 0) {
            try {

                // Ensure that we have an open database connection
                this.open();

                // Acquire a Principal object for this user
                Principal principal = this.authenticate(
                    this.dbConnection, username, password);

                // Return the Principal (if any)
                return (principal);

            } catch (SQLException e) {

                // Log the problem for posterity
                containerLog.error(
                    JDBCRealm.SM.getString("jdbcRealm.exception"), e);

                // Close the connection so that it gets reopened next
                // time
                if (this.dbConnection != null) {
                    this.close(this.dbConnection);
                }

            }

            numberOfTries--;
        }

        // Worst case scenario
        return null;

    }

    // --------------------------------------------------------
    // Package
    // Methods

    // ------------------------------------------------------
    // Protected
    // Methods

    /**
     * Return the Principal associated with the specified username and
     * credentials, if there is one; otherwise return
     * <code>null</code>.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username of the Principal to look up
     * @param credentials
     *            Password or other credentials to use in
     *            authenticating this username
     * @return the Principal associated with the specified username
     *         and credentials
     */
    public synchronized Principal authenticate(
            Connection dbConnection, String username, String credentials) {

        // No user - can't possibly authenticate
        if (username == null) {
            return (null);
        }

        // Look up the user's credentials
        String dbCredentials = this.getPassword(username);

        // Validate the user's credentials
        boolean validated = false;
        if (this.hasMessageDigest()) {
            // Hex hashes should be compared case-insensitive
            validated =
                    (this.digest(credentials).equalsIgnoreCase(dbCredentials));
        } else {
            validated = (this.digest(credentials).equals(dbCredentials));
        }

        if (validated) {
            if (containerLog.isTraceEnabled()) {
                containerLog.trace(JDBCRealm.SM.getString(
                    "jdbcRealm.authenticateSuccess", username));
            }
        } else {
            if (containerLog.isTraceEnabled()) {
                containerLog.trace(JDBCRealm.SM.getString(
                    "jdbcRealm.authenticateFailure", username));
            }
            return (null);
        }

        ArrayList<String> roles = this.getRoles(username);

        // Create and return a suitable Principal for this user
        return (new GenericPrincipal(
                this, username, credentials, roles));

    }

    /**
     * Close the specified database connection.
     * 
     * @param dbConnection
     *            The connection to be closed
     */
    protected void close(
            Connection dbConnection) {

        // Do nothing if the database connection is already closed
        if (dbConnection == null) {
            return;
        }

        // Close our prepared statements (if any)
        try {
            this.preparedCredentials.close();
        } catch (Throwable f) {

        }
        this.preparedCredentials = null;

        try {
            this.preparedRoles.close();
        } catch (Throwable f) {

        }
        this.preparedRoles = null;

        // Close this database connection, and log any errors
        try {
            dbConnection.close();
        } catch (SQLException e) {
            containerLog.warn(
                JDBCRealm.SM.getString("jdbcRealm.close"), e); // Just
            // log
            // it
            // here
        } finally {
            this.dbConnection = null;
        }

    }

    /**
     * Return a PreparedStatement configured to perform the SELECT
     * required to retrieve user credentials for the specified
     * username.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which credentials should be retrieved
     * @return a PreparedStatement configured
     * @exception SQLException
     *                if a database error occurs
     */
    protected PreparedStatement credentials(
            Connection dbConnection, String username) throws SQLException {

        if (this.preparedCredentials == null) {
            StringBuffer sb = new StringBuffer(
                    "SELECT ");
            sb.append(this.userCredCol);
            sb.append(" FROM ");
            sb.append(this.userTable);
            sb.append(" WHERE ");
            sb.append(this.userNameCol);
            sb.append(" = ?");

            if (containerLog.isDebugEnabled()) {
                containerLog.debug("credentials query: " + sb.toString());
            }

            this.preparedCredentials =
                    dbConnection.prepareStatement(sb.toString());
        }

        if (username == null) {
            this.preparedCredentials.setNull(
                1, java.sql.Types.VARCHAR);
        } else {
            this.preparedCredentials.setString(
                1, username);
        }

        return (this.preparedCredentials);
    }

    /** {@inheritDoc} */
    protected String getName() {

        return (JDBCRealm.NAME);

    }

    /** {@inheritDoc} */
    protected String getPassword(
            String username) {

        // Look up the user's credentials
        String dbCredentials = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;

        // Number of tries is the numebr of attempts to connect to the
        // database
        // during this login attempt (if we need to open the database)
        // This needs rewritten wuth better pooling support, the
        // existing
        // code
        // needs signature changes since the Prepared statements needs
        // cached
        // with the connections.
        // The code below will try twice if there is a SQLException so
        // the
        // connection may try to be opened again. On normal conditions
        // (including
        // invalid login - the above is only used once.
        int numberOfTries = 2;
        while (numberOfTries > 0) {
            try {

                // Ensure that we have an open database connection
                this.open();

                try {
                    stmt = this.credentials(
                        this.dbConnection, username);
                    rs = stmt.executeQuery();

                    if (rs.next()) {
                        dbCredentials = rs.getString(1);
                    }
                    rs.close();
                    rs = null;
                    if (dbCredentials == null) {
                        return (null);
                    }

                    dbCredentials = dbCredentials.trim();
                    return dbCredentials;

                } finally {
                    if (rs != null) {
                        try {
                            rs.close();
                        } catch (SQLException e) {
                            containerLog
                                    .warn(JDBCRealm.SM
                                            .getString("jdbcRealm.abnormalCloseResultSet"));
                        }
                    }
                    this.dbConnection.commit();
                }

            } catch (SQLException e) {

                // Log the problem for posterity
                containerLog.error(
                    JDBCRealm.SM.getString("jdbcRealm.exception"), e);

                // Close the connection so that it gets reopened next
                // time
                if (this.dbConnection != null) {
                    this.close(this.dbConnection);
                }

            }

            numberOfTries--;
        }

        return (null);
    }

    /** {@inheritDoc} */
    protected Principal getPrincipal(
            String username) {

        return (new GenericPrincipal(
                this, username, this.getPassword(username), this
                        .getRoles(username)));

    }

    /**
     * Return the roles associated with the gven user NAME.
     * 
     * @return the roles associated with the gven user NAME.
     */
    protected ArrayList<String> getRoles(
            String username) {

        PreparedStatement stmt = null;
        ResultSet rs = null;

        // Number of tries is the numebr of attempts to connect to the
        // database
        // during this login attempt (if we need to open the database)
        // This needs rewritten wuth better pooling support, the
        // existing
        // code
        // needs signature changes since the Prepared statements needs
        // cached
        // with the connections.
        // The code below will try twice if there is a SQLException so
        // the
        // connection may try to be opened again. On normal conditions
        // (including
        // invalid login - the above is only used once.
        int numberOfTries = 2;
        while (numberOfTries > 0) {
            try {

                // Ensure that we have an open database connection
                this.open();

                try {
                    // Accumulate the user's roles
                    ArrayList<String> roleList = new ArrayList<String>();
                    stmt = this.roles(
                        this.dbConnection, username);
                    rs = stmt.executeQuery();
                    while (rs.next()) {
                        String role = rs.getString(1);
                        if (null != role) {
                            roleList.add(role.trim());
                        }
                    }
                    rs.close();
                    rs = null;

                    return (roleList);

                } finally {
                    if (rs != null) {
                        try {
                            rs.close();
                        } catch (SQLException e) {
                            containerLog
                                    .warn(JDBCRealm.SM
                                            .getString("jdbcRealm.abnormalCloseResultSet"));
                        }
                    }
                    this.dbConnection.commit();
                }

            } catch (SQLException e) {

                // Log the problem for posterity
                containerLog.error(
                    JDBCRealm.SM.getString("jdbcRealm.exception"), e);

                // Close the connection so that it gets reopened next
                // time
                if (this.dbConnection != null) {
                    this.close(this.dbConnection);
                }

            }

            numberOfTries--;
        }

        return (null);

    }

    /**
     * Open (if necessary) and return a database connection for use by
     * this Realm.
     * 
     * @return a database connection
     * @exception SQLException
     *                if a database error occurs
     */
    protected Connection open() throws SQLException {

        // Do nothing if there is a database connection already open
        if (this.dbConnection != null) {
            return (this.dbConnection);
        }

        // Instantiate our database driver if necessary
        if (this.driver == null) {
            try {
                Class<?> clazz = Class.forName(this.driverName);
                this.driver = (Driver) clazz.newInstance();
            } catch (Throwable e) {
                throw new SQLException(
                        e.getMessage());
            }
        }

        // Open a new connection
        Properties props = new Properties();
        if (this.connectionName != null) {
            props.put(
                "user", this.connectionName);
        }
        if (this.connectionPassword != null) {
            props.put(
                "password", this.connectionPassword);
        }
        this.dbConnection = this.driver.connect(
            this.connectionURL, props);
        this.dbConnection.setAutoCommit(false);
        return (this.dbConnection);

    }

    /**
     * Return a PreparedStatement configured to perform the SELECT
     * required to retrieve user roles for the specified username.
     * 
     * @param dbConnection
     *            The database connection to be used
     * @param username
     *            Username for which roles should be retrieved
     * 
     * @return a PreparedStatement configured
     * @exception SQLException
     *                if a database error occurs
     */
    protected PreparedStatement roles(
            Connection dbConnection, String username) throws SQLException {

        if (this.preparedRoles == null) {
            StringBuffer sb = new StringBuffer(
                    "SELECT ");
            sb.append(this.roleNameCol);
            sb.append(" FROM ");
            sb.append(this.userRoleTable);
            sb.append(" WHERE ");
            sb.append(this.userNameCol);
            sb.append(" = ?");
            this.preparedRoles = dbConnection.prepareStatement(sb.toString());
        }

        this.preparedRoles.setString(
            1, username);
        return (this.preparedRoles);

    }

    // ------------------------------------------------------
    // Lifecycle
    // Methods

    /**
     * 
     * Prepare for active use of the public methods of this Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                prevents it from being started
     */
    public void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

        // Validate that we can open our connection - but let tomcat
        // startup in case the database is temporarily unavailable
        try {
            this.open();
        } catch (SQLException e) {
            containerLog.error(
                JDBCRealm.SM.getString("jdbcRealm.open"), e);
        }

    }

    /**
     * Release our use of this connection so that it can be recycled.
     * 
     * @param dbConnection
     *            The connection to be released
     */
    protected void release(
            Connection dbConnection) {

    }

    /**
     * Gracefully shut down active use of the public methods of this
     * Component.
     * 
     * @exception LifecycleException
     *                if this component detects a fatal error that
     *                needs to be reported
     */
    public void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

        // Close any open DB connection
        this.close(this.dbConnection);

    }

}
