/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.io.UnsupportedEncodingException;
import java.security.Principal;

import org.apache.catalina.util.HexUtils;

import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * Simple implementation of <b>Realm</b> that reads an XML file to
 * configure the valid users, passwords, and roles. The file format
 * (and default file location) are identical to those currently
 * supported by Tomcat 3.X.
 * 
 * @author Craig R. McClanahan
 * @version $Revision: 1.1 $ $Date: 2006-03-26 13:55:03 -0600 (Sun, 26
 *          Mar 2006) $
 */

public abstract class RealmBaseAcube extends
        org.apache.catalina.realm.RealmBase {

    /** {@inheritDoc} */
    public Principal authenticate(
            String username, String credentials) {

        // Prise en compte du mot de passe saisi par le clavier
        // virtuel

        String password = null;
        if (credentials != null) {
            password = GestionnaireRefCles.getPassword(credentials);
        }

        String serverCredentials = this.getPassword(username);

        boolean validated;
        if (serverCredentials == null) {
            validated = false;
        } else if (this.hasMessageDigest()) {
            validated =
                    serverCredentials.equalsIgnoreCase(this.digest(password));
        } else {
            validated = serverCredentials.equals(password);
        }
        if (!validated) {
            if (containerLog.isTraceEnabled()) {
                containerLog.trace(sm.getString(
                    "realmBase.authenticateFailure", username));
            }
            return null;
        }
        if (containerLog.isTraceEnabled()) {
            containerLog.trace(sm.getString(
                "realmBase.authenticateSuccess", username));
        }

        return this.getPrincipal(username);
    }

    /** {@inheritDoc} */
    protected boolean hasMessageDigest() {

        return !(md == null);
    }

    /** {@inheritDoc} */
    protected String digest(
            String credentials) {

        // If no MessageDigest instance is specified, return unchanged
        if (!this.hasMessageDigest()) {
            return (credentials);
        }

        // Digest the user credentials and return as hexadecimal
        synchronized (this) {
            try {
                md.reset();

                byte bytes[] = null;
                if (this.getDigestEncoding() == null) {
                    bytes = credentials.getBytes();
                } else {
                    try {
                        bytes = credentials.getBytes(this.getDigestEncoding());
                    } catch (UnsupportedEncodingException uee) {
                        containerLog.error(
                            "Illegal digestEncoding: "
                                    + this.getDigestEncoding(), uee);
                        throw new IllegalArgumentException(
                                uee.getMessage());
                    }
                }
                md.update(bytes);

                return (HexUtils.convert(md.digest()));
            } catch (Exception e) {
                containerLog.error(
                    sm.getString("realmBase.digest"), e);
                return (credentials);
            }
        }

    }
}
