/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.realm;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.naming.Context;

import org.apache.catalina.Group;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Role;
import org.apache.catalina.ServerFactory;
import org.apache.catalina.User;
import org.apache.catalina.UserDatabase;
import org.apache.catalina.core.StandardServer;
import org.apache.catalina.realm.Constants;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.util.StringManager;

/**
 * <p>
 * Implementation of {@link org.apache.catalina.Realm} that is based
 * on an implementation of {@link UserDatabase} made available through
 * the global JNDI resources configured for this instance of Catalina.
 * Set the <code>resourceName</code> parameter to the global JNDI
 * resources NAME for the configured instance of
 * <code>UserDatabase</code> that we should consult.
 * </p>
 * 
 * @author Craig R. McClanahan
 * @version $Revision: 1.1 $ $Date: 2007-03-26 08:14:20 $
 * @since 4.1
 */

public class UserDatabaseRealm extends RealmBaseAcube {

    // ----------------------------------------------------- Instance
    // Variables

    /**
     * Descriptive information about this Realm implementation.
     */
    protected static final String NAME = "UserDatabaseRealm";

    /**
     * The string manager for this package.
     */
    private static StringManager sm =
            StringManager.getManager(Constants.Package);

    /**
     * The <code>UserDatabase</code> we will use to authenticate
     * users and identify associated roles.
     */
    private UserDatabase database = null;

    /**
     * Descriptive information about this Realm implementation.
     */
    private final String info =
            "org.apache.catalina.realm.UserDatabaseRealm/1.0";

    /**
     * The global JNDI NAME of the <code>UserDatabase</code>
     * resource we will be utilizing.
     */
    private String resourceName = "UserDatabase";

    // -------------------------------------------------------------
    // Properties

    /** {@inheritDoc} */
    public String getInfo() {

        return this.info;

    }

    /**
     * Return the global JNDI NAME of the <code>UserDatabase</code>
     * resource we will be using.
     * 
     * @return the global JNDI NAME of the <code>UserDatabase</code>
     */
    public String getResourceName() {

        return this.resourceName;

    }

    /**
     * Set the global JNDI NAME of the <code>UserDatabase</code>
     * resource we will be using.
     * 
     * @param resourceName
     *            The new global JNDI NAME
     */
    public void setResourceName(
            String resourceName) {

        this.resourceName = resourceName;

    }

    // ---------------------------------------------------------
    // Public
    // Methods

    /** {@inheritDoc} */
    public boolean hasRole(
            Principal principal, String role) {

        if (principal instanceof GenericPrincipal) {
            GenericPrincipal gp = (GenericPrincipal) principal;
            if (gp.getUserPrincipal() instanceof User) {
                principal = gp.getUserPrincipal();
            }
        }
        if (!(principal instanceof User)) {
            // Play nice with SSO and mixed Realms
            return super.hasRole(
                principal, role);
        }
        if ("*".equals(role)) {
            return true;
        } else if (role == null) {
            return false;
        }
        User user = (User) principal;
        Role dbrole = this.database.findRole(role);
        if (dbrole == null) {
            return false;
        }
        if (user.isInRole(dbrole)) {
            return true;
        }
        Iterator<?> groups = user.getGroups();
        while (groups.hasNext()) {
            Group group = (Group) groups.next();
            if (group.isInRole(dbrole)) {
                return true;
            }
        }
        return false;
    }

    // ------------------------------------------------------
    // Protected
    // Methods

    /** {@inheritDoc} */
    protected String getName() {

        return (UserDatabaseRealm.NAME);

    }

    /** {@inheritDoc} */
    protected String getPassword(
            String username) {

        User user = this.database.findUser(username);

        if (user == null) {
            return null;
        }

        return (user.getPassword());

    }

    /** {@inheritDoc} */
    protected Principal getPrincipal(
            String username) {

        User user = this.database.findUser(username);
        if (user == null) {
            return null;
        }

        List<String> roles = new ArrayList<String>();
        Iterator<?> uroles = user.getRoles();
        while (uroles.hasNext()) {
            Role role = (Role) uroles.next();
            roles.add(role.getName());
        }
        Iterator<?> groups = user.getGroups();
        while (groups.hasNext()) {
            Group group = (Group) groups.next();
            uroles = group.getRoles();
            while (uroles.hasNext()) {
                Role role = (Role) uroles.next();
                roles.add(role.getName());
            }
        }
        return new GenericPrincipal(
                this, username, user.getPassword(), roles, user);
    }

    // ------------------------------------------------------
    // Lifecycle
    // Methods

    /** {@inheritDoc} */
    public synchronized void start() throws LifecycleException {

        // Perform normal superclass initialization
        super.start();

        try {
            StandardServer server = (StandardServer) ServerFactory.getServer();
            Context context = server.getGlobalNamingContext();
            this.database = (UserDatabase) context.lookup(this.resourceName);
        } catch (Throwable e) {
            containerLog.error(
                UserDatabaseRealm.sm.getString(
                    "userDatabaseRealm.lookup", this.resourceName), e);
            this.database = null;
        }
        if (this.database == null) {
            throw new LifecycleException(
                    UserDatabaseRealm.sm.getString(
                        "userDatabaseRealm.noDatabase", this.resourceName));
        }

    }

    /** {@inheritDoc} */
    public synchronized void stop() throws LifecycleException {

        // Perform normal superclass finalization
        super.stop();

        // Release reference to our user database
        this.database = null;

    }

}
