/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.clavierVirtuel.servlet;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import acube.framework.clavierVirtuel.GestionnaireImage;
import acube.framework.clavierVirtuel.GestionnaireRefCles;

/**
 * <p>
 * Servlet de gnration du clavier virtuel.
 * </p>
 * <p>
 * Cette servlet est appel par le composant ClavierVirtuel du
 * framework ERGO (Version suprieur  2.8.0).
 * </p>
 * 
 * @version 2.7.0
 * @date 5 fev. 08
 * @author EDS
 * 
 */
public class GenerateurCV extends HttpServlet {

    /**
     * <code>ERROR_PATH</code> the path to redirect in the error
     * action
     */
    private static final String ERROR_PATH =
            "/flux/framework/error.xml";

    /**
     * <code>serialVersionUID</code> the serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    /** {@inheritDoc} */
    public void doGet(
            HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {

        String key = request.getParameter("key");
        String styleClavier = request.getParameter("stylecv");
        String iconsCount = request.getParameter("nbIcons");
        int numStyle;
        int nbIcons;

        String erreur = null;

        // We check that all parameters hav been set
        if (styleClavier != null && iconsCount != null) {

            // Code of the style
            numStyle = Integer.parseInt(styleClavier);

            // Check of the number of image to display
            nbIcons = Integer.parseInt(iconsCount);

            // If the style is correct
            if (numStyle > 0 && numStyle <= 2) {

                // If there are no key we create a key and the
                // configuration
                if (key == null) {
                    response.setContentType("text/xml");
                    int digitWidth = 0;
                    int digitHeight = 0;
                    int keyPadWidth = 0;
                    int keyPadHeight = 0;
                    String fluxXMLRetour = "";

                    int nbColumns =
                            GestionnaireImage.getNumberColumnsKeyPad(nbIcons);
                    List<String> listIcons = GestionnaireImage.getListIcons(
                        nbIcons, nbColumns);

                    // We save this list and we get a key to
                    // obtain
                    // this list
                    String newKey = GestionnaireRefCles.generateKey(listIcons);
                    switch (numStyle) {
                        case GestionnaireImage.STYLE_CIRCLE:
                            digitWidth = GestionnaireImage.CIRCLE_DIGIT_WIDTH;
                            digitHeight = GestionnaireImage.CIRCLE_DIGIT_HEIGHT;
                            keyPadWidth = GestionnaireImage.CIRCLE_WIDTH;
                            keyPadHeight = GestionnaireImage.CIRCLE_HEIGHT;
                            break;
                        default:
                            digitWidth = GestionnaireImage.RECT_DIGIT_WIDTH;
                            digitHeight = GestionnaireImage.RECT_DIGIT_HEIGHT;
                            keyPadWidth = GestionnaireImage.RECT_WIDTH;
                            keyPadHeight = GestionnaireImage.RECT_HEIGHT;
                            break;
                    }

                    int margeX = GestionnaireImage.getMarge(
                        keyPadWidth, nbColumns, digitWidth);
                    int margeY = GestionnaireImage.getMarge(
                        keyPadHeight, nbColumns, digitHeight);
                    fluxXMLRetour =
                            "<page><key>" + newKey
                                    + "</key><keypad><nbColumns>" + nbColumns
                                    + "</nbColumns>" + "<digitWidth>"
                                    + digitWidth + "</digitWidth>"
                                    + "<digitHeight>" + digitHeight
                                    + "</digitHeight>" + "<margeX>" + margeX
                                    + "</margeX>" + "<margeY>" + margeY
                                    + "</margeY>" + "<width>" + keyPadWidth
                                    + "</width>" + "<height>" + keyPadHeight
                                    + "</height><emptyCells>";
                    int cellulesLibres[] =
                            GestionnaireImage.genererMapTouchesLibres(
                                listIcons, nbIcons);
                    for (int i = 0; i < cellulesLibres.length; i++) {
                        fluxXMLRetour += cellulesLibres[i] + "|";
                    }
                    fluxXMLRetour += "</emptyCells></keypad></page>";
                    response.getWriter().println(
                        fluxXMLRetour);
                } else {

                    // Now We have a key so we create the pictures
                    GestionnaireImage gestImg = new GestionnaireImage();
                   response.setContentType("image/png");
                    response.getOutputStream().write(
                        gestImg.createImage(
                            key, numStyle, nbIcons));
                }
            } else {

                // We return an exception of configuration
                erreur = "Erreur de configuration";
            }
        } else {

            // We return an exception of configuration
            erreur = "Erreur de configuration";
        }

        // We check if no error appended
        if (erreur != null) {

            String url =
                    GenerateurCV.ERROR_PATH + "?code=ERR-TEC-05&paramLibelle="
                            + erreur
                            + "&paramNota=&mailTo=false&faq=false&aide=false";

            request.getRequestDispatcher(
                url).forward(
                request, response);
        }

    }

    /** {@inheritDoc} */
    public void doPost(
            HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {

        this.doGet(
            request, response);
    }
}
