/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */
package acube.framework.export.struts2;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import acube.framework.export.vo.presentation.ColVO;
import acube.framework.export.vo.presentation.RowVO;

/**
 * <p>
 * This class is use to export table into Excel or CSV format
 * </p>
 * 
 * @version 2.6.1
 * @memberof General
 */
public final class Export {

    /**
     * <code>NUMBER_OF_GROUP</code> the number of group in a table
     * composant
     */
    private static final int NUMBER_OF_GROUP = 3;

    /**
     * <code>EURO</code> the EURO caracter
     */
    private static final String EURO = "&#8364;";

    /**
     * <code>COLUMN_FORMAT_PREFIX</code> the prefix of column format
     * of the request parameter
     */
    private static final String COLUMN_FORMAT_PREFIX = "column_format_";

    /**
     * Private constructor
     */
    private Export() {

    }

    /**
     * Method use to generate all rows present in the request.
     * 
     * @param request
     *            request that contains rows
     * @return list of VO
     */
    public static List<RowVO> generateRows(
            Map<String, String[]> request) {

        Integer nbColumns = new Integer(
                request.get("nbColumns")[0]);
        Integer nbLines = new Integer(
                request.get("nbLines")[0]);

        ArrayList<RowVO> rows = new ArrayList<RowVO>();

        // We add the column's titles
        RowVO rowColumnsVO = new RowVO();
        for (int i = 0; i < nbColumns; i++) {

            String column = request.get("column_title_" + i)[0];

            ColVO columnVO = new ColVO();
            columnVO.setFormat("HEADER");
            columnVO.setValue(column);

            rowColumnsVO.getCols().add(
                columnVO);
        }
        rows.add(rowColumnsVO);

        // We add lines
        for (int i = 0; i < nbLines; i++) {

            // We add the line for calcul
            rows.addAll(Export.getCalcul(
                request, i, nbColumns));

            // We add the line for group
            rows.addAll(Export.getGroup(
                request, i, nbColumns));

            RowVO lineVO = new RowVO();

            for (int j = 0; j < nbColumns; j++) {

                String column = request.get("line_" + i + "_" + j)[0];

                ColVO columnVO = new ColVO();
                columnVO.setFormat(request
                        .get(Export.COLUMN_FORMAT_PREFIX + j)[0]);
                if (column != null) {
                    columnVO.setValue(column.replaceAll(
                        Export.EURO, ""));
                }

                lineVO.getCols().add(
                    columnVO);
            }
            rows.add(lineVO);
        }

        // We add the line for the last calcul
        rows.addAll(Export.getCalcul(
            request, nbLines, nbColumns));

        // We add the line for the total
        rows.addAll(Export.getTotal(
            request, nbColumns));

        return rows;
    }

    /**
     * this method is use generate the calcul row of a table.
     * 
     * @param request
     *            request that contains calcul
     * @param line
     *            the line where the calcul is to put
     * @param nbColumns
     *            the number of columns of the table
     * @return list of row of calcul
     */
    private static List<RowVO> getCalcul(
            Map<String, String[]> request, int line, int nbColumns) {

        ArrayList<RowVO> list = new ArrayList<RowVO>();

        for (int i = Export.NUMBER_OF_GROUP; i >= 1; i--) {

            // We add line for calcul
            if (request.get("calcul_"
                + i + "_" + line + "_0") != null) {

                RowVO calculLineVO = new RowVO();

                for (int j = 0; j < nbColumns; j++) {

                    String column =
                            request.get("calcul_" + i + "_" + line
                                    + "_" + j)[0];

                    ColVO columnVO = new ColVO();
                    columnVO.setFormat(request
                            .get(Export.COLUMN_FORMAT_PREFIX + j)[0]);
                    if (column != null) {
                        columnVO.setValue(column.replaceAll(
                            Export.EURO, ""));
                    }

                    calculLineVO.getCols().add(
                        columnVO);
                }
                list.add(calculLineVO);
            }
        }

        return list;
    }

    /**
     * Method use to export a group line.
     * 
     * @param request
     *            the request that contains the table
     * @param line
     *            the line where the group is to put
     * @param nbColumns
     *            the number of columns in the table
     * @return the list of rows for the group
     */
    private static List<RowVO> getGroup(
        Map<String, String[]> request, int line, int nbColumns) {

        ArrayList<RowVO> list = new ArrayList<RowVO>();

        for (int i = 1; i <= Export.NUMBER_OF_GROUP; i++) {
            // We add line for group
            if (request.get("group_" + i + "_" + line) != null) {

                RowVO groupeLineVO = new RowVO();

                String column = request.get("group_" + i + "_" + line)[0];

                ColVO columnVO = new ColVO();
                columnVO.setValue(column);
                columnVO.setNumberSpan(nbColumns);

                groupeLineVO.getCols().add(
                    columnVO);

                list.add(groupeLineVO);

            }

        }

        return list;
    }

    /**
     * Method use to export a total line
     * 
     * @param request
     *            the request that contains the table
     * @param nbColumns
     *            the number of column in the table
     * @return the list of row of the total line
     */
    private static List<RowVO> getTotal(
        Map<String, String[]> request, int nbColumns) {

        ArrayList<RowVO> list = new ArrayList<RowVO>();

        // We add line for total
        if (request.get("total_0") != null) {

            RowVO calculLineVO = new RowVO();

            for (int j = 0; j < nbColumns; j++) {

                String column = request.get("total_" + j)[0];

                ColVO columnVO = new ColVO();
                columnVO.setFormat(request
                        .get(Export.COLUMN_FORMAT_PREFIX + j)[0]);
                if (column != null) {
                    columnVO.setValue(column.replaceAll(
                        Export.EURO, ""));
                }

                calculLineVO.getCols().add(
                    columnVO);
            }
            list.add(calculLineVO);
        }

        return list;
    }
}
