/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.technical;

import java.util.Enumeration;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import acube.framework.exception.ConfigurationException;

/**
 * Classe dfinissant un manager de configuration donn :
 * une seule instance, pour un fichier de configuration donn, 
 * est cre par la classe statique ConfigurationManager.
 * 
 * @version 1.0
 * @see ConfigurationManager
 * @memberof General
 */
public class Configuration {
    
    /**
     * Fichier de proprits associ  l'instance de Configuration.
     */
    private ResourceBundle m_Config;
    
    /**
     * Cre une instance de la classe servant  la lecture du fichier 
     * dont le nom est pass en paramtre.
     *
     * @param nomConfig nom du fichier de proprits (sans l'extension ".properties").			
     */
    public Configuration(String nomConfig) {	
    	try {
    		m_Config = PropertyResourceBundle.getBundle(nomConfig);
    	}
    	catch (Exception e) {
    		throw new ConfigurationException(this.getClass().getName(),
    			"Fichier de configuration \"" + nomConfig + ".properties\" manquant.", e);
    	}
    }
    
    /**
     * Retourne un booleen en fonction de la valeur de la proprit correspondant
     *  la cl passe en paramtre.
     * <ul>
     * 		<li>Si la valeur = "true", renvoie true
     * 		<li>Sinon, renvoie false
     * </ul><p>
     *
     * @param cle cl de la proprit dans le fichier ".properties"
     * @return valeur de la proprit passe en paramtre
     */
    public boolean getBoolean(String cle) {
    
    	try {
    		return new Boolean(m_Config.getString(cle)).booleanValue();
    	} catch (Exception e) {
    		throw new ConfigurationException(this.getClass().getName(),
    			"Ressource manquante : " + cle, e);
    	}
    }
    
    /**
     * Rcupre sous forme d'entier (int) la valeur de la proprit spcifie en paramtre.
     *  
     * @param cle cl de la proprit dans le fichier ".properties"
     * @return valeur de la proprit passe en paramtre
     */
    public int getInt(String cle) {
    
    	try {
    		return new Integer(m_Config.getString(cle)).intValue();
    	}
    	catch (Exception e) {
    		throw new ConfigurationException(this.getClass().getName(),
    			"Ressource manquante : " + cle, e);
    	}
    }
    
    /**
     * Rcupre sous forme de String la valeur de la proprit spcifie en paramtre
     * 
     * @param cle cl de la proprit dans le fichier ".properties"
     * @return valeur de la proprit passe en paramtre
     */
    public String getString(String cle) {
    
    	try {
    		return m_Config.getString(cle);
    	}
    	catch (Exception e) {
    		throw new ConfigurationException(this.getClass().getName(),
    			"Ressource manquante : " + cle, e);
    	}
    }
    
    /**
     * Recupre la liste des cles
     * @return Enumeration la liste de cles
     */
    public Enumeration getKeys() {
        
       return m_Config.getKeys();
    }
}
