/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.technical;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

import acube.framework.exception.TechnicalException;

/**
 * La classe Libelle permet de charger un fichier de type "properties"
 * (dont le contenu est du type "cl = valeur"), et de remplacer les
 * diffrents paramtres contenu dans un champ valeur, par la valeur
 * de ces paramtres. Cette classe utilise des objets de types
 * ParametresLibelle pour la dtermination de la valeur de ces
 * paramtres.
 * 
 * ex: - un fichier "libelle" contient les lignes suivantes: ...
 * message = fichier [0] introuvable ... - ce fichier est charg dans
 * un objet de type Libelle (cf constructeur) - cration d'un objet de
 * type ParametreLibelle pour dfinir la valeur que doit prendre le
 * parametre "[0]"
 * 
 * ... libelle.ajouterParametre("0", "D:\monFichier.txt"); ... - lors
 * de l'appel  la mthode "getlibelle("message", parametres)",
 * l'objet de type Libelle remplace le paramtre contenu dans le
 * libelle correspondant par sa valeur
 * 
 * @version 1.0
 * @see LabelManager
 * @memberof General
 */
public final class Label {

    /**
     * Constructor private for utilities classes. It's only to hide the
     * public constructor extended of Object.
     */
    private Label() {
        // nothing
    }

    /**
     * Retourne la valeur d'un libelle contenue dans le fichier de
     * libelles, aprs remplacement des parametres par les valeurs
     * contenues dans l'objet parametres
     * 
     * @param cleLibelle
     *            String cle du libelle a retourner
     * @param parametres
     *            LabelManager valeur des parametres  remplacer dans
     *            le libelle, null si aucun parametres dans ce libelle
     * @return String la valeur du libelle correspondant a la cle
     * @throws TechnicalException
     *             exception technique
     */
    public static String getLabel(
            String cleLibelle, LabelManager parametres)
        throws TechnicalException {

        // liste de cls de parametres contenu dans l'objet parametres
        Vector<String> listeCles;
        // libelle contenue dans le fichier
        String libelle;
        // libelle rsultat, aprs avoir remplacer les paramtres
        // par leurs valeurs
        String libelleFinale;
        // cl parametre courrant
        String cleParamCourant;
        // valeur du parametre courant
        String valeurParamCourant;
        // objet PropertyResourceBundle representant le fichier de
        // libelles
        ResourceBundle resourceLibelle;

        try {
            // rcupration du fichier de libelles
            resourceLibelle = ResourceBundle.getBundle(
                Constants.FICHIER_LIBELLES, new Locale(
                        ""));

        } catch (MissingResourceException e) {
            throw new TechnicalException(
                    "Fichier de libelles " + Constants.FICHIER_LIBELLES
                            + " inacessible");

        }

        // rcupration du libelle contenu dans le fichier de libelle
        libelle = resourceLibelle.getString(cleLibelle);

        // construction du libelle finale
        libelleFinale = libelle;

        // si l'objet parametres est non nul,
        // on remplace les parametres contenu
        // dans le libelle par les valeurs contenues dans l'objet
        // parametres
        if ((null != parametres) && (null != libelleFinale)) {

            // rcupration de la liste de cls de paramtres contenu
            // dans l'objet parametres
            listeCles = parametres.getParameterKeys();

            // balayage des parametres
            for (int i = 0; i < listeCles.size(); i++) {

                // rcupration de la cl du parametre courant
                cleParamCourant = (String) listeCles.get(i);
                // rcupration de la valeur du parametre courant
                valeurParamCourant =
                        parametres.getParameterValue(cleParamCourant);

                // remplacement des parametres gales 
                // "cleParametreCourant"
                // contenu dans le libelle courant par la valeur
                // "valeurParametreCourant"
                if (null == valeurParamCourant) {
                    valeurParamCourant = "";
                }

                libelleFinale =
                        libelleFinale.replaceAll(
                            "\\[" + cleParamCourant + "\\]",
                            valeurParamCourant);
            }
        }

        return libelleFinale;
    }

}
