/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.technical.file;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;
import org.apache.log4j.Logger;

import acube.framework.exception.FileWrapperException;
import acube.framework.exception.JDBCWrapperException;
import acube.framework.technical.Configuration;
import acube.framework.technical.ConfigurationManager;
import acube.framework.technical.jdbc.JDBCWrapper;
import acube.framework.technical.log.BuildLogger;
import acube.framework.vo.FileVO;

/**
 * Classe d'encapsulation des accs aux fichiers.
 * Les librairies suivantes sont ncssaires :
 * <ul>
 * <li>common-fileupload-1.1.1</li>
 * <li>common-io-1.2</li>
 * </ul>
 * Cette classe fonctionne avec le fichier de properties file.properties.
 * Il doit contenir :
 * <ul>
 * <li>XXX.path : donne le chemin dans le cas d'un stockage sur filesystem</li>
 * <li>XXX.datasource : donne le nom de la datasource  utiliser</li>
 * <li>XXX.datasourceTomcat : (ligne optionnelle) mettre true ou false pour dire si la datasource est Tomcat (true) ou JDBCWrapper (false)</li>
 * <li>XXX.table : donne le nom de la table ou vont tre stock les informations</li>
 * <li>XXX.lobname : donne le nom de la colonne de la table qui va contenir le BLOB</li>
 * <li>XXX.filename : donne le nom de la colonne de la table qui va contenir le nom du fichier.</li>
 * <li>XXX.index.insert : donne le nom des colonnes spars par des virgules qui vont contenir l'index pour faire l'insert et l'update en base</li>
 * <li>XXX.index.select : donne le nom des colonnes spars par des virgules qui vont contenir l'index pour faire le select et le delete en base</li>
 * </ul>
 * le vecteur de paramtres devra devra respecter la construction des index.
 * L'index est construit de la faon suivante :
 * <ul>
 * <li>Pour INSERT : on prend ce qui est entr dans XXX.index.insert on ajoute XXX.filename si demand et XXX.lobname</li>
 * <li>Pour UPDATE : Ici on fait un SET de XXX.lobname avec une clause WHERE sur XXX.filename si demand et les index</li>
 * <li>Pour SELECT : Cette requte fait un select de XXX.lobname et XXX.filename sur une clause WHERE comportant XXX.filename si demand plus XXX.index.select</li>
 * <li>Pour DELETE : Cette requte se fait en effectuant une clause WHERE sur XXX.filename si demand et sur XXX.index.select</li>
 * </ul>
 * @version 2.5
 * @memberof FileWrapper
 */
public class FileWrapper{
	
	/** <code>m_logger</code>: Definition du logger */
	protected static Logger logger = BuildLogger.getLogger();

	/**
	 * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
	 * logger
	 */
	private static final String THIS_CLASS = "[acube.framework.technical.FileWrapper]";


	/**
	 * Configuration : Donnees specifiques au serveur.
	 */
	protected static Configuration _configServer;
	
	/**
	 * <code>TYPE_FILE_ERASE</code> pass sur les mthodes uploadFileInFile() ou uploadFileInSGBD() permet d'autoriser l'crasement
	 * du fichier cible si il existe 
	 */
	public static final boolean TYPE_FILE_ERASE = true;
	
	/**
	 * <code>TYPE_FILE_NOT_ERASE</code> pass sur les mthodes uploadFileInFile() ou uploadFileInSGBD() permet d'interdire l'crasement
	 * du fichier cible si il existe 
	 */
	public static final boolean TYPE_FILE_NOT_ERASE = false;
	
	/**
	 * <code>TYPE_INSERT</code>  
	 */
	public static final int TYPE_INSERT = 1;
	/**
	 * <code>TYPE_UPDATE</code>  
	 */
	public static final int TYPE_UPDATE = 2;
	/**
	 * <code>TYPE_SELECT</code>  
	 */
	public static final int TYPE_SELECT = 3;
	/**
	 * <code>TYPE_DELETE</code>  
	 */
	public static final int TYPE_DELETE = 4;

	/**
	 * <code>UPLOAD_INSERT</code>  
	 * @deprecated
	 */
    public static final String UPLOAD_INSERT = "insert";

	/**
	 * <code>UPLOAD_UPDATE</code>  
	 * @deprecated
	 */
    public static final String UPLOAD_UPDATE = "update";

    /**
	 * Constructeur par defaut de l'objet <code>FileWrapper</code>.
	 * Rcupre de fichier properties qui permettra de spcifier les diffrentes sources (file ou SGBD)
	 */
	public FileWrapper() {
		// Rcupration des fichiers de proprits.
		_configServer = ConfigurationManager.getInstance("file");		
	}

	/**
	 * Verification sur l'existance du chemin du fichier et du fichier en lui mme
	 * @param path chemin
	 * @param name nom du fichier
	 * @throws FileWrapperException
	 */
	private void verifyFile(String path,String name) throws FileWrapperException{
		File fpath = new File(path);
		File file = new File(path+name);
		
		if (!fpath.exists())
			throw new FileWrapperException("Erreur.FileWrapper.verifieFichier : Chemin inexistant ("+path+")");
		
		if (!file.exists())
			throw new FileWrapperException("Erreur.FileWrapper.verifieFichier : fichier inexistant ("+path+name+")");
	}
	
	/**
	 * Prpare le FileVO pour le transfert de fichier.
	 * Mthode commune utilise dans les mthodes public downloadFileFromFile et
	 * downloadFileFromSGBD
	 * Le ContentType est adapt automatiquement au type de fichier :
	 * <ul>
	 * <li>Word</li>
	 * <li>Excel</li>
	 * <li>Powerpoint</li>
	 * <li>Adobe</li>
	 * <li>zip</li>
	 * </ul>
	 * @param data InputStream contenant les donnes  envoyer
	 * @param fileName nom du fichier envoy
	 * @return FileVO valoris
	 */
	private FileVO downloadFile(InputStream data, String fileName)
	{		
		FileVO fvo = new FileVO();
		fvo.setFichier(fileName);
		fvo.setFluxDonnees(data);
		return fvo;
	}

	/**
	 * Rcupere les information sur un fichier transmis en Multipart.
	 * @param request Request d'ou vient le fichier
	 * @return retourne un FileVO contenant les informations sur le fichier
	 * @throws FileWrapperException
	 * @deprecated voir le constructeur de l'objet FileVO prenant un HttpServletRequest en argument
	 * @version 2.3
	 * @date 14/11/2006
	 */
	public FileVO getInfoFile(HttpServletRequest request) throws FileWrapperException {
		ServletRequestContext sReq = new ServletRequestContext(request);
		FileVO file = null;
		
		boolean isMultipart = ServletFileUpload.isMultipartContent(sReq);
		
		if(!isMultipart){			
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Ceci n'est pas une requte multipart");			
		}

		int nbFile=0;
		try{
			Map params = (Map)request.getAttribute(acube.framework.technical.Constants.PARAMS_REQUEST);
			Iterator itr = params.keySet().iterator();
			while(itr.hasNext()){
				String key = (String)itr.next();
				FileItem item = (FileItem) params.get(key);
				if (!item.isFormField()) {
					if (nbFile==1)
						throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Dfinition de plusieurs fichiers impossible");
					nbFile++;
					String name = item.getName();
					int last = name.lastIndexOf("/");
					if (last==-1)
						last = name.lastIndexOf("\\");
					if (last==-1)
						last = name.lastIndexOf(":");
					
					name=name.substring(last+1);
					file=new FileVO();
					file.setFichier(name);
					file.setFluxDonnees(item.getInputStream());
				}
			}
		}catch (IOException ex){
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : "+ex.getMessage(),ex);									
		}
		if ( nbFile == 0)
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Aucun champs fichier trouv");
		
		return file;
	}
	
	/**
	 * Retourne un InputStream permettant de manipuler le fichier 
	 * @param path chemin o se trouve le fichier
	 * @param name Nom du fichier
	 * @return InputStream correrspondant au fichier
	 * @throws FileWrapperException
	 */
	private InputStream readFile(String path,String name) throws FileWrapperException{
		FileInputStream fi=null;
		ByteArrayInputStream bi=null;
		
		try{
			verifyFile(path,name);
			fi = new FileInputStream(path+name);
			
			ByteArrayOutputStream bo = new ByteArrayOutputStream();
			byte[] data = new byte[fi.available()];
			fi.read(data);
			bo.write(data);
			fi.close();
			bi = new ByteArrayInputStream(bo.toByteArray());
			
		} catch (FileNotFoundException e){
			throw new FileWrapperException("Erreur.FileWrapper.lectureFichier : "+e.getMessage(),e);									
		} catch (FileWrapperException ex){
			throw new FileWrapperException("Erreur.FileWrapper.lectureFichier : "+ex.getMessage(),ex);									
		} catch (IOException ioe){
			throw new FileWrapperException("Erreur.FileWrapper.lectureFichier : "+ioe.getMessage(),ioe);
		}
		return bi;
	}
	
	/**
	 * Ecriture d'un fichier sur disque
	 * @param path chemin o se trouve le fichier
	 * @param name Nom du fichier
	 * @param data InputStream contenant les donnes  crire.
	 * @param override permet ou non d'craser un fichier si il existe dja
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 */
	private void writeFile(String path,String name,InputStream data,boolean override) throws FileWrapperException{
		File fileOut = new File(path+name);
		byte[] responseBody=new byte[8192];

		File fpath = new File (path);
		if (!fpath.exists())
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : Chemin "+path+" inexistant");
		
		if (!fpath.canWrite())
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : Vous n'avez pas les droits d'criture");
		
		File foverride = new File(path+name);
		if (!override && foverride.exists())
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : Vous essayez d'craser un fichier existant");
		
	    ByteArrayOutputStream by = new ByteArrayOutputStream();
	      
	    int length;
	    try{
		    while ((length = data.read(responseBody,0,8192)) != -1){
				by.write(responseBody,0,length);
		    } 
	    } catch (IOException e){
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : "+path+" "+name+"  "+e.getMessage(),e);	    	
	    }
		
	    try{
			FileOutputStream fiOut = new FileOutputStream(fileOut);
			fiOut.write(by.toByteArray());
			fiOut.close();
	    }catch (FileNotFoundException e){
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : "+path+" "+name+"  "+e.getMessage(),e);	    		    	
	    }catch (IOException iex){
			throw new FileWrapperException("Erreur.FileWrapper.ecritureFichier : "+iex.getMessage(),iex);	    		    	
	    }
	}

	/**
	 * Efface un fichier du disque
	 * @param source source dans le fichier de properties
	 * @param name Nom du fichier
	 * @return un boolean vrai si le fichier est ffac ou faux dans le cas contraire
	 * @throws FileWrapperException
	 */
	public boolean deleteFileFromFileSystem(String source,String name)throws FileWrapperException{
		logger.info(THIS_CLASS + "effacerFichier");

		boolean ret=false;
		String pPath=null;
		
		try{
			pPath = _configServer.getString(source+".path");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.deleteFileFromFileSystem : ligne manquante dans le fichier properties "+source+".path");
		}
		
		File fpath=new File(pPath);
		
		if (!fpath.exists())
			throw new FileWrapperException("Erreur.FileWrapper.deleteFileFromFileSystem : Chemin inexistant "+pPath);
		
		File file = new File(pPath+name);

		verifyFile(pPath,name);
		if (!file.canWrite())
			throw new FileWrapperException("Erreur.FileWrapper.deleteFileFromFileSystem : Vous n'avez pas les droits d'criture");

		ret = file.delete();
		
		return ret;
	}
	
	/**
	 * Renomme un fichier 
	 * @param source prfixe properties
	 * @param nameOrg Nom d'origine
	 * @param nameDest Nom de destination
	 * @return vrai ou faux suivant le rsultat du renommage
	 * @throws FileWrapperException
	 */
	public boolean renameFile(String source,String nameOrg,String nameDest)throws FileWrapperException{
		logger.info(THIS_CLASS + "renommerFichier");
		boolean ret = false;
		String path=null;
		
		try{
			path = _configServer.getString(source+".path");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.renommerFichier : ligne manquante dans le fichier properties "+source+".path");
		}

		try{
			verifyFile(path,nameOrg);
		}catch (FileWrapperException e){
			throw new FileWrapperException("Erreur.FileWrapper.renommerFichier : "+e.getMessage(),e);
		}
		
		File fileSrc = new File(path+nameOrg);
		File fileDest = new File(path+nameDest);
		
		try{
			ret = fileSrc.renameTo(fileDest);
		}catch (NullPointerException e){
			throw new FileWrapperException("Erreur.FileWrapper.renommerFichier : "+e.getMessage(),e);
		}
		
		return ret;
	}
	
	/**
	 * Tlcharge un fichier du client vers un fichier sur le serveur 
	 * @param source source dans le fichier de properties
	 * @param request Request contenant le fichier
	 * @param override permet ou non d'craser un fichier si il existe dja
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 */
	public void uploadFileInFile(String source, HttpServletRequest request, boolean override) 
	throws FileWrapperException {
		logger.info(THIS_CLASS + "uploadFileInFile");
		String pPath=null;
		
		try{
			pPath = _configServer.getString(source+".path");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.uploadFileInFile : ligne manquante dans le fichier properties "+source+".path");
		}

		// Lecture du fichier depuis le client
		FileVO file = getInfoFile(request);
		// Ecriture du fichier sur le disque
		writeFile(pPath,file.getFichier(),file.getFluxDonnees(),override);
	}

	/**
	 * Tlcharge un fichier du client vers un fichier sur le serveur 
	 * @param source source dans le fichier de properties
	 * @param file FileVO contenant le fichier
	 * @param override permet ou non d'craser un fichier si il existe dja
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @date 14/11/2006
	 * @version 2.3
	 */
	public void uploadFileInFile(String source, FileVO file, boolean override) 
	throws FileWrapperException {
		logger.info(THIS_CLASS + "uploadFileInFile");
		String pPath=null;
		
		try{
			pPath = _configServer.getString(source+".path");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.uploadFileInFile : ligne manquante dans le fichier properties "+source+".path");
		}
		
		// Ecriture du fichier sur le disque
		writeFile(pPath,file.getFichier(),file.getFluxDonnees(),override);
	}

	/**
	 * Mthode permettant de configurer le JDBCWrapper
	 * @param source source dans le fichier de properties
	 * @param pDataSource type de datasource.
	 * @return le JDBCWrapper configur
	 * @throws JDBCWrapperException
	 */
	private JDBCWrapper getJDBCWrapper(String source,boolean pDataSource) throws JDBCWrapperException{
		String dataSource = null;
		boolean typeDatasource=false;
		
		if (pDataSource==true){
			try{
				dataSource = _configServer.getString(source+".datasource");
			} catch(Exception e){
				throw new JDBCWrapperException("Erreur.FileWrapper.getJDBCWrapper : ligne manquante dans le fichier properties "+source+".datasource");
			}
			
			try{
				typeDatasource = Boolean.parseBoolean(_configServer.getString(source+".datasourceTomcat"));
			}catch (Exception e){
				typeDatasource=false;
			}
		}
		
		return new JDBCWrapper(dataSource,typeDatasource);
	}
	
	/**
	 * Mthode permettant de construir dynamiquement une requete SQL par rapport au fichier file.properties
	 * @param source Source dans le fichier properties
	 * @param stockName, Va t'on stocker en base le nom du fichier
	 * @param type, type de requete  construir
	 * Peut prendre les valeurs suivantes :
	 * <ul>
	 * <li>FileWrapper.TYPE_INSERT : Requete INSERT</li>
	 * <li>FileWrapper.TYPE_UPDATE : Requete UPDATE</li>
	 * <li>FileWrapper.TYPE_SELECT : Requete SELECT</li>
	 * <li>FileWrapper.TYPE_DELETE : Requete DELETE</li>
	 * </ul>
	 * @return La requte souhaite
	 * @throws FileWrapperException
	 */
	private String getSqlRequest(String source,boolean stockName,int type) throws FileWrapperException{
		String req=null;
		String nomfic=null;
		String table=null;
		String nomlob=null;
		String nomIndexInsert=null;
		String nomIndexSelect=null;
		
		try{
			table = _configServer.getString(source+".table");
			nomlob = _configServer.getString(source+".lobname");
			nomIndexInsert = _configServer.getString(source+".index.insert");
			nomIndexSelect = _configServer.getString(source+".index.select");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.getSqlRequest : ligne manquante dans le fichier properties");
		}

		try{
			nomfic = _configServer.getString(source+".filename");
		} catch(Exception e){
			// Ne rien faire ce champs n'est pas obligatoire.
		}

		StringTokenizer token = new StringTokenizer(nomIndexInsert,",");
		StringTokenizer tokenSelect = new StringTokenizer(nomIndexSelect,",");
	
		switch (type){
		case TYPE_INSERT:
			int nbArg = 1;
			req = new String("INSERT INTO "+table+"("+nomIndexInsert+",");
			nbArg+=token.countTokens();
			if (nomfic!=null && !nomfic.equals("") && stockName){
				req=req+nomfic+",";
				nbArg++;
			}
			req=req+nomlob+") VALUES (";
			for (int i=0;i<nbArg;i++)
				req=req+"?,";
			req = req.substring(0,req.length()-1);
			req=req+")";
			break;
		case TYPE_UPDATE:
			req = new String("UPDATE "+table+" SET "+nomlob+"=?");
			if (nomfic!=null && !nomfic.equals("") && stockName)
				req=req+","+nomfic+"=?";
			req=req+" WHERE ";
			while (token.hasMoreTokens()){
				req = req+token.nextToken()+"=?";
				if ( token.hasMoreTokens() )
					req=req+" AND ";
			}
			break;
		case TYPE_SELECT:
			req = new String("SELECT "+nomlob);
			if (nomfic!=null && !nomfic.equals("") && stockName)
				req=req+","+nomfic;
			req=req+" FROM "+table+" WHERE ";
			while (tokenSelect.hasMoreTokens()){
				req = req+tokenSelect.nextToken()+"=?";
				if ( tokenSelect.hasMoreTokens() )
					req=req+" AND ";
			}
			break;
		case TYPE_DELETE:
			req = new String("DELETE FROM "+table+" WHERE ");
			while (tokenSelect.hasMoreTokens()){
				req = req+tokenSelect.nextToken()+"=?";
				if ( tokenSelect.hasMoreTokens() )
					req=req+" AND ";
			}
			break;
		}
		
		return req;
	}
	
	
	
	/**
	 * Tlcharge un fichier du client vers une SGBD sur le serveur 
	 * @param source prfixe dans le fichier properties
	 * @param dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param request Request contenant le fichier
	 * @param indice indice de l'objet dans la base.
	 * @param pSql pointe du le fichier properties donnant la requete sql  executer
	 * @param blName on stock ou pas le nom du fichier
	 * @param override on permet l'crasement d'un enregistrement existant
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @deprecated voir la mthode uploadFileInSGBD prenant un vercteur  la place d'un Object
	 * @see #uploadFileInSGBD(String, String, HttpServletRequest, Object, String, boolean, boolean)
	 */
	public void uploadFileInSGBD(String source, String pDataSource, HttpServletRequest request, Object indice, String pSql, boolean blName, boolean override) 
		throws FileWrapperException,JDBCWrapperException{
		logger.info(THIS_CLASS + "uploadFileInSGBD");

		Vector<Object> vecteur = new Vector<Object>();
		
		// Indice dans la base
		vecteur.add(indice);
    	boolean datasource = false;
    	if (pDataSource!=null)
    		datasource=true;
    	
		uploadFileInSGBD(source,datasource,request,vecteur,blName,override);
	}

	/**
	 * Tlcharge un fichier du client vers une SGBD sur le serveur.
	 * La requte SQL passe doit bien tenir compte que les deux derniers paramtres sont le nom du fichier
	 * (si on doit le stocker) et le champs BLOB. 
	 * @param source prfixe dans le fichier properties
	 * @param pDataSource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param request Request contenant le fichier
	 * @param params paramtres servant de cl d'insertion dans la base.
	 * @param blName on stock ou pas le nom du fichier
	 * @param override on permet l'crasement d'un enregistrement existant
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @version 2.5
	 * @date 03/10/2007
	 */
	public void uploadFileInSGBD(String source, boolean pDataSource, HttpServletRequest request, Vector<Object> params, boolean blName, boolean override) 
	throws FileWrapperException,JDBCWrapperException{
		logger.info(THIS_CLASS + "uploadFileInSGBD");

		// Lecture du fichier depuis le client
		FileVO file = new FileVO(request);
		uploadSGBD(source,pDataSource,file.getFluxDonnees(),params, file.getFichier(),blName,override);
	}
	
	/**
	 * Tlcharge un fichier du client vers une SGBD sur le serveur 
	 * @param source prfixe dans le fichier properties
	 * @param pDataSource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param file objet FileVO contenant le fichier
	 * @param indice indice de l'objet dans la base.
	 * @param pSql pointe du le fichier properties donnant la requete sql  executer
	 * @param blName on stock ou pas le nom du fichier
	 * @param override on permet l'crasement d'un enregistrement existant
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @version 2.3
	 * @date 14/11/2006
	 * @deprecated voir la mthode uploadFileInSGBD prenant un Vector  la place d'Object
	 */
	public void uploadFileInSGBD(String source, String pDataSource, FileVO file, Object indice, String pSql, boolean blName, boolean override) 
		throws FileWrapperException,JDBCWrapperException{

    	Vector<Object> vecteur = new Vector<Object>();
    	
    	// Indice dans la base
    	vecteur.add(indice);
    	boolean datasource = false;
    	if (pDataSource!=null)
    		datasource=true;
    	
    	uploadFileInSGBD(source,datasource,file,vecteur,blName,override);
	}

	/**
	 * Tlcharge un fichier du client vers une SGBD sur le serveur 
	 * @param source prfixe dans le fichier properties
	 * @param pDataSource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param file objet FileVO contenant le fichier
	 * @param params paramtres servant de cl d'insertion dans la base.
	 * @param blName on stock ou pas le nom du fichier
	 * @param override on permet l'crasement d'un enregistrement existant
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @version 2.5
	 * @date 14/11/2006
	 */	
	public void uploadFileInSGBD(String source, boolean pDataSource, FileVO file, Vector<Object> params, boolean blName, boolean override) 
	throws FileWrapperException,JDBCWrapperException{
		logger.info(THIS_CLASS + "uploadFileInSGBD");
	
		uploadSGBD(source,pDataSource,file.getFluxDonnees(),params, (String)file.getFichier(),blName,override);
	}
	
	
	/**
	 * Tlcharge un fichier du client vers une SGBD sur le serveur 
	 * @param source prfixe dans le fichier properties
	 * @param pDataSource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param in flux vers le fichier
	 * @param params paramtres servant de cl d'insertion dans la base.
	 * @param fileName nom du fichier
	 * @param blName on stock ou pas le nom du fichier
	 * @param override on permet l'crasement d'un enregistrement existant
	 * <ul>
	 * <li>FileWrapper.TYPE_ECRASEMENT_FICHIER    : L'crasement est autoris</li>
	 * <li>FileWrapper.TYPE_ECRASEMENT_IMPOSSIBLE : L'crasement n'est pas autoris</li>
	 * </ul>
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 */	
	private void uploadSGBD(String source, boolean pDataSource, InputStream in, Vector<Object> params, String fileName, boolean blName, boolean override)throws FileWrapperException,JDBCWrapperException{	
		String sql=null;
		if (override)
			sql = getSqlRequest(source,blName,TYPE_UPDATE);
		else
			sql = getSqlRequest(source,blName,TYPE_INSERT);
				
		JDBCWrapper jdbc = getJDBCWrapper(source,pDataSource);
		
	    ByteArrayOutputStream by = new ByteArrayOutputStream();
		byte[] responseBody=new byte[8192];
	    int length;
	    try{
		    while ((length = in.read(responseBody,0,8192)) != -1){
				by.write(responseBody,0,length);
		    } 
	    } catch (IOException e){
	    	jdbc.close();
			jdbc=null;
			throw new FileWrapperException("Erreur.FileWrapper.uploadSGBD : "+e.getMessage(),e);	    	
	    }
	
		// Positionne le nom et le binaire
	    if (!override){
		    if (blName)
				params.addElement(fileName);
			params.addElement(by);
	    } else {
		    if (blName)
				params.add(0,fileName);
			params.add(0,by);	    	
	    }
			    
		int ret = jdbc.execute(sql, JDBCWrapper.TYPE_REQUETE_UPDATE,params);
		if (ret==0){
			jdbc.close();
			jdbc=null;
			throw new FileWrapperException("Erreur.FileWrapper.uploadSGBD : Fichier non insr en base");
		}
		
		jdbc.close();
		jdbc=null;
	}
	
	
	
	/**
	 * Tlcharge un fichier depuis le serveur (fichier) vers le client 
	 * @param source nom du prfixe dans le fichier porperties
	 * @param name Nom du fichier sur le serveur
	 * @return FileVO valoris
	 * @throws FileWrapperException
	 */
	public FileVO downloadFileFromFile(String source,String name) throws FileWrapperException{
		logger.info(THIS_CLASS + "downloadFileFromFile");
		String pPath=null;
		
		try{
			pPath = _configServer.getString(source+".path");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.downloadFileFromFile : ligne manquante dans le fichier properties "+source+".path");
		}

		// Lecture du fichier depuis le disque
		InputStream inp = readFile(pPath,name);
		
		// Envoie du fichier au client
		return downloadFile(inp,name);
	}
	
	/**
	 * Tlcharge un fichier depuis une SGBD vers le client
	 * @param source nom du prfixe dans le fichier porperties
	 * @param pDatasource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param params Les paramtres tels qu'ils sont pass au JDBCWrapper dans la mthode execute.
	 * @return FileVO valoris
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @deprecated
	 */
	public FileVO downloadFileFromSGBD(String source, String pDataSource, Vector params)
	throws FileWrapperException, JDBCWrapperException{
		boolean datasource=false;
		if (pDataSource!=null)
    		datasource=true;
		
		return downloadFileFromSGBD(source,datasource,params);
	}
	
	
	/**
	 * Tlcharge un fichier depuis une SGBD vers le client
	 * @param source nom du prfixe dans le fichier porperties
	 * @param pDatasource dfinit si on prend une datasource diffrente de la datasource par dfaut 
	 * @param params Les paramtres tels qu'ils sont pass au JDBCWrapper dans la mthode execute.
	 * @return FileVO valoris
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 */
	public FileVO downloadFileFromSGBD(String source, boolean pDataSource, Vector params)
		throws FileWrapperException, JDBCWrapperException{
	
		logger.info(THIS_CLASS + "downloadFileFromSGBD");
		String nomBlob=null;
		String nomFichier=null;
		
		try{
			nomBlob = _configServer.getString(source+".lobname");
		} catch(Exception e){
			throw new FileWrapperException("Erreur.FileWrapper.downloadFileFromSGBD : ligne manquante dans le fichier properties ");
		}

		try{
			nomFichier = _configServer.getString(source+".filename");
		} catch(Exception e){
		}

		String sql = getSqlRequest(source,true,TYPE_SELECT);
		
		JDBCWrapper jdbc = getJDBCWrapper(source,pDataSource);
		jdbc.execute(sql,JDBCWrapper.TYPE_REQUETE_QUERY,params);
		
		String nomFi=null;
		InputStream inp=null;

		try{
			while (jdbc.next()) {
				try{
					if (nomFichier!=null && !nomFichier.equals(""))
						nomFi = jdbc.getString(nomFichier);
				}catch (JDBCWrapperException e){
					//Ne rien faire, le nom ne figure pas dans la requete
				}
				inp = jdbc.getBlob(nomBlob);
				break;
			}
		} catch (JDBCWrapperException e){
			jdbc.close();
			jdbc=null;
			throw new FileWrapperException("Erreur.FileWrapper.uploadFileInSGBD : Fichier indisponible : "+e.getMessage(),e);
		} 
		jdbc.close();
		jdbc=null;
		
		return downloadFile(inp,nomFi);
	}

	/**
	 * Efface un fichier sur une SGBD
	 * @param source nom du prfixe dans le fichier porperties
	 * @param pDatasource nom de variable porperties dsignant la datasource. Paramettre pouvant tre null 
	 * @param params Les paramtres tels qu'ils sont pass au JDBCWrapper dans la mthode execute.
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 * @deprecated
	 */
	public void deleteFileFromSGBD(String source, String pDataSource, Vector params)
	throws FileWrapperException, JDBCWrapperException{
		boolean datasource=false;
		if (pDataSource!=null)
    		datasource=true;
		deleteFileFromSGBD(source, datasource,params);
	}
	
	/**
	 * Efface un fichier sur une SGBD
	 * @param source nom du prfixe dans le fichier porperties
	 * @param pDatasource dfinit si on prend une datasource diffrente de la datasource par dfaut
	 * @param params Les paramtres tels qu'ils sont pass au JDBCWrapper dans la mthode execute.
	 * @throws FileWrapperException
	 * @throws JDBCWrapperException
	 */
	public void deleteFileFromSGBD(String source, boolean pDataSource, Vector params)
		throws FileWrapperException, JDBCWrapperException{
	
		logger.info(THIS_CLASS + "downloadFileFromSGBD");
		
		String sql=getSqlRequest(source,false,TYPE_DELETE);
		
		JDBCWrapper jdbc = getJDBCWrapper(source,pDataSource);

		jdbc.execute(sql,JDBCWrapper.TYPE_REQUETE_UPDATE,params);
			
		jdbc.close();
		jdbc=null;
	}
}

