/*
 * Copyright ou  ou Copr. Ministre des Affaires Etrangres, Aubay
 * SA, (21/06/07)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.technical.jdbc;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Vector;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.log4j.Logger;

import acube.framework.exception.ConfigurationException;
import acube.framework.exception.JDBCWrapperException;
import acube.framework.technical.Configuration;
import acube.framework.technical.ConfigurationManager;
import acube.framework.technical.log.BuildLogger;
import acube.framework.technical.log.Chrono;
import acube.framework.technical.log.level.Performance;

/**
 * Classe d'encapsulation des accs  la base de donnes via l'API
 * JDBC. Drivers recommands :
 * <ul>
 * <li> Pour Oracle 8 : classes12.jar et nls_charset12.jar est utilis
 * pour les STRUCT et COLLECTION
 * <li> Pour Oracle 9 et 10 : ojdbc14.jar et nls_charset12.jar est
 * utilis pour les STRUCT et COLLECTION
 * <li> Pour MSSQL : jtds-1.2.jar
 * <li> Gestion de pool : common-pool-1.3
 * </ul>
 * 
 * <img src="../../../../images/acube.gif"></img>
 * 
 * @version 1.0
 * @memberof JDBCWrapper
 */
public class JDBCWrapper {

    /** <code>logger</code>: Definition du logger */
    protected static final  Logger logger = BuildLogger.getLogger();

    /**
     * <code>THIS_CLASS</code>: Dfinition de la classe utilise
     * par le logger
     */
    private static final String THIS_CLASS =
            "[acube.framework.technical.JDBCWrapper]";

    /**
     * Cl  utiliser en cas de base unique (compatibilit avec
     * anciennes version)
     */
    protected static final String BASE_UNIQUE = "baseUnique";

    /**
     * <code>configuration</code> : gestionnaire de la configuration
     * charge
     */
    protected static java.util.Hashtable<String, DataSource> dataSources = null;

    /**
     * Configuration : Donnees specifiques au serveur.
     */
    protected static Configuration _configServer;

    /**
     * Recordset contenant les donnes rcupres par la requte
     */
    private ResultSet _rset;

    /**
     * Statement
     */
    private PreparedStatement _prep;

    /**
     * CallableStatement
     */
    private CallableStatement _call;

    /**
     * Connexion  la base de donnes
     */
    private Connection _connection;

    /**
     * Valeur de l'autocommit
     */
    private boolean _autoCommit = true;

    /**
     * Si true alors JDBCWrapper a dj une connexion vers la base
     * ouverte
     */
    private boolean _isConnected = false;

    /**
     * Nom Jndi de la DataSource  utiliser
     */
    protected String _dataSourceName;

    /**
     * Type de datasource true : datasource serveur d'app via JNDI
     * false : datasource dans server.properties
     */
    protected boolean _dataSourceType;

    /**
     * variable indiquant l'tat de la transaction
     */
    protected boolean inTransaction;

    /**
     * Mutex permettant de proteger la Hashtable de datasources
     */
    protected static Object mutexDataSource = new Object();

    /**
     * <code>TYPE_REQUETE_QUERY</code> appel d'une methode
     * executeQuery() sur le statement qui valorise un ResultSet qui
     * sera accede par la suite via une methode du JDBCWrapper : en
     * consequence cette fonctionnalite renvoie toujours un int de
     * valeur 0 pour signifier que tout c'est bien deroule.
     */
    public static final int TYPE_REQUETE_QUERY = 0;

    /**
     * <code>TYPE_REQUETE_UPDATE</code> appel d'une methode
     * executeUpdate() sur le statement qui renvoie un int
     * correspondant au nombre de lignes impactees par la requete : il
     * s'agit de requetes de type update, delete ou insert.
     */
    public static final int TYPE_REQUETE_UPDATE = 1;

    /**
     * <code>TYPE_REQUETE_EXECUTE</code> appel d'une methode
     * execute() sur le statement qui renvoie un boolean : celui-ci
     * est converti pour le retour de la faon suivante : true -> 1 et
     * false -> 0.
     */
    public static final int TYPE_REQUETE_EXECUTE = 2;

    /**
     * <code>TYPE_LOB_CLOB</code> selectionne la mise en base d'un
     * CLOB pour la mthode saveOracleLob
     */
    public static final int TYPE_LOB_CLOB = 1;

    /**
     * <code>TYPE_LOB_BLOB</code> selectionne la mise en base d'un
     * BLOB pour la mthode saveOracleLob
     */
    public static final int TYPE_LOB_BLOB = 2;

    /**
     * <code>TYPE_DATE_DATE</code> selectionne le format Date comme
     * retour de getDate()
     */
    public static final String TYPE_DATE_DATE = "date";

    /**
     * <code>TYPE_DATE_TIMESTAMP</code> selectionne le format
     * Timestamp comme retour de getDate()
     */
    public static final String TYPE_DATE_TIMESTAMP = "timestamp";

    /**
     * <code>outParamProc</code> Vecteur en retour contenant les
     * elements de type OUT et IN_OUT passes en parametre des
     * procedure stockees.
     */
    private Vector<Object> outParamProc = null;

    /**
     * <code>hTypes</code> Table de hashage contenant les
     * identifiants des diffrents types.
     */
    private static java.util.Hashtable<String, Object> hTypes = null;

    private ArrayList<ResultSet> outCursorList = null;

    /**
     * Constructeur par defaut de l'objet <code>JDBCWrapper</code>.
     */
    public JDBCWrapper() {
        this(null, false);
    }

    /**
     * Constructeur de l'objet <code>JDBCWrapper</code>.
     * @param dataSourceName dataSourceName
     */
    public JDBCWrapper(
            String dataSourceName) {

        this(dataSourceName, true);
    }

    /**
     * Constructeur de l'objet <code>JDBCWrapper</code>. Permet de
     * spcifier le nom et le type de datasource
     *
     * @param dataSourceName dataSourceName
     * @param type type
     */
    public JDBCWrapper(
            String dataSourceName, boolean type) {

        this.initDataSource(
            dataSourceName, type);
    }

    /**
     * Initialisation des datasources non standard
     * 
     */
    public void initOtherDataSource() {
        // implement in oracle. pourquoi 
        //ce n'est pas une redfinition de mthode?
    }

    /**
     * Initialisation du pool de datasource ainsi que le nom et le
     * type de la datasource
     * 
     * @param dataSourceName dataSourceName
     * @param type type
     */
    public void initDataSource(
            String dataSourceName, boolean type) {

        // Initialisation de la Hashtable contenant les pools de
        // connexion
        if (dataSources == null) {
            dataSources = new java.util.Hashtable<String, DataSource>();
        }

        this.initOtherDataSource();

        if (hTypes == null) {
            initTypes();
        }

        _dataSourceType = type;

        // Dfinition du nom JNDI de la ressource  utiliser
        if (dataSourceName != null && dataSourceName.compareTo("") != 0) {
            _dataSourceName = dataSourceName;
        } else {
            _dataSourceName = BASE_UNIQUE;
        }

        // Rcupration des fichiers de proprits.
        _configServer = ConfigurationManager.getInstance("server");
    }

    /**
     * @param outCursorList
     *            outCursorList  dfinir.
     */
    void setOutCursorList(
            ArrayList<ResultSet> outCursorList) {

        this.outCursorList = outCursorList;
    }

    /**
     * @return Renvoie outCursorList.
     */
    ArrayList getOutCursorList() {

        return this.outCursorList;
    }

    /**
     * @return Renvoie dataSources.
     */
    static java.util.Hashtable getDataSources() {

        return JDBCWrapper.dataSources;
    }

    /**
     * @param dataSources
     *            dataSources  dfinir.
     */
    static void setDataSources(
            java.util.Hashtable<String, DataSource> dataSources) {

        JDBCWrapper.dataSources = dataSources;
    }

    /**
     * @return Renvoie _call.
     */
    protected CallableStatement get_call() {

        return this._call;
    }

    /**
     * @param _call
     *            _call  dfinir.
     */
    protected void set_call(
            CallableStatement _call) {

        this._call = _call;
    }

    /**
     * @return Renvoie _prep.
     */
    protected PreparedStatement get_prep() {

        return this._prep;
    }

    /**
     * @param _prep
     *            _prep  dfinir.
     */
    protected void set_prep(
            PreparedStatement _prep) {

        this._prep = _prep;
    }

    /**
     * @return Renvoie _rset.
     */
    protected ResultSet get_rset() {

        return this._rset;
    }

    /**
     * @param _rset
     *            _rset  dfinir.
     */
    protected void set_rset(
            ResultSet _rset) {

        this._rset = _rset;
    }

    /**
     * @return Renvoie inTransaction.
     */
    protected boolean isInTransaction() {

        return this.inTransaction;
    }

    /**
     * @param inTransaction
     *            inTransaction  dfinir.
     */
    protected void setInTransaction(
            boolean inTransaction) {

        this.inTransaction = inTransaction;
    }

    /**
     * @return Renvoie _isConnected.
     */
    protected boolean is_isConnected() {

        return this._isConnected;
    }

    /**
     * @param connected
     *            _isConnected  dfinir.
     */
    protected void set_isConnected(
            boolean connected) {

        this._isConnected = connected;
    }

    /**
     * @return Renvoie _dataSourceName.
     */
    protected String get_dataSourceName() {

        return this._dataSourceName;
    }

    /**
     * @return Renvoie _configServer.
     */
    protected static Configuration get_configServer() {

        return JDBCWrapper._configServer;
    }

    /**
     * @return Renvoie logger.
     */
    protected static Logger getLogger() {

        return JDBCWrapper.logger;
    }

    /**
     * @param outParamProc
     *            outParamProc  dfinir.
     */
    void setOutParamProc(
            Vector<Object> outParamProc) {

        this.outParamProc = outParamProc;
    }

    /**
     * Cette mthode permet de prparer l'excution d'une requte en
     * fermant ventuellement le PreparedStatement et/ou le RecordSet
     * pralablement ouverts.
     * 
     * @throws JDBCWrapperException JDBCWrapperException
     */
    private void beforeExecute() throws JDBCWrapperException {

        boolean error = false;

        try {
            // Fermeture du RecordSet.
            if (_rset != null) {
                _rset.close();
                _rset = null;
            }
        } catch (Exception e) {
            error = true;
            _rset = null;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.beforeExecute : "
                        + e.getMessage(), e);
        }

        try {
            // Fermeture du PreparedStatement
            closePreparedStatement();
        } catch (Exception e) {
            error = true;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.beforeExecute : "
                        + e.getMessage(), e);
        }

        try {
            // Fermeture du CallableStatement.
            closeCallableStatement();
        } catch (Exception e) {
            error = true;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.beforeExecute : "
                        + e.getMessage(), e);
        }

        if (error) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.beforeExecute : Problme pour raliser le before execute");
        }
    }

    /**
     * Cloture des connextions non standard.
     * @throws SQLException SQLException
     */
    public void closeOtherCnx() throws SQLException {
        // implement in oracle. pourquoi 
        //ce n'est pas une redfinition de mthode?
    }

    /**
     * Ferme la connexion  la base de donnes ainsi que les ventuels
     * objets affrents (ResultSet et PreparedStatement). Ici nous
     * avons des actions spcifiques dans le cas d'une transaction.
     * 
     * @throws JDBCWrapperException
     *                Si problme  la fermeture de la connection
     */

    public void close() throws JDBCWrapperException {

        boolean error = false;

        try {
            // Fermeture du ResultSet.
            if (_rset != null) {
                _rset.close();
                _rset = null;
            }
        } catch (SQLException ex) {
            _rset = null;
            error = true;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.close : " + ex.getMessage(),
                ex);
        }

        try {
            // Fermeture du PreparedStatement
            closePreparedStatement();
        } catch (Exception e) {
            error = true;
            logger
                    .warn(
                        THIS_CLASS + "Warning.JDBCWrapper.close : "
                                + e.getMessage(), e);
        }

        try {
            // Fermeture du CallableStatement.
            closeCallableStatement();
        } catch (Exception e) {
            error = true;
            logger
                    .warn(
                        THIS_CLASS + "Warning.JDBCWrapper.close : "
                                + e.getMessage(), e);
        }

        try {
            // Fermeture de la connexion.
            // gestion de transaction
            // attention : ne pas appeler de mthode sur le
            // TransactionManger ici, en particulier inTransaction()
            // car le close() est aussi appel par le thread du
            // Garbage Collector lors de la finalisation du
            // JDBCWrapper
            if (inTransaction) {
                // dans une transaction applicative, la connection est
                // ferme en fin de transaction
            } else {
                if (_connection != null && !_connection.isClosed()) {
                    _connection.close();
                    _connection = null;
                } else {
                    _connection = null;
                }
            }
        } catch (Exception e) {
            _connection = null;
            error = true;
            logger
                    .warn(
                        THIS_CLASS + "Warning.JDBCWrapper.close : "
                                + e.getMessage(), e);
        }

        try {
            this.closeOtherCnx();
        } catch (SQLException ex) {
            error = true;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.close : " + ex.getMessage(),
                ex);
        }

        _isConnected = false;

        if (error) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.close : Problme de fermeture de la connexion");
        }
    }

    /**
     * Ferme le PreparedStatement
     * 
     * @exception JDBCWrapperException
     *                Si problme  la fermeture du PreparedStatement
     * @version 2.3
     * @date 14/11/2006
     */
    private void closePreparedStatement() throws JDBCWrapperException {

        try {

            // Fermeture du preparedStatement.
            if (_prep != null) {
                _prep.close();
                _prep = null;
            }

        } catch (Exception e) {
            _prep = null;
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.closePreparedStatement : "
                            + e.getMessage(), e);
        }
    }

    /**
     * Ferme le CallableStatement
     * 
     * @param call
     *            CallableStatement  fermer.
     * @exception JDBCWrapperException
     *                Si problme  la fermeture du CallableStatement
     */
    private void closeCallableStatement() throws JDBCWrapperException {

        boolean error = false;

        if (outCursorList != null) {
            ResultSet rsFromCursor = null;
            try {
                for (int i = 0; i < outCursorList.size(); i++) {
                    rsFromCursor = (ResultSet) outCursorList.get(i);
                    if (rsFromCursor != null) rsFromCursor.close();
                    rsFromCursor = null;
                }
                outCursorList = null;
            } catch (Exception e) {
                if (!error) {
                    error = true;
                    logger
                            .warn(
                                THIS_CLASS
                                        + "Warning.JDBCWrapper.closeCallableStatement : "
                                        + e.getMessage(), e);
                }
                rsFromCursor = null;
            }
        }

        try {
            // Fermeture du CallableStatement.
            if (_call != null) {
                _call.close();
                _call = null;
            }

        } catch (Exception e) {
            _call = null;
            error = true;
            logger.warn(
                THIS_CLASS + "Warning.JDBCWrapper.closeCallableStatement : "
                        + e.getMessage(), e);
        }

        if (error)
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.closeCallableStatement : Problme de fermeture de la connexion");
    }

    /**
     * Commit manuel. Dans le cas d'un transaction rien n'est fait.
     * 
     * @exception JDBCWrapperException
     *                dans le cas d'erreur lors de l'xcution de la
     *                requte (erreur SQL par ex.).
     * 
     */
    public void commit() throws JDBCWrapperException {

        logger.info(THIS_CLASS + "commit");

        try {
            // gestion des transactions
            if (inTransaction) {
                // dans une transaction applicative, le commit doit
                // tre effectu au niveau de la transaction
            } else {
                if (!getConnection().isClosed()){
                    getConnection().commit();
                } else {
                    throw new JDBCWrapperException(
                            "Erreur.JDBCWrapper.commit : Tentative de commit sur une connexion ferme");
                }
            }
        } catch (JDBCWrapperException e) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.commit : " + e.getMessage(), e);
        } catch (Exception ex) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.commit : " + ex.getMessage(), ex);
        }
    }

    /**
     * Ralise un Rollback. Dans le cas d'une transaction rien n'est
     * fait.
     * 
     * @exception JDBCWrapperException
     *                dans le cas d'erreur lors de l'xcution de la
     *                requte (erreur SQL par ex.).
     * 
     */
    public void rollback() throws JDBCWrapperException {

        logger.info(THIS_CLASS + "rollback");

        try {
            // gestion des transactions
            // dans une transaction applicative, le rollback doit
            // tre effectu au niveau de la transaction
            if (!inTransaction) {
                if (!getConnection().isClosed()) getConnection().rollback();
                else {
                    throw new JDBCWrapperException(
                            "Erreur.JDBCWrapper.commit : Tentative de Rollback sur une connexion ferme");
                }
            }
        } catch (JDBCWrapperException e) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.rollback : " + e.getMessage(), e);
        } catch (Exception ex) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.rollback : " + ex.getMessage(), ex);
        }
    }

    /**
     * Execute une requete
     * 
     * @param sql
     *            la requete SQL a executer
     * @param type
     *            le type de requete : TYPE_REQUETE_QUERY ou
     *            TYPE_REQUETE_UPDATE ou TYPE_REQUETE_EXECUTE
     * @return int resultat depend du type
     *         <ul>
     *         <li>TYPE_REQUETE_EXECUTE : retour 0 ou 1
     *         <li>TYPE_REQUETE_UPDATE : retourne le nombre de ligne
     *         impactes
     *         <li>TYPE_REQUETE_QUERY : retourne 0 + renseignement de
     *         _rset
     *         </ul>
     * @throws JDBCWrapperException
     *             dans le cas d'erreur lors de l'xcution de la
     *             requte (erreur SQL par ex.).
     */
    public int execute(
            String sql, int type) throws JDBCWrapperException {

        logger.info(THIS_CLASS + "execute(String sql, int type)");

        return execute(
            sql, type, new Vector());
    }

    /**
     * Execute une requete avec un paramtre en type String Remet le
     * paramtre dans un vecteur.
     * 
     * @param sql
     *            la requete SQL a executer
     * @param type
     *            le type de requete : TYPE_REQUETE_QUERY ou
     *            TYPE_REQUETE_UPDATE ou TYPE_REQUETE_EXECUTE
     * @param param
     *            un parametre de type String
     * @return int resultat depend du type de la requte.
     *         <ul>
     *         <li>TYPE_REQUETE_EXECUTE : retour 0 ou 1
     *         <li>TYPE_REQUETE_UPDATE : retourne le nombre de ligne
     *         impactes
     *         <li>TYPE_REQUETE_QUERY : retourne 0 + renseignement de
     *         _rset
     *         </ul>
     * @throws JDBCWrapperException
     *             dans le cas d'erreur lors de l'xcution de la
     *             requte (erreur SQL par ex.).
     */
    public int execute(
            String sql, int type, String param) throws JDBCWrapperException {

        logger.info(THIS_CLASS + "execute(String sql, int type, String param)");

        Vector<String> params = new Vector<String>();
        params.addElement(param);

        return execute(
            sql, type, params);
    }

    /**
     * Verification du type de base et renvoie une exception en cas de
     * MSSQL
     * 
     * @throws JDBCWrapperException
     */
    void verifBaseMSSQL() throws JDBCWrapperException {

        try {
            if (_connection == null || _connection.isClosed()) {
                throw new JDBCWrapperException(
                        "Erreur.JDBCWrapper.verifBaseMSSQL : Non connect  la base");
            }

            // Verifie le type de base, dans le cas de MSSQL on envoie
            // un exception
            java.sql.DatabaseMetaData md = _connection.getMetaData();
            if (md.getDatabaseProductName().indexOf(
                "SQL Server") != 0) {
                throw new JDBCWrapperException(
                        "Erreur.JDBCWrapper.verifBaseMSSQL : Timestamp pas gr dans le Wrapper pour MSSQL");
            }
        } catch (SQLException e) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.verifBaseMSSQL : Problme de rcupration des infos sur la base",e);
        }
    }

    /**
     * Prise en compte de l'absence de la mthode getParameterMetaData
     * dans le preparedstatement Oracle
     * 
     * @param prep
     *            preparedstatment correspondant  la requete
     * @param index
     *            index dans la table
     * @throws SQLException
     */
    public void setNull(
            PreparedStatement prep, int index) throws SQLException {

        /* on essaie avec une mthode normale */
        try {
            prep.setNull(
                index, prep.getParameterMetaData().getParameterType(
                    index));
        } catch (Exception e) {
            /*
             * en cas d'chec on donne  la basse de donnes un type
             * convertibe en tout type
             */
            prep.setNull(
                index, Types.VARCHAR);
        }
    }

    /**
     * Execute une requete avec un vecteur de paramtres
     * 
     * @param sql
     *            la requete SQL a executer
     * @param type
     *            le type de requete : TYPE_REQUETE_QUERY ou
     *            TYPE_REQUETE_UPDATE ou TYPE_REQUETE_EXECUTE
     * @param params
     *            une liste de parametres sous forme de Vector <table
     *            border=1>
     *            <tr>
     *            <td></td>
     *            <td>ORACLE</td>
     *            <td>SQL SERVER</td>
     *            </tr>
     *            <tr>
     *            <td>setLong</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>ByteArrayOutputStream</td>
     *            <td>O</td>
     *            <td>O (BLOB)</td>
     *            </tr>
     *            <tr>
     *            <td>setBoolean</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setString</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBigDecimal</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setDouble</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setInt</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBigDecimal</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setFloat</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setShort</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setTimestamp</td>
     *            <td>O</td>
     *            <td>voir setDate</td>
     *            </tr>
     *            <tr>
     *            <td>setDate</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setString</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            </table>
     * 
     * Dans le cas d'un passage d'un paramettre  null dans le vecteur
     * de paramtres il faut passer par l'objet ObjectNull et
     * renseigner correctement le type avec les contantes de l'objet
     * Types.
     * 
     * @return int resultat depend du type de la requte.
     *         <ul>
     *         <li>TYPE_REQUETE_EXECUTE : retour 0 ou 1
     *         <li>TYPE_REQUETE_UPDATE : retourne le nombre de ligne
     *         impactes
     *         <li>TYPE_REQUETE_QUERY : retourne 0 + renseignement de
     *         _rset
     *         </ul>
     * @throws JDBCWrapperException
     *             dans le cas d'erreur lors de l'xcution de la
     *             requte (erreur SQL par ex.).
     */
    public int execute(
            String sql, int type, Vector params) throws JDBCWrapperException {

        // Demarrage du Timer pour la mesure de PERF
        Chrono chrono = new Chrono();
        chrono.start();

        logger.info(THIS_CLASS + "execute(String sql, Vector params)");

        int resultat = 0;
        Object param = null;

        try {
            beforeExecute();

            // rcupration du preparedStatement
            _prep = getConnection().prepareStatement(
                sql);

            for (int i = 0; i < params.size(); i++) {

                param = params.elementAt(i);

                // traitement des parametres nuls
                if (param == null) {
                    // si le parametre courant est nul, on fixe le
                    // champ sql  nul
                    setNull(
                        _prep, i + 1);
                } else if (param instanceof ObjectNull) {
                    _prep.setNull(
                        i + 1, ((ObjectNull) param).getType());
                } else {
                    String className = param.getClass().getName();
                    if (className.equals("java.lang.String")) {
                        _prep.setString(
                            i + 1, (String) param);
                    } else if (className.equals("java.lang.Integer")) {
                        _prep.setInt(
                            i + 1, ((Integer) param).intValue());
                    } else if (className.equals("java.lang.Long")) {
                        _prep.setLong(
                            i + 1, ((Long) param).longValue());
                    } else if (className.equals("java.lang.Double")) {
                        _prep.setDouble(
                            i + 1, ((Double) param).doubleValue());
                    } else if (className.equals("java.util.Date")
                            || className.equals("java.sql.Date")) {
                        // _prep.setDate ne met que la partie Date de
                        // la valeur fournie
                        // en paramtre dans la base de donnes
                        // _prep.setDate(i+1, new
                        // java.sql.Date(((java.util.Date)
                        // params.elementAt(i)).getTime()));
                        _prep.setTimestamp(
                            i + 1, new java.sql.Timestamp(
                                    ((java.util.Date) param).getTime()));
                    } else if (className.equals("java.sql.Time")) {
                        _prep.setTime(
                            i + 1, (java.sql.Time) param);
                    } else if (className.equals("java.sql.Timestamp")) {

                        _prep.setTimestamp(
                            i + 1, (java.sql.Timestamp) param);
                    } else if (className
                            .equals("java.io.ByteArrayOutputStream")) {
                        // recuperation du parametre courant
                        ByteArrayOutputStream baos =
                                (ByteArrayOutputStream) param;
                        // creation d'un input stream
                        ByteArrayInputStream bais = new ByteArrayInputStream(
                                baos.toByteArray());
                        _prep.setBinaryStream(
                            i + 1, bais, baos.size());
                    } else if (className.equals("java.lang.Boolean")) {
                        _prep.setBoolean(
                            i + 1, ((Boolean) param).booleanValue());
                    } else if (className.equals("java.math.BigDecimal")) {
                        _prep.setBigDecimal(
                            i + 1, (BigDecimal) param);
                    } else if (className.equals("java.lang.Float")) {
                        _prep.setFloat(
                            i + 1, ((Float) param).floatValue());
                    } else if (className.equals("java.lang.Short")) {
                        _prep.setShort(
                            i + 1, ((Short) param).shortValue());
                    } else {
                        chrono.stop();
                        if (logger.isEnabledFor(Performance.TRACE)) {
                            logger.log(
                                Performance.TRACE, THIS_CLASS
                                        + "Temps d'excution: " + chrono.time()
                                        + " ms");
                        }

                        // La classe d'un des paramtres n'est pas
                        // traite ici
                        throw new JDBCWrapperException(
                                "Erreur.JDBCWrapper.execute.unknownParameterClass");
                    }
                }
            }

            // Envoi des traces au logger si il est actif
            if (logger.isDebugEnabled()) {
                StringBuffer sb = new StringBuffer(
                        "([Requte = " + sql + "]");
                for (int i = 0; i < params.size(); i++) {
                    sb.append(", [P");
                    sb.append(i);
                    sb.append("= ");
                    if (params.elementAt(i) == null) sb.append("null");
                    else sb.append(params.elementAt(
                        i).toString().trim());
                    sb.append("]");
                }
                sb.append(").");
                logger.info(THIS_CLASS + "requte SQL : " + sb);
            }

            // retour suivant le type de requte
            if (type == TYPE_REQUETE_EXECUTE) {
                if (_prep.execute()) resultat = 1;
            } else if (type == TYPE_REQUETE_UPDATE) {
                resultat = _prep.executeUpdate();
            } else if (type == TYPE_REQUETE_QUERY) {
                _rset = _prep.executeQuery();
            } else {
                chrono.stop();
                if (logger.isEnabledFor(Performance.TRACE)) {
                    logger.log(
                        Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                                + chrono.time() + " ms");
                }

                // Le type de requete n'est pas connu
                throw new JDBCWrapperException(
                        "Erreur.JDBCWrapper.execute.unknownTypeRequete");
            }

        } catch (SQLException se) {
            chrono.stop();
            if (logger.isEnabledFor(Performance.TRACE)) {
                logger.log(
                    Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                            + chrono.time() + " ms");
            }

            // En cas de perte de connexion  la base, il faut forcer
            // la reconstitution du pool de connexion
            dataSources.remove(_dataSourceName);
            close();
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.execute = " + se.getMessage(), se);
        } catch (Exception e) {
            chrono.stop();
            if (logger.isEnabledFor(Performance.TRACE)) {
                logger.log(
                    Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                            + chrono.time() + " ms");
            }

            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.execute = " + e.getMessage(), e);
        }

        chrono.stop();
        if (logger.isEnabledFor(Performance.TRACE)) {
            logger.log(
                Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                        + chrono.time() + " ms");
        }
        return resultat;
    }

    /**
     * Traitement des parametres non standard dans l'execute procedure
     * 
     * @param _call
     * @param paramProc
     * @param i
     * @return un entier indiquant si le traitement du paramtre
     *         transmis a t fait
     * @throws SQLException
     * @throws Exception
     */
    public int traitementParamNonStandard(
            CallableStatement _call, ProcedureParameter paramProc, int i)
        throws SQLException, Exception {

        return 0;
    }

    /**
     * Traitement des retours non standard
     * 
     * @param paramProc
     * @param outParamProc
     * @param i
     * @return un entier indiquant si le traitement du paramtre
     *         transmis a t fait
     * @throws SQLException
     * @throws Exception
     */
    public int traitementRetourNonStandard(
            ProcedureParameter paramProc, Vector<Object> outParamProc, int i)
        throws SQLException, Exception {

        return 0;
    }

    /**
     * Execute une procedure
     * 
     * @param sql
     *            la requete SQL a executer
     * @param type
     *            le type de requete : TYPE_REQUETE_QUERY ou
     *            TYPE_REQUETE_UPDATE ou TYPE_REQUETE_EXECUTE
     * @param params
     *            une liste de parametres sous forme de Vector <table
     *            border=1>
     *            <tr>
     *            <td></td>
     *            <td>ORACLE</td>
     *            <td>SQL SERVER</td>
     *            </tr>
     *            <tr>
     *            <td>setLong</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBinaryStream</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBoolean</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setString</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBigDecimal</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setDouble</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setInt</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setBigDecimal</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setFloat</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setShort</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>setTimestamp</td>
     *            <td>O</td>
     *            <td>x</td>
     *            </tr>
     *            <tr>
     *            <td>setString</td>
     *            <td>O</td>
     *            <td>O</td>
     *            </tr>
     *            <tr>
     *            <td>STRUCT</td>
     *            <td>O</td>
     *            <td>x</td>
     *            </tr>
     *            <tr>
     *            <td>COLLECTION</td>
     *            <td>O</td>
     *            <td>x</td>
     *            </tr>
     *            </table>
     * @return int resultat depend du type de requte.
     *         <ul>
     *         <li>TYPE_REQUETE_EXECUTE : retour 0 ou 1
     *         <li>TYPE_REQUETE_UPDATE : retourne le nombre de ligne
     *         impactes
     *         <li>TYPE_REQUETE_QUERY : retourne 0 + renseignement de
     *         _rset
     *         </ul>
     * 
     * Les rsultats sont rcuprables uniquement au travers des
     * mthodes getXxxx(int index)
     * 
     * @throws JDBCWrapperException
     *             dans le cas d'erreur lors de l'xcution de la
     *             requte (erreur SQL par ex.).
     */
    public int executeProcedure(
            String sql, int type, Vector<Object> params)
        throws JDBCWrapperException {

        // Demarrage du Timer pour la mesure de PERF
        Chrono chrono = new Chrono();
        chrono.start();

        logger.info(THIS_CLASS
                + "executeProcedure(String sql, int type, Vector params)");

        int resultat = 0;
        ProcedureParameter paramProc = null;
        String className = null;
        outParamProc = new Vector<Object>();

        try {
            beforeExecute();

            _call = getConnection().prepareCall(
                sql);

            for (int i = 0; i < params.size(); i++) {

                paramProc = (ProcedureParameter) params.elementAt(i);

                // traitement des parametres nuls.
                if (paramProc.getParam() == null
                        && paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_STANDARD) {
                    // si le parametre courant est nul, on fixe le
                    // champ sql  nul
                    if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                        _call.setNull(
                            i + 1, decodeClassName(paramProc.getClassName()));
                    } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                        _call.registerOutParameter(
                            i + 1, decodeClassName(paramProc.getClassName()));
                    } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                        _call.setNull(
                            i + 1, decodeClassName(paramProc.getClassName()));
                        _call.registerOutParameter(
                            i + 1, decodeClassName(paramProc.getClassName()));
                    } else {
                        chrono.stop();
                        if (logger.isEnabledFor(Performance.TRACE)) {
                            logger.log(
                                Performance.TRACE, THIS_CLASS
                                        + "Temps d'excution: " + chrono.time()
                                        + " ms");
                        }

                        throw new JDBCWrapperException(
                                "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                    }
                } else if (traitementParamNonStandard(
                    _call, paramProc, i) == 1) {
                    // traitement des types de paramtres standards.
                } else {
                    className = paramProc.getParam().getClass().getName();

                    if (className.equals("java.lang.String")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setString(
                                i + 1, (String) paramProc.getParam());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.VARCHAR);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setString(
                                i + 1, (String) paramProc.getParam());
                            _call.registerOutParameter(
                                i + 1, Types.VARCHAR);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Integer")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setInt(
                                i + 1, ((Integer) paramProc.getParam())
                                        .intValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.INTEGER);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setInt(
                                i + 1, ((Integer) paramProc.getParam())
                                        .intValue());
                            _call.registerOutParameter(
                                i + 1, Types.INTEGER);
                        } else {
                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Long")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setLong(
                                i + 1, ((Long) paramProc.getParam())
                                        .longValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.NUMERIC);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setLong(
                                i + 1, ((Long) paramProc.getParam())
                                        .longValue());
                            _call.registerOutParameter(
                                i + 1, Types.NUMERIC);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Double")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setDouble(
                                i + 1, ((Double) paramProc.getParam())
                                        .doubleValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.DOUBLE);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setDouble(
                                i + 1, ((Double) paramProc.getParam())
                                        .doubleValue());
                            _call.registerOutParameter(
                                i + 1, Types.DOUBLE);
                        } else {
                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.util.Date")
                            || className.equals("java.sql.Date")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setTimestamp(
                                i + 1, new java.sql.Timestamp(
                                        ((java.util.Date) paramProc.getParam())
                                                .getTime()));
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.TIMESTAMP);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setTimestamp(
                                i + 1, new java.sql.Timestamp(
                                        ((java.util.Date) paramProc.getParam())
                                                .getTime()));
                            _call.registerOutParameter(
                                i + 1, Types.TIMESTAMP);
                        } else {
                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.sql.Time")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setTime(
                                i + 1, (java.sql.Time) paramProc.getParam());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.TIME);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setTime(
                                i + 1, (java.sql.Time) paramProc.getParam());
                            _call.registerOutParameter(
                                i + 1, Types.TIME);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.sql.Timestamp")) {
                        // Verifie le type de base, dans le cas de
                        // MSSQL on envoie un exception
                        verifBaseMSSQL();

                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setTimestamp(
                                i + 1, (java.sql.Timestamp) paramProc
                                        .getParam());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.TIMESTAMP);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setTimestamp(
                                i + 1, (java.sql.Timestamp) paramProc
                                        .getParam());
                            _call.registerOutParameter(
                                i + 1, Types.TIMESTAMP);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className
                            .equals("java.io.ByteArrayOutputStream")) {
                        // recuperation du parametre courant
                        ByteArrayOutputStream baos =
                                (ByteArrayOutputStream) paramProc.getParam();
                        // creation d'un input stream
                        ByteArrayInputStream bais = new ByteArrayInputStream(
                                baos.toByteArray());
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setBinaryStream(
                                i + 1, bais, baos.size());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.VARBINARY);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setBinaryStream(
                                i + 1, bais, baos.size());
                            _call.registerOutParameter(
                                i + 1, Types.VARBINARY);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Boolean")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setBoolean(
                                i + 1, ((Boolean) paramProc.getParam())
                                        .booleanValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.BOOLEAN);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setBoolean(
                                i + 1, ((Boolean) paramProc.getParam())
                                        .booleanValue());
                            _call.registerOutParameter(
                                i + 1, Types.BOOLEAN);
                        } else {
                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.math.BigDecimal")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setBigDecimal(
                                i + 1, (BigDecimal) paramProc.getParam());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.DECIMAL);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setBigDecimal(
                                i + 1, (BigDecimal) paramProc.getParam());
                            _call.registerOutParameter(
                                i + 1, Types.DECIMAL);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Float")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setFloat(
                                i + 1, ((Float) paramProc.getParam())
                                        .floatValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.FLOAT);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setFloat(
                                i + 1, ((Float) paramProc.getParam())
                                        .floatValue());
                            _call.registerOutParameter(
                                i + 1, Types.FLOAT);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else if (className.equals("java.lang.Short")) {
                        if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
                            _call.setShort(
                                i + 1, ((Short) paramProc.getParam())
                                        .shortValue());
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
                            _call.registerOutParameter(
                                i + 1, Types.SMALLINT);
                        } else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
                            _call.setShort(
                                i + 1, ((Short) paramProc.getParam())
                                        .shortValue());
                            _call.registerOutParameter(
                                i + 1, Types.SMALLINT);
                        } else {
                            chrono.stop();
                            if (logger.isEnabledFor(Performance.TRACE)) {
                                logger.log(
                                    Performance.TRACE, THIS_CLASS
                                            + "Temps d'excution: "
                                            + chrono.time() + " ms");
                            }

                            throw new JDBCWrapperException(
                                    "Erreur.JDBCWrapper.executeProcedure.unknownParameterType, usage : IN, OUT or IN_OUT");
                        }
                    } else {
                        // La classe d'un des paramtres n'est pas
                        // traite ici
                        chrono.stop();
                        if (logger.isEnabledFor(Performance.TRACE)) {
                            logger.log(
                                Performance.TRACE, THIS_CLASS
                                        + "Temps d'excution: " + chrono.time()
                                        + " ms");
                        }

                        throw new JDBCWrapperException(
                                "Erreur.JDBCWrapper.executeProcedure.unknownParameterClass");
                    }
                }
            }

            if (logger.isDebugEnabled()) {
                StringBuffer sb = new StringBuffer(
                        "([Requte = " + sql + "]");
                for (int i = 0; i < params.size(); i++) {
                    sb.append(", [P");
                    sb.append(i);
                    sb.append("= ");
                    if (((ProcedureParameter) params.elementAt(i)).getParam() == null) sb
                            .append("null");
                    else sb.append(((ProcedureParameter) params.elementAt(i))
                            .getParam().toString());
                    sb.append("]");
                }
                sb.append(").");
                logger.info(THIS_CLASS + "requte SQL : " + sb);
            }

            // prparation du retour suivant le type de requte
            if (type == TYPE_REQUETE_EXECUTE) {
                if (_call.execute()) resultat = 1;
            } else {
                // Le type de requete n'est pas connu
                chrono.stop();
                if (logger.isEnabledFor(Performance.TRACE)) {
                    logger.log(
                        Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                                + chrono.time() + " ms");
                }

                throw new JDBCWrapperException(
                        "Erreur.JDBCWrapper.executeProcedure.unknownTypeRequete");
            }

            className = null;

            for (int i = 0; i < params.size(); i++) {

                paramProc = (ProcedureParameter) params.elementAt(i);

                if (logger.isDebugEnabled()) {
                    StringBuffer sb =
                            new StringBuffer(
                                    "paramProc - lment numro " + i
                                            + ", paramProc.getType() = "
                                            + paramProc.getType()
                                            + ", paramProc.getTypeSQL() = "
                                            + paramProc.getTypeSQL()
                                            + ", paramProc.getClassName() = "
                                            + paramProc.getClassName()
                                            + ", paramProc.getStructName() = "
                                            + paramProc.getStructName());
                    if (paramProc.getParam() == null) {
                        sb.append(", paramProc.getParam() = " + null);
                    } else {
                        sb
                                .append(", paramProc.getParam().getClass().getName() = "
                                        + paramProc.getParam().getClass()
                                                .getName());
                    }
                    logger.info(THIS_CLASS + sb);
                }

                // Ce paramtre est forcment de type OUT
                if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_CURSOR) {
                    ResultSet rsFromCursor = null;
                    try {
                        rsFromCursor = (ResultSet) getObject(i + 1);
                    } catch (Exception e) {
                        // gestion du cas o le curseur n'a pas t
                        // ouvert dans la procdure
                        // ou bien ferm prmaturment : l'erreur
                        // gnre est REF cursor non valide
                        rsFromCursor = getConnection().prepareStatement(
                            "select * from DUAL").executeQuery();
                        logger.info(THIS_CLASS
                                + "rsFromCursor.getFetchSize() = "
                                + rsFromCursor.getFetchSize());
                        while (rsFromCursor.next())
                            logger.info(THIS_CLASS + "rsFromCursor.next()");
                    }
                    if (outCursorList == null)
                        outCursorList = new ArrayList<ResultSet>();
                    outCursorList.add(rsFromCursor);
                    outParamProc.addElement(rsFromCursor);
                } else if (paramProc.getType() == ProcedureParameter.TYPE_OUT
                        || paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {

                    if (paramProc.getParam() == null) {
                        className = paramProc.getClassName();
                    } else {
                        className = paramProc.getParam().getClass().getName();
                    }
                    if (traitementRetourNonStandard(
                        paramProc, outParamProc, i) == 1) {
                        // traitement des types de paramtres
                        // standards.
                    } else if (className.equals("java.lang.String")) {
                        outParamProc.addElement(getString(i + 1));
                    } else if (className.equals("java.lang.Integer")) {
                        outParamProc.addElement(Integer.valueOf(getInt(i + 1)));
                    } else if (className.equals("java.lang.Long")) {
                        outParamProc.addElement(Long.valueOf(getLong(i + 1)));
                    } else if (className.equals("java.lang.Double")) {
                        outParamProc.addElement(new Double(
                                getDouble(i + 1)));
                    } else if (className.equals("java.util.Date")) {
                        outParamProc.addElement(getDate(i + 1));
                    } else if (className.equals("java.sql.Time")) {
                        outParamProc.addElement(getDate(i + 1));
                    } else if (className.equals("java.sql.Timestamp")) {
                        outParamProc.addElement(getDate(i + 1));
                    } else if (className
                            .equals("java.io.ByteArrayOutputStream")) {
                        outParamProc
                                .addElement((ByteArrayOutputStream) getObject(i + 1));
                    } else if (className.equals("java.lang.Boolean")) {
                        outParamProc.addElement(getObject(i + 1));
                    } else if (className.equals("java.math.BigDecimal")) {
                        outParamProc.addElement(getBigDecimal(i + 1));
                    } else if (className.equals("java.lang.Float")) {
                        outParamProc.addElement(new Float(
                                getFloat(i + 1)));
                    } else if (className.equals("java.lang.Short")) {
                        outParamProc.addElement(Short.valueOf(getShort(i + 1)));
                    } else {
                        // La classe d'un des paramtres n'est pas
                        // traite ici
                        chrono.stop();
                        if (logger.isEnabledFor(Performance.TRACE)) {
                            logger.log(
                                Performance.TRACE, THIS_CLASS
                                        + "Temps d'excution: " + chrono.time()
                                        + " ms");
                        }

                        throw new JDBCWrapperException(
                                "Erreur.JDBCWrapper.executeProcedure.unknownParameterClass");
                    }
                }
            }

        } catch (SQLException se) {
            chrono.stop();
            if (logger.isEnabledFor(Performance.TRACE)) {
                logger.log(
                    Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                            + chrono.time() + " ms");
            }

            // En cas de perte de connexion  la base, il faut forcer
            // la reconstitution du pool de connexion
            dataSources.remove(_dataSourceName);
            close();
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.executeProcedure : " + se.getMessage()
                            + " ", se);
        } catch (Exception e) {
            chrono.stop();
            if (logger.isEnabledFor(Performance.TRACE)) {
                logger.log(
                    Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                            + chrono.time() + " ms");
            }

            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.executeProcedure : " + e.getMessage(),
                    e);
        }

        chrono.stop();
        if (logger.isEnabledFor(Performance.TRACE)) {
            logger.log(
                Performance.TRACE, THIS_CLASS + "Temps d'excution: "
                        + chrono.time() + " ms");
        }

        return resultat;
    }

    /**
     * Retourne le type dpendant du nom d'une class contenu dans la
     * table da hashage
     * 
     * @param className
     *            Nom de la class
     * @return Type de la class
     * @throws JDBCWrapperException
     */
    protected int decodeClassName(
            String className) throws JDBCWrapperException {

        Object objectClassName = hTypes.get(className);
        if (objectClassName != null) {
            return ((Integer) objectClassName).intValue();
        } else {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.decodeClassName.unknownParameterClass");
        }
    }

    /**
     * Initialisation de la table de hashage faisant la correspondance
     * entre le nom d'une classe et sont type.
     */
    protected void initTypes() {

        hTypes = ObjectNull.getAllValues();
    }

    /**
     * Methode excute avant de dcharger l'objet JDBCWrapper de la
     * mmoire. Ferme la connection  la BD si ce n'est pas dj fait.
     * La mthode finalize n'etant pas contrle, il ne faut pas
     * compter dessus pour fermer la connexion  la BD et librer
     * cette ressource. Aussi la mthode close doit tre appele
     * explicitement sur les objets JDBCWrapper dans la clause
     * finally.
     * 
     */
    protected void finalize() throws Throwable {

        if ((!inTransaction) && (_connection != null)) {
            logger.warn(THIS_CLASS
                    + "finalize() : la connexion n'a pas t libre");
        }

        close();
        super.finalize();
    }

    /**
     * Rcupration d'une connexion  la base de donnes. Ici nous
     * avons des actions spcifiques dans le cas d'une transaction.
     * 
     * @return une connexion  la base de donnes
     */
    Connection getConnection() throws Exception {

        if (logger.isDebugEnabled()) {
            logger.info(THIS_CLASS + "getConnection() - _connection = "
                    + _connection);
        }

        if (_connection == null) {

            try {
                // gestion de transaction
                if (TransactionManager.getInstance().inTransaction()) {
                    // on est  l'intrieur d'une transaction
                    // on initialise un boolean car on ne pas appeler
                    // la mthode inTransaction() partout,
                    // notamment dans la mthode close() car cette
                    // dernire est aussi appele par le thread du
                    // Garbage Collector lors de la finalisation du
                    // JDBCWrapper
                    inTransaction = true;
                    // une connexion a t-elle et tablie?
                    _connection =
                            TransactionManager.getInstance().getConnection();
                    if (_connection == null) {
                        // non, on la rcupre
                        _connection = getDataSource().getConnection();
                        // on l'attache au thread courant
                        TransactionManager.getInstance().attacheConnection(
                            _connection);// on attache la
                        // connextion au thread
                    }
                } else {
                    // le wrapper n'est pas appel dans une
                    // transaction applicative
                    inTransaction = false;
                    _connection = getDataSource().getConnection();
                    _connection.setAutoCommit(_autoCommit);
                }
                _isConnected = true;

            } catch (Exception e) {
                _isConnected = false;
                throw new JDBCWrapperException(
                        "Erreur.JDBCWrapper.getConnection : " + e.getMessage(),
                        e);
            }
        }

        //
        if (TransactionManager.getInstance().inTransaction())
            TransactionManager.getInstance().setIsUse(
                Boolean.TRUE);
        return _connection;
    }

    /**
     * Permet de rcuprer une datasource dans le fichier de
     * properties
     * 
     * @param name
     * @return la datasource cre  partir des informations lu dans le
     *         fichier de properties
     * @throws Exception
     */
    private DataSource getDataSourceInProperties(
            String name) throws Exception {

        // Dfinition du driver JDBC
        String driver = _configServer.getString(name + ".jdbc.driver");

        // Utilisateur de la base de donnes
        String user = _configServer.getString(name + ".jdbc.user");

        // Son password
        String pw = _configServer.getString(name + ".jdbc.password");

        // URL JDBC de connexion
        String connectURI = _configServer.getString(name + ".jdbc.URL");

        // Dfinition du nombre max de connexion simultan
        Integer maxPoolSize = Integer.valueOf(
                _configServer.getString(name + ".jdbc.maxpoolsize"));

        // Dfinition du temps maximum d'attente
        Long maxWait = Long.valueOf(
                _configServer.getString(name + ".jdbc.maxwait"));

        String tableValidation = null;
        // Table a utiliser pour la mthode setValidationQuery
        try {
            tableValidation =
                    _configServer.getString(name + ".jdbc.validating");
        } catch (ConfigurationException e) {
        }

        if (logger.isDebugEnabled()) {
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.driver = " + driver);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.user = " + user);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.password = " + pw);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.URL = " + connectURI);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.maxpoolsize = " + maxPoolSize);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.maxwait = " + maxWait);
            logger.info(THIS_CLASS + "getDataSource() - " + name
                    + ".jdbc.validating = " + tableValidation);
        }

        BasicDataSource bds = new BasicDataSource();
        bds.setDriverClassName(driver);
        bds.setUsername(user);
        bds.setPassword(pw);
        bds.setMaxWait(maxWait.longValue());
        bds.setMaxActive(maxPoolSize.intValue());

        if (tableValidation != null && !tableValidation.equals("")) {
            bds.setValidationQuery("SELECT 1 FROM " + tableValidation);
            bds.setTestOnBorrow(true);
            bds.setTestOnReturn(false);
        }

        bds.setUrl(connectURI);

        return bds;
    }

    /**
     * Rcupration d'une des sources de donnes mises en cache pour
     * amliorer les performances.
     * 
     * @return objet DataSource correspondant au dataSourceName
     */
    protected DataSource getDataSource() throws JDBCWrapperException {

        DataSource ds = null;

        synchronized (mutexDataSource) {
            ds = (BasicDataSource) dataSources.get(_dataSourceName);
        }

        if (logger.isDebugEnabled()) {
            logger.info(THIS_CLASS + "getDataSource() - _dataSourceName = "
                    + _dataSourceName);
        }

        if (ds == null) {

            // Plusieurs threads peuvent accder  cette fonction
            // simultanment
            synchronized (mutexDataSource) {

                try {

                    if (BASE_UNIQUE.equals(_dataSourceName)) {
                        // Pour garder la compatibilit avec les
                        // anciennes versions
                        ds = this.getDataSourceInProperties("JDBCWrapper");
                    } else {
                        if (!_dataSourceType) {
                            ds =
                                    this
                                            .getDataSourceInProperties(_dataSourceName);
                        } else {
                            // rcupration de la source de donne
                            Context initCtx = new InitialContext();

                            if (logger.isDebugEnabled()) {
                                logger.info(THIS_CLASS
                                        + "getDataSource() - initCtx = "
                                        + initCtx);
                            }

                            ds =
                                    (BasicDataSource) initCtx
                                            .lookup(_dataSourceName);
                        }
                    }

                } catch (Exception e) {
                    throw new JDBCWrapperException(
                            "Erreur.JDBCWrapper.getDataSource : "
                                    + e.getMessage(), e);
                }

                if (ds == null) {
                    throw new JDBCWrapperException(
                            "Erreur.JDBCWrapper.getDataSource : Erreur de cration de DataSource");
                }

                // Ajout du pool de connexion dans la table des
                // DataSources
                dataSources.put(
                    _dataSourceName, ds);
            }
        }
        return ds;
    }

    /**
     * Rcupration de la valeur d'un champ de type Date, resultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ en String
     * @return valeur du champ en objet Date
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public java.util.Date getDate(
            String champ) throws JDBCWrapperException {

        String dateFormat;
        try {
            dateFormat = _configServer.getString("JDBCWrapper.date.type");
        } catch (ConfigurationException e) {
            dateFormat = JDBCWrapper.TYPE_DATE_TIMESTAMP;
        }

        try {
            // TODO --- A Revoir pour tre globaliser sur les
            // datasource;
            java.util.Date timestamp = _rset.getTimestamp(champ);
            if ((timestamp == null)
                    || (JDBCWrapper.TYPE_DATE_TIMESTAMP.equals(dateFormat))) return timestamp;
            else return new java.util.Date(
                    timestamp.getTime());
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type Date, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public java.util.Date getDate(
            int index) throws JDBCWrapperException {

        String dateFormat;
        try {
            dateFormat = _configServer.getString("JDBCWrapper.date.type");
        } catch (ConfigurationException e) {
            dateFormat = JDBCWrapper.TYPE_DATE_TIMESTAMP;
        }

        try {
            // TODO --- A Revoir pour tre globaliser sur les
            // datasource;
            java.util.Date timestamp;
            if (_call != null) timestamp = _call.getTimestamp(index);
            else timestamp = _rset.getTimestamp(index);

            if ((timestamp == null)
                    || (JDBCWrapper.TYPE_DATE_TIMESTAMP.equals(dateFormat))) return timestamp;
            else return new java.util.Date(
                    timestamp.getTime());
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type double, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public double getDouble(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getDouble(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type double, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public double getDouble(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getDouble(index);
            else return _rset.getDouble(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type double, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Double getObjectDouble(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Double")) return (Double) o;
            else {
                if (o == null) return null;
                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type double, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Double getObjectDouble(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Double")) return (Double) o;
            else {
                if (o == null) return null;
                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type int, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public int getInt(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getInt(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type int, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public int getInt(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getInt(index);
            else return _rset.getInt(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * 
     * @param big
     * @return
     * @throws JDBCWrapperException
     */
    private Integer bigDecimalToInteger(
            BigDecimal big) throws JDBCWrapperException {

        try {
            return big.intValueExact();
        } catch (ArithmeticException e) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.bigDecimalToInteger : Convertion impossible",
                    e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type int, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Integer getObjectInt(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o == null) return null;

            if (o.getClass().getName().equals(
                "java.lang.Integer")) {
                return (Integer) o;
            } else if (o.getClass().getName().equals(
                "java.lang.BigDecimal")) {
                return bigDecimalToInteger((BigDecimal) o);
            } else {
                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type int, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Integer getObjectInt(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Integer")) return (Integer) o;
            else {
                if (o == null) return null;
                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type long, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public long getLong(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getLong(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type long, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public long getLong(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getLong(index);
            else return _rset.getLong(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type Long, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return Objet Long ou null
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Long getObjectLong(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Long")) return (Long) o;
            else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type Long, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return Objet Long ou null
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Long getObjectLong(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Long")) return (Long) o;
            else {
                if (o == null) return null;
                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type Object (par
     * exemple Objet Oracle), rsultant d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Object getObject(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getObject(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type Object (par
     * exemple Objet Oracle), resultant d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Object getObject(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getObject(index);
            else return _rset.getObject(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message + e.getMessage(), e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type String, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public String getString(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getString(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type String, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ par String.ValueOf() ou null
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public String getString(
            int index) throws JDBCWrapperException {

        Object resultObject = null;
        String result = null;
        try {
            if (this._call != null && this._call.getMoreResults()) {
                resultObject = this._call.getObject(index);
            } else if (this._rset != null) {
                resultObject = this._rset.getObject(index);
            }
        } catch (SQLException e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
        if (resultObject != null) {
            result = String.valueOf(resultObject);
        }
        return result;
    }

    /**
     * Rcupration de la valeur d'un champ de type String, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public String getStringNotNull(
            String champ) throws JDBCWrapperException {

        try {
            String stVal = this._rset.getString(champ);
            if (stVal == null) {
                stVal = "";
            }
            return stVal;

        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type String, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public String getStringNotNull(
            int index) throws JDBCWrapperException {

        try {
            String stVal;

            if (_call != null) stVal = _call.getString(index);
            else stVal = _rset.getString(index);

            if (stVal == null) {
                stVal = "";
            }
            return stVal;

        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration d'un flux de donnes binaires (utilis pour
     * rcuprer les images stockes en base)
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public InputStream getBinaryStream(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getBinaryStream(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration d'un flux de donnes binaires (utilis pour
     * rcuprer les images stockes en base)
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public InputStream getBlob(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getBinaryStream(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public boolean getBoolean(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getBoolean(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public boolean getBoolean(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getBoolean(index);
            else return _rset.getBoolean(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Boolean getObjectBoolean(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Boolean")) return (Boolean) o;
            else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Boolean getObjectBoolean(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Boolean")) return (Boolean) o;
            else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type BigDecimal,
     * rsultant d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public BigDecimal getBigDecimal(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getBigDecimal(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type BigDecimal,
     * resultant d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public BigDecimal getBigDecimal(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getBigDecimal(index);
            else return _rset.getBigDecimal(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public BigDecimal getObjectBigDecimal(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null && o.getClass().getName().equals(
                "java.lang.BigDecimal")) return (BigDecimal) o;
            else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type boolean, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public BigDecimal getObjectBigDecimal(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.BigDecimal")) return (BigDecimal) o;
            else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type float, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public float getFloat(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getFloat(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type float, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public float getFloat(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getFloat(index);
            else return _rset.getFloat(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type float, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Float getObjectFloat(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null) {
                Float f;
                if (o.getClass().getName().equals(
                    "java.lang.Double")) f = new Float(
                        ((Double) o).floatValue());
                else if (o.getClass().getName().equals(
                    "java.lang.Float")) f = (Float) o;
                else throw new Exception();

                return (Float) f;
            } else {
                return null;
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type float, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Float getObjectFloat(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Double")) {
                Float f = new Float(
                        ((Double) o).floatValue());
                return (Float) f;
            } else {
                if (o == null) return null;
                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type short, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public short getShort(
            String champ) throws JDBCWrapperException {

        try {
            return _rset.getShort(champ);
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type short, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public short getShort(
            int index) throws JDBCWrapperException {

        try {
            if (_call != null) return _call.getShort(index);
            else return _rset.getShort(index);
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type short, rsultant
     * d'une requte SQL
     * 
     * @param champ
     *            nom du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Short getObjectShort(
            String champ) throws JDBCWrapperException {

        try {
            Object o = _rset.getObject(champ);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Integer")) {
                Short s = Short.valueOf(((Integer) o).shortValue());
                return (Short) s;
            } else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message = MessageFormat.format(
                "Erreur.JDBCWrapper.LectureChamp", new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Rcupration de la valeur d'un champ de type short, resultant
     * d'une requte SQL
     * 
     * @param index
     *            index du champ
     * @return valeur du champ
     * @exception JDBCWrapperException
     *                Si erreur lors de la lecture du resultset,
     *                rsultat de la requte SQL
     */
    public Short getObjectShort(
            int index) throws JDBCWrapperException {

        try {
            Object o = getObject(index);
            if (o != null && o.getClass().getName().equals(
                "java.lang.Integer")) {
                Short s = Short.valueOf(((Integer) o).shortValue());
                return (Short) s;
            } else {
                if (o == null) return null;

                throw new Exception();
            }
        } catch (Exception e) {
            String message =
                    MessageFormat.format(
                        "Erreur.JDBCWrapper.LectureChamp", new Object[]{Integer
                                .valueOf(index)});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Envoie une trace au logger si il est actif
     * 
     * @param sql
     *            Requte SQL  envoyer au Logger si il est actif
     */
    protected void lobLogger(
            String sql) {

        if (logger.isDebugEnabled()) {
            StringBuffer sb = new StringBuffer(
                    "([Requte = " + sql + "]");
            logger.info(THIS_CLASS + "requte SQL : " + sb);
        }
    }

    /**
     * Charge un LOB dans la base MSSQL. Les deux vecteurs d'entre
     * doivent avoir exactement le mme nombre d'objet. Ces Objet sont
     * essentiellement des String.
     * 
     * @param table
     *            nom de la table
     * @param cle
     *            Vecteur de String contenant l'ensemble des champs
     *            ncssaire a la selection de la bonne ligne
     * @param valCle
     *            Vecteur de String contrnant les valeurs des champs
     *            pour la selection de la ligne
     * @param champ
     *            nom du champs LOB
     * @param tableau
     *            de byte contenant le LOB.
     * @param bInsert
     *            boolean autorisant ou non l'insert de la ligne
     *            contenant le blob. Permet d'updater un blob en
     *            positionnant sur false.
     * @param iTypeLob
     *            indique le type du LOB  insrer : TYPE_LOB_CLOB ou
     *            TYPE_LOB_BLOB
     * @return succes ou chec
     * @exception JDBCWrapperException
     *                Si erreur sur Vector, lors de la lecture du
     *                resultset, rsultat de la requte SQL
     */
    public boolean saveMSSQLLob(
            String table, Vector cle, Vector valCle, String champ, byte[] data,
            boolean bInsert, int iTypeLob) throws JDBCWrapperException {

        int ret;
        String cle1 = "";
        String val1 = "";
        String cleVal = "";
        Object o;
        boolean etatAutoCommit = false;
        String empty = "";

        logger
                .info(THIS_CLASS
                        + "saveMSSQLLob(String ,Vector ,Vector ,String ,byte [],boolean ,int )");

        // Vrification de la taille des deux vecteurs
        if (cle.size() != valCle.size()) {
            String message =
                    MessageFormat
                            .format(
                                "Erreur.JDBCWrapper.saveMSSQLLob : Vector de taille diffrente",
                                new Object[]{cle, valCle});
            throw new JDBCWrapperException(
                    message);
        }

        // Construction des requtes avec vrification des paramtres
        for (int i = 0; i < cle.size(); i++) {
            o = cle.get(i);
            if (o.getClass().getName().equals(
                "java.lang.String") == false) {
                String message =
                        MessageFormat
                                .format(
                                    "Erreur.JDBCWrapper.saveMSSQLLob : Vector cl contenant un Objet non String",
                                    new Object[]{o});
                throw new JDBCWrapperException(
                        message);
            }
            cle1 = cle1 + (String) o + ",";
            o = valCle.get(i);
            if (o.getClass().getName().equals(
                "java.lang.String") == false) {
                String message =
                        MessageFormat
                                .format(
                                    "Erreur.JDBCWrapper.saveMSSQLLob : Vector valeur de cl contenant un Objet non String",
                                    new Object[]{o});
                throw new JDBCWrapperException(
                        message);
            }
            val1 = val1 + (String) o + ",";
            cleVal =
                    cleVal + (String) cle.get(i) + "=" + (String) valCle.get(i);
            if (i < cle.size() - 1) cleVal = cleVal + " AND ";
        }

        try {
            // sauvegarde de l'tat de l'autocommit
            if (!inTransaction) {
                etatAutoCommit = this.is_autoCommit();
                this.set_autoCommit(false);
            }

            if (bInsert) {
                // insert de la ligne contenant le blob
                String sql1 =
                        "INSERT INTO " + table + " (" + cle1 + champ
                                + ") VALUES (" + val1 + champ + ")";
                // Envoie des traces au logger
                lobLogger(sql1);

                _prep = getConnection().prepareStatement(
                    sql1);

                InputStream is = new ByteArrayInputStream(
                        data);

                _prep.setBinaryStream(
                    1, is, data.length);

                ret = this.execute(
                    sql1, TYPE_REQUETE_UPDATE);
                if (ret == 0) {
                    String message =
                            MessageFormat
                                    .format(
                                        "Erreur.JDBCWrapper.saveMSSQLLob : Impossible d'inserer de la ligne contenant le LOB",
                                        new Object[]{cle, valCle});
                    throw new JDBCWrapperException(
                            message);
                }
            } else {
                // On reinit le blob avant d'crire dedans
                String sql3 ="UPDATE " + table + " SET " + champ + "="
                                        + empty + " WHERE " + cleVal;

                // Envoie des traces au logger
                lobLogger(sql3);

                ret = this.execute(
                    sql3, TYPE_REQUETE_UPDATE);
                if (ret == 0) {
                    String message =
                            MessageFormat
                                    .format(
                                        "Erreur.JDBCWrapper.saveMSSQLLob : Impossible d'initialiser le LOB  EMPTY",
                                        new Object[]{cle, valCle});
                    throw new JDBCWrapperException(
                            message);
                }
            }

            if (!this.inTransaction) {
                this.commit();

                // replace l'autocommit dans son tat initial
                this.set_autoCommit(etatAutoCommit);
            }

            return true;
        } catch (Exception e) {
            if (!this.inTransaction) {
                this.rollback();
            }

            String message =
                    MessageFormat.format(
                        e.getMessage() + e.getCause()
                                + "Erreur.JDBCWrapper.saveMSSQLLob",
                        new Object[]{champ});
            throw new JDBCWrapperException(
                    message, e);
        }
    }

    /**
     * Renvoie le Vecteur contenant la liste des parametres OUT et
     * IN_OUT d'une procedure stockee.
     * 
     * @return outParamProc Vector liste des parametres OUT et IN_OUT
     */
    public Vector getOutParamProc() {

        return this.outParamProc;
    }

    /**
     * Parcours du resultset (rsultat d'une requte SQL). Passe 
     * l'lment suivant.
     * 
     * @return True s'il y a un lment suivant, false sinon
     * @exception JDBCWrapperException
     *                Si erreur dans la lecture du resultset, rsultat
     *                de la requte SQL
     */
    public boolean next() throws JDBCWrapperException {

        try {
            return this._rset.next();
        } catch (Exception e) {
            throw new JDBCWrapperException(
                    "Erreur.JDBCWrapper.next : " + e.getMessage(), e);
        }
    }

    /**
     * @return Return the _autoCommit.
     */
    public boolean is_autoCommit() {

        return this._autoCommit;
    }

    /**
     * Contrle de l'autocommit. Verification si nous somme en
     * transaction. Si oui dsactivation de cette fonction.
     * 
     * @param commit
     *            The _autoCommit to set.
     */
    public void set_autoCommit(
            boolean commit) {

        // c'est une transaction?
        if (!this.inTransaction) {
            this._autoCommit = commit;
        }
    }

    /**
     * Controle si le critre formatt est un paramtre  passer 
     * JDBC
     * 
     * @param sCritereFormateSql
     *            Critre formatt par la mthode
     * @see #formatCriteria(String, String)
     * @return Indicateur de paramtre
     */
    public static boolean isValidCriteria(
            String sCritereFormateSql) {

        boolean bRetour = false;
        if (sCritereFormateSql != null && !sCritereFormateSql.equals("")) {
            if (sCritereFormateSql.indexOf("=?") != -1) {
                bRetour = true;
            }
        }
        return bRetour;
    }

    /**
     * Ajouter un critre  la requte en construction
     * 
     * @param sSql
     *            Requte en construction
     * @param sSqlCritere
     *            Critre formatt  ajouter
     * @return Requte mise  jour
     */
    public static String addCriteria(
            String sSql, String sSqlCritere) {

        String sRetour = sSql;
        if (sSql != null && !sSql.equals("") && sSqlCritere != null
                && !sSqlCritere.equals("")) {
            String sMotCle = "WHERE ";
            if (sSql.toUpperCase().indexOf(
                sMotCle) != -1) {
                sMotCle = "AND ";
            }
            sRetour = sSql + " " + sMotCle + sSqlCritere;
        }
        return sRetour;
    }

    /**
     * Formate un critre pour les requtes SQL
     * 
     * @param sNomCritere
     *            Nom du critre
     * @param sValeurCritere
     *            Valeur du critre
     * @return Element SQL  placer dans le requte
     */
    public static String formatCriteria(
            String sNomCritere, String sValeurCritere) {

        String sCritereFormate = "";
        if (sNomCritere != null) {
            if (sValeurCritere != null && !sValeurCritere.equals("")) {
                // Potitionne le nom du critre
                sCritereFormate = " " + sNomCritere;
                // Formate le selecteur de valeur
                if (sValeurCritere.indexOf("*") != -1) {
                    sCritereFormate += " like '" + sValeurCritere.replace(
                        '*', '%') + "'";
                } else {
                    sCritereFormate += " =?";
                }
            }
        }
        return sCritereFormate;
    }
}
