/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.technical.jdbc;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import acube.framework.exception.JDBCWrapperException;
import acube.framework.technical.log.BuildLogger;

/**
 * Classe permettant de grer des transactions JDBC  l'extrieur des dao.
 * Une transaction est dlimite par les appels de begin() d'une part et commit() ou rollback() d'autre part
 * Limitations : un thread ne peut commencer qu'une transaction  la fois (pas de transactions imbriques) et une transaction ne gre qu'une connexion  la fois.
 * 
 * Attention : une transaction ne peut se faire qu' l'interieur d'une mme action. A la sortie de l'action
 * la transaction est termine par un Rollback. Dans le cas ou la transaction est dj en utilisation par
 * un execute, un message est envoy en Warning dans les logs du Framework.
 * 
 * Exemple type des enchainements dans une action :
 * 
 * instance = TransactionManager.getInstance
 * instance.begin
 * 
 * Utilisation du JDBCWrapper
 * 
 * instance.commit ou instance.rollback
 * 
 * @memberof JDBCWrapper
 */
public class TransactionManager {
	/** <code>logger</code>: Definition du logger */
	protected static Logger logger = BuildLogger.getLogger();

	private static TransactionManager instance = null;

	private static ThreadLocal<Transaction> tx = new ThreadLocal<Transaction>();

	private class Transaction {

		/**
		 * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
		 * logger
		 */
		private static final String THIS_CLASS = "[acube.framework.technical.jdbc.TransactionManager]";
		
		Connection connection=null;
		Boolean isUse = new Boolean(false);
		
		public Boolean getIsUse() {
			return isUse;
		}

		public void setIsUse(Boolean isUse) {
			this.isUse = isUse;
		}

		/**
		 * Cloture d'une transaction
		 * @throws SQLException
		 */
		protected void closeTransaction() throws SQLException{
		
			if(connection!=null && !connection.isClosed()) {
				connection.close();
				connection=null;
			}else{
				connection=null;
			}
		}
		
		
		//au cas o on n'aurait pas commit la transaction lors de l'utilisation du thread
		//la connexion peut tre encore ouverte, ce qui est mal!
		protected void finalize() throws Throwable {
			if (isUse){
				logger.warn(THIS_CLASS + "Warning.Transaction : Fermeture de la transaction alors qu'elle est en utilisation");
			}
			
			try{
				if (connection!=null && !connection.isClosed())
				  connection.rollback();
			}catch(Exception e){
			}
			
			try{
				closeTransaction();
			} catch (Exception e){
				connection=null;
			}			
			super.finalize();
		}
	}
	

	/**
	 * rcupre le gestionnaire de transaction
	 * @return Une instance
	 */
	public static TransactionManager getInstance() {
		synchronized(ThreadLocal.class) {
			if (instance==null)
			instance=new TransactionManager();
		}
		
		return instance;
	}

	/**
	 * Positionne l'tat d'utilisation de la transaction.
	 * @param isUse tat  positionner.
	 */
	public void setIsUse(Boolean isUse){
		Transaction transac = tx.get();
		if(transac!=null)
			transac.setIsUse(isUse);
	}
	
	
	/**
	 * indique si le thread courant est dans une transaction
	 * @return true si le thread courant est dans une transaction (appel  begin())
	 */
	public boolean inTransaction() {
		return tx.get() != null;
	}

	/**
	 * commence une transaction et l'associe au thread courant
	 * celui-ci ne pourra plus faire d'appel  cette mthode sauf s'il termine la transaction par commit ou rollback
	 * @throws JDBCWrapperException si une transaction est dj associe au thread courant
	 */
	public  void begin() throws JDBCWrapperException{

		Transaction t = (Transaction) tx.get();
		if(t != null)
			throw new JDBCWrapperException("Erreur.TransactionMamager.begin : une transaction est dj associe au thread courant");
		tx.set(new Transaction());
	}

	/**
	 * valide une transaction en validant la connexion jdbc associe
	 * @throws JDBCWrapperException si pas de transaction associe au thread courant  ou SQLException
	 */
	public void commit() throws JDBCWrapperException{
		Transaction t = (Transaction) tx.get();
		if(t == null)
			throw new JDBCWrapperException("Erreur.TransactionMamager.commit : pas de transaction associe au thread courant");
		else {
			try{
				t.setIsUse(new Boolean(false));
				t.connection.commit();
				t.closeTransaction();
   		    } catch (SQLException se) {
				String message=null;
				try{
					t.connection.close();
				} catch (SQLException e) {
					message = new String("Erreur.TransactionMamager.rollback : Erreur de fermeture de connexion. ");
				}
				t.connection=null;
				tx.set(null);
				throw new JDBCWrapperException(message+se.getMessage());
			} 
			tx.set(null);//si on souhaite refaire un begin
		}
	}

	/**
	 * annule une transaction en annulant la connexion jdbc associe
	 * @throws JDBCWrapperException si pas de transaction associe au thread courant ou SQLException
	 */
	public void rollback() throws JDBCWrapperException{
		Transaction t = (Transaction) tx.get();
		if(t == null)
			throw new JDBCWrapperException("Erreur.TransactionMamager.rollback : pas de transaction associe au thread courant");
		else {
			if(t.connection !=null) {
				try{
					t.setIsUse(new Boolean(false));
					t.connection.rollback();
				} catch (SQLException se) {
					throw new JDBCWrapperException(se.getMessage());
				} finally {
					try{
						t.closeTransaction();
					} catch (SQLException se) {
						String message=null;
						try{
							t.connection.close();
						} catch (SQLException e) {
							message = new String("Erreur.TransactionMamager.rollback : Erreur de fermeture de connexion. ");
						}
						t.connection=null;
						tx.set(null);
						throw new JDBCWrapperException(message+se.getMessage());
					}
				}
			}
			tx.set(null);//si on souhaite refaire un begin
		}
	}

	/**
	 * rcupre la connexion jdbc associe
	 * @return la connexion associ  la transaction en cours
	 * @throws JDBCWrapperException si pas de transaction associe au thread courant
	 */
	public Connection getConnection() throws JDBCWrapperException{
		Transaction t = (Transaction) tx.get();
		if(t == null)
			throw new JDBCWrapperException("Erreur.TransactionMamager.getConnection : pas de transaction associe au thread courant");

		return t.connection;
	}

	/**
	 * attache une connexion jdbc  la transaction
	 * @param connection
	 * @throws JDBCWrapperException si pas de transaction associe au thread courant ou qu'une connexion exite dj ou SQLException
	 */
	public void attacheConnection(Connection connection) throws JDBCWrapperException{
		Transaction t = (Transaction) tx.get();
		if(t == null)
			throw new JDBCWrapperException("Erreur.TransactionMamager.attacheConnection : pas de transaction associe au thread courant");

		if(t.connection !=null) {
			throw new JDBCWrapperException("Erreur.TransactionMamager.attacheConnection : une transaction ne gre qu'une connexion  la fois");
		}
		try {
			connection.setAutoCommit(false);
		} catch (SQLException se ){
			throw new JDBCWrapperException(se.getMessage());
		}
		t.connection = connection;
	}
	
	/**
	 * Permet de refaire un begin. Sans l'appel  cette mthode le begin n'est possible qu'a la prochaine action.
	 *
	 */
	public void close(){
		tx.set(null);
	}
}
