/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.technical.jdbc.oracle;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.sql.Array;
import java.sql.CallableStatement;
import java.sql.SQLException;
import java.sql.Struct;
import java.sql.Types;
import java.text.MessageFormat;
import java.util.Vector;

import javax.naming.Context;
import javax.naming.InitialContext;

import oracle.jdbc.driver.OracleConnection;
import oracle.jdbc.driver.OracleTypes;
import oracle.jdbc.pool.OracleConnectionPoolDataSource;
import oracle.jdbc.pool.OracleDataSource;
import oracle.sql.BLOB;
import oracle.sql.CLOB;
import acube.framework.exception.JDBCWrapperException;
import acube.framework.technical.jdbc.ProcedureParameter;
import acube.framework.technical.jdbc.TransactionManager;

/**
 * Classe d'encapsulation des accs  la base de donnes via l'API JDBC
 * Drivers recommands :
 * <ul>
 * <li> Pour Oracle 8 : classes12.jar et nls_charset12.jar est utilis pour les STRUCT et COLLECTION
 * <li> Pour Oracle 9 et 10 : ojdbc14.jar et nls_charset12.jar est utilis pour les STRUCT et COLLECTION
 * <li> Pour MSSQL : jtds-1.2.jar
 * </ul> 
 * 
 * @version 1.0
 * @memberof JDBCOracleWrapper
 */
public class JDBCWrapper extends acube.framework.technical.jdbc.JDBCWrapper {
	/**
	 * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
	 * logger
	 */
	private static final String THIS_CLASS = "[acube.framework.technical.JDBCWrapperOracle]";

	/**
	 * <code>configuration</code> : gestionnaire de la configuration charge
	 */
	private static java.util.Hashtable<String,OracleDataSource> oracleDataSources = null;

	/**
	 * Connexion  la base de donnes
	 */
	private OracleConnection _oracleConnection;

	/**
	 * Initialisation des datasources non standard
	 *
	 */
	public void initOtherDataSource(){
		//	Initialisation de la Hashtable contenant les pools de connexion
		if (oracleDataSources == null) {
			oracleDataSources = new java.util.Hashtable<String,OracleDataSource>();
		}
	}

	/**
	 * Constructeur par defaut de l'objet <code>JDBCWrapper</code>.
	 */
	public JDBCWrapper() {
		initDataSource(null,false);
	}

	/**
	 * Constructeur de l'objet <code>JDBCWrapper</code>.
	 */
	public JDBCWrapper(String dataSourceName) {
		initDataSource(dataSourceName,true);
	}

	/**
	 * Constructeur de l'objet <code>JDBCWrapper</code>.
	 * Permet de spcifier le nom et le type de datasource
	 * @param dataSourceName
	 * @param type
	 */
	public JDBCWrapper(String dataSourceName, boolean type){
		initDataSource(dataSourceName,type);		
	}

	/**
	 * Cloture des connextions non standard.
	 *
	 */
	public void closeOtherCnx()throws SQLException {
		// Fermeture de la connexion Oracle (utilise dans le cas d'insertion ou d'update de STRUCT et de COLLECTION 
		// pour la cration d'un oracle.sql.StructDescriptor et d'un oracle.sql.STRUCT 
		// ou d'un oracle.sql.ArrayDescriptor et d'un oracle.sql.ARRAY).
		if (isInTransaction()) {
			//dans une transaction applicative, la connection est ferme en fin de transaction
		} else {
			if (_oracleConnection != null && !_oracleConnection.isClosed()) {
				_oracleConnection.close();
				_oracleConnection = null;
			} else {
				_oracleConnection = null;
			}
		}
	}

	/**
	 * Traitement des parametres non standard dans l'execute procedure
	 * @param _call
	 * @param paramProc
	 * @param i
	 * @return un entier indiquant si le traitement du paramtre transmis a t fait
	 * @throws SQLException
	 * @throws Exception
	 */
	public int traitementParamNonStandard(CallableStatement _call,ProcedureParameter paramProc,int i) throws SQLException,Exception{
		// traitement des types de paramtres non standards tels que les cursors 
		// qui seront renvoys sous forme de ResultSet.
		if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_CURSOR) {
			// Ce paramtre est forcment de type OUT	    	
			_call.registerOutParameter(i + 1, OracleTypes.CURSOR);
			return 1;
		}
		// traitement des types de paramtres non standards tels que les structures 
		// qui correspondent  des types dfinis en base par l'utilisateur.
		else if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_STRUCT) {
			if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
				Object[] objectValues = (Object[]) paramProc.getParam();
				oracle.sql.StructDescriptor structDesc = oracle.sql.StructDescriptor
				.createDescriptor(paramProc.getStructName()
						.trim().toUpperCase(),
						getOracleConnection());
				oracle.sql.STRUCT object = new oracle.sql.STRUCT(
						structDesc, getOracleConnection(), objectValues);
				_call.setObject(i + 1, object);
				return 1;
			} else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
				_call.registerOutParameter(i + 1, Types.STRUCT,
						paramProc.getStructName().trim().toUpperCase());
				return 1;
			} else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
				Object[] objectValues = (Object[]) paramProc.getParam();
				oracle.sql.StructDescriptor structDesc = oracle.sql.StructDescriptor
				.createDescriptor(paramProc.getStructName()
						.trim().toUpperCase(),
						getOracleConnection());
				oracle.sql.STRUCT object = new oracle.sql.STRUCT(
						structDesc, getOracleConnection(), objectValues);
				_call.setObject(i + 1, object);
				_call.registerOutParameter(i + 1, Types.STRUCT,
						paramProc.getStructName().trim().toUpperCase());
				return 1;
			} 
		}
		// traitement des types de paramtres non standards tels que les collections 
		// qui correspondent  des types dfinis en base par l'utilisateur.
		else if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_COLLECTION) {
			if (paramProc.getType() == ProcedureParameter.TYPE_IN) {
				Object[] objectValues = (Object[]) paramProc.getParam();
				oracle.sql.ArrayDescriptor structDesc = oracle.sql.ArrayDescriptor
				.createDescriptor(paramProc.getStructName()
						.trim().toUpperCase(),
						getOracleConnection());
				oracle.sql.ARRAY object = new oracle.sql.ARRAY(
						structDesc, getOracleConnection(), objectValues);
				_call.setObject(i + 1, object);
				return 1;
			} else if (paramProc.getType() == ProcedureParameter.TYPE_OUT) {
				_call.registerOutParameter(i + 1, Types.ARRAY,
						paramProc.getStructName().trim().toUpperCase());
				return 1;
			} else if (paramProc.getType() == ProcedureParameter.TYPE_IN_OUT) {
				Object[] objectValues = (Object[]) paramProc.getParam();
				oracle.sql.ArrayDescriptor structDesc = oracle.sql.ArrayDescriptor
				.createDescriptor(paramProc.getStructName()
						.trim().toUpperCase(),
						getOracleConnection());
				oracle.sql.ARRAY object = new oracle.sql.ARRAY(
						structDesc, getOracleConnection(), objectValues);
				_call.setObject(i + 1, object);
				_call.registerOutParameter(i + 1, Types.ARRAY,
						paramProc.getStructName().trim().toUpperCase());
				return 1;
			} 
		}

		return 0;
	}

	/**
	 * Traitement des retours non standard 
	 * @param paramProc
	 * @param outParamProc
	 * @param i
	 * @return un entier indiquant si le traitement du paramtre transmis a t fait
	 * @throws SQLException
	 * @throws Exception
	 */
	public int traitementRetourNonStandard(ProcedureParameter paramProc,Vector<Object> outParamProc,int i) throws SQLException,Exception{
		if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_STRUCT) {
			outParamProc.addElement(((Struct) getObject(i + 1))
					.getAttributes());
			return 1;
		} else if (paramProc.getTypeSQL() == ProcedureParameter.TYPE_SQL_COLLECTION) {
			outParamProc.addElement(((Array) getObject(i + 1))
					.getResultSet());
			return 1;
		}
		return 0;
	}


	/**
	 * Rcupration d'une connexion  la base de donnes.
	 * Ici nous avons des actions spcifiques dans le cas d'une transaction. 
	 *
	 * @return une connexion  la base de donnes (type Oracle)
	 */
	private OracleConnection getOracleConnection() throws Exception {

		if (logger.isDebugEnabled()) {
			logger.info(THIS_CLASS
					+ "getOracleConnection() - _oracleConnection = "
					+ _oracleConnection);
		}

		if (_oracleConnection == null) {

			try {
				if (TransactionManager.getInstance().inTransaction()) {
					//on est  l'intrieur d'une transaction
					// on initialise un boolean car on ne pas appeler la mthode inTransaction() partout,
					// notamment dans la mthode close() car cette dernire est aussi appele par le thread du Garbage Collector lors de la finalisation du JDBCWrapper
					setInTransaction(true);
					//une connexion a t-elle et tablie?   
					_oracleConnection = (OracleConnection) TransactionManager.getInstance().getConnection();
					if (_oracleConnection == null) {
						//non, on la rcupre 
						_oracleConnection = (OracleConnection) getOracleDataSource().getConnection();
						//on l'attache au thread courant
						TransactionManager.getInstance().attacheConnection(_oracleConnection);//on attache la connextion au thread
					}
				} else {
					_oracleConnection = (OracleConnection) getOracleDataSource()
					.getConnection();
					_oracleConnection.setAutoCommit(is_autoCommit());
					setInTransaction(false);
				}
				set_isConnected(true);

			} catch (Exception e) {
				throw new JDBCWrapperException(
						"Erreur.JDBCWrapper.getOracleConnection : "
						+ e.getMessage(), e);
			}
		}

		return _oracleConnection;
	}

	/**
	 * Rcupration d'une des sources de donnes mises en cache 
	 * pour amliorer les performances.
	 * 
	 * @return Rcupration du OracleDataSource correspondant au dataSourceName 
	 */
	private OracleDataSource getOracleDataSource() throws JDBCWrapperException {

		OracleDataSource ods = (OracleDataSource) oracleDataSources.get(get_dataSourceName());

		if (logger.isDebugEnabled()) {
			logger.info(THIS_CLASS
					+ "getOracleDataSource() - _dataSourceName = "
					+ get_dataSourceName());
		}

		if (ods == null) {

			// Plusieurs threads peuvent accder  cette fonction simultanment			
			synchronized (this) {

				try {

					if (BASE_UNIQUE.equals(get_dataSourceName())) {
						// Pour garder la compatibilit avec les anciennes versions

						// Dfinition du driver JDBC 
						String driver = get_configServer()
						.getString("JDBCWrapper.jdbc.driver");

						// Utilisateur de la base de donnes
						String user = get_configServer()
						.getString("JDBCWrapper.jdbc.user");

						// Son password
						String pw = get_configServer()
						.getString("JDBCWrapper.jdbc.password");

						// URL JDBC de connexion
						String connectURI = get_configServer()
						.getString("JDBCWrapper.jdbc.URL");

						// Dfinition du nombre max de connexion simultan
						Integer maxPoolSize = new Integer(get_configServer()
								.getString("JDBCWrapper.jdbc.maxpoolsize"));

						// Dfinition du temps maximum d'attente
						Long maxWait = new Long(get_configServer()
								.getString("JDBCWrapper.jdbc.maxwait"));

						if (logger.isDebugEnabled()) {
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.driver = "
									+ driver);
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.user = "
									+ user);
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.password = "
									+ pw);
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.URL = "
									+ connectURI);
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.maxpoolsize = "
									+ maxPoolSize);
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - JDBCWrapper.jdbc.maxwait = "
									+ maxWait);
						}

						// Positionnement du driver 
						//Class.forName(driver, true, DriverManagerConnectionFactory.class.getClassLoader());
						Class.forName(driver);

						if (logger.isDebugEnabled()) {
							logger
							.info(THIS_CLASS
									+ "getOracleDataSource() - Chargemen" +
											"t du driver OK");
						}

						OracleConnectionPoolDataSource odsp = new OracleConnectionPoolDataSource();
						odsp.setURL(connectURI);
						odsp.setUser(user);
						odsp.setPassword(pw);
						ods=odsp;
						
/*						ods = new OracleDataSource();
						ods.setURL(connectURI);
						ods.setUser(user);
						ods.setPassword(pw);
*/
					} else {
						//  rcupration de la source de donne
						Context initCtx = new InitialContext();

						if (logger.isDebugEnabled()) {
							logger.info(THIS_CLASS
									+ "getOracleDataSource() - initCtx = "
									+ initCtx);
						}

						ods = (OracleDataSource) initCtx
						.lookup(get_dataSourceName());

					}

				} catch (Exception e) {
					throw new JDBCWrapperException(
							"Erreur.JDBCWrapper.getDataSource : "
							+ e.getMessage(), e);
				}
				// Ajout du pool de connexion dans la table des DataSources
				oracleDataSources.put(get_dataSourceName(), ods);
			}
		}
		return ods;
	}

	/**
	 * Charge un LOB dans la base Oracle. 
	 * Les deux vecteurs d'entre doivent avoir exactement le mme nombre d'objet.
	 * Ces Objet sont essentiellement des String.
	 *
	 * @param table nom de la table
	 * @param cle Vecteur de String contenant l'ensemble des champs ncssaire a la selection de la bonne ligne 
	 * @param valCle Vecteur de String contrnant les valeurs des champs pour la selection de la ligne
	 * @param champ nom du champs LOB
	 * @param dataStream LOB en InputStream.
	 * @param bInsert boolean autorisant ou non l'insert de la ligne contenant le blob. Permet d'updater un blob en positionnant sur false.
	 * @param iTypeLob indique le type du LOB  insrer : TYPE_LOB_CLOB ou TYPE_LOB_BLOB 
	 * @return succes ou chec
	 * @exception JDBCWrapperException Si erreur sur Vector, lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public boolean saveOracleLob(String table, Vector cle, Vector valCle,
			String champ, InputStream dataStream, boolean bInsert, int iTypeLob)
	throws JDBCWrapperException {
		logger
		.info(THIS_CLASS
				+ "saveOracleLob(String ,Vector ,Vector ,String ,InputStream ,boolean ,int )");

		try {
			// convertion de l'InputStream en byte[]
			byte[] data = new byte[dataStream.available()];
			dataStream.read(data);

			return saveOracleLob(table, cle, valCle, champ, data, bInsert,
					iTypeLob);
		} catch (Exception e) {
			throw new JDBCWrapperException(e.getMessage(), e);
		}
	}

	/**
	 * Charge un LOB dans la base. 
	 * Les deux vecteurs d'entre doivent avoir exactement le mme nombre d'objet.
	 * Ces Objet sont essentiellement des String.
	 * 
	 * @param table nom de la table
	 * @param cle Vecteur de String contenant l'ensemble des champs ncssaires  la selection de la bonne ligne 
	 * @param valCle Vecteur de String contrnant les valeurs des champs pour la selection de la ligne
	 * @param champ nom du champs LOB
	 * @param data LOB byte[].
	 * @param bInsert boolean autorisant ou non l'insert de la ligne contenant le blob. Permet d'updater un blob en positionnant sur false.
	 * @param iTypeLob indique le type du LOB  insrer : TYPE_LOB_CLOB ou TYPE_LOB_BLOB 
	 * @return true sinon exception
	 * @exception JDBCWrapperException Si erreur sur Vecteor, lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public boolean saveOracleLob(String table, Vector cle, Vector valCle,
			String champ, byte[] data, boolean bInsert, int iTypeLob)
	throws JDBCWrapperException {
		CLOB clob;
		BLOB blob;
		int ret;
		String cle1 = new String("");
		String val1 = new String("");
		String cleVal = new String("");
		Object o;
		boolean etatAutoCommit = false;
		String empty = new String();
		OutputStream writer;

		logger
		.info(THIS_CLASS
				+ "saveOracleLob(String ,Vector ,Vector ,String ,byte [],boolean ,int )");

		// Vrification de la taille des deux vecteurs
		if (cle.size() != valCle.size()) {
			String message = MessageFormat
			.format(
					"Erreur.JDBCWrapper.saveOracleLob : Vector de taille diffrente",
					new Object[] { cle, valCle });
			throw new JDBCWrapperException(message);
		}

		// Adaptation pour les requtes suivant le type de LOB
		switch (iTypeLob) {
		case 1:
			empty = "EMPTY_CLOB()";
			break;

		case 2:
			empty = "EMPTY_BLOB()";
			break;

		default:
			empty = "null";
		break;
		}

		// Construction des requtes avec vrification des paramtres
		for (int i = 0; i < cle.size(); i++) {
			o = cle.get(i);
			if (o.getClass().getName().equals("java.lang.String") == false) {
				String message = MessageFormat
				.format(
						"Erreur.JDBCWrapper.saveOracleLob : Vector cl contenant un Objet non String",
						new Object[] { o });
				throw new JDBCWrapperException(message);
			}
			cle1 = cle1 + (String) o + ",";
			o = valCle.get(i);
			if (o.getClass().getName().equals("java.lang.String") == false) {
				String message = MessageFormat
				.format(
						"Erreur.JDBCWrapper.saveOracleLob : Vector valeur de cl contenant un Objet non String",
						new Object[] { o });
				throw new JDBCWrapperException(message);
			}
			val1 = val1 + (String) o + ",";
			cleVal = cleVal + (String) cle.get(i) + "="
			+ (String) valCle.get(i);
			if (i < cle.size() - 1)
				cleVal = cleVal + " AND ";
		}

		try {
			// c'est une transaction?
			if (!isInTransaction()) {
				// sauvegarde de l'tat de l'autocommit
				etatAutoCommit = this.is_autoCommit();
				this.set_autoCommit(false);
			}

			if (bInsert) {
				// insert de la ligne contenant le blob
				String sql1 = new String("INSERT INTO " + table + " (" + cle1
						+ champ + ") VALUES (" + val1 + empty + ")");

				// Envoie des traces au logger
				lobLogger(sql1);

				ret = this.execute(sql1, TYPE_REQUETE_UPDATE);
				if (ret == 0) {
					String message = MessageFormat
					.format(
							"Erreur.JDBCWrapper.saveOracleLob : Impossible d'inserer de la ligne contenant le LOB",
							new Object[] { cle, valCle });
					throw new JDBCWrapperException(message);
				}
			} else {
				// On reinit le blob avant d'crire dedans
				String sql3 = new String("UPDATE " + table + " SET " + champ
						+ "=" + empty + " WHERE " + cleVal);

				// Envoie des traces au logger
				lobLogger(sql3);

				ret = this.execute(sql3, TYPE_REQUETE_UPDATE);
				if (ret == 0) {
					String message = MessageFormat
					.format(
							"Erreur.JDBCWrapper.saveOracleLob : Impossible d'initialiser le LOB  EMPTY",
							new Object[] { cle, valCle });
					throw new JDBCWrapperException(message);
				}
			}

			// rcupration de la rfrence au blob dans Oracle
			String sql2 = new String("SELECT " + champ + " FROM " + table
					+ " WHERE " + cleVal); //+" FOR UPDATE");
			// Envoie des traces au logger
			lobLogger(sql2);
			ret = this.execute(sql2, TYPE_REQUETE_QUERY);

			// Insertion du blob
			if (next()) {
				switch (iTypeLob) {
				case 1:
					clob = (CLOB) get_rset().getClob(champ);
					writer = clob.getAsciiOutputStream();
					break;

				default:
				case 2:
					blob = (BLOB) get_rset().getBlob(champ);
					writer = blob.getBinaryOutputStream();
					break;
				}

				writer.write(data);
				writer.flush();
				writer.close();
			} else {
				// c'est une transaction?
				if (!isInTransaction()) {
					this.rollback();
				}
				String message = MessageFormat
				.format(
						"Erreur.JDBCWrapper.saveOracleLob : Impossible de rcuprer de la ligne contenant le LOB",
						new Object[] { cle, valCle });
				throw new JDBCWrapperException(message);
			}

			// c'est une transaction?
			if (!isInTransaction()) {
				this.commit();

				// replace l'autocommit dans son tat initial
				this.set_autoCommit(etatAutoCommit);
			}

			return true;
		} catch (Exception e) {
			// c'est une transaction?
			if (!isInTransaction()) {
				this.rollback();
			}

			String message = MessageFormat.format(e.getMessage() + e.getCause()
					+ "Erreur.JDBCWrapper.saveOracleLob",
					new Object[] { champ });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type CLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public String loadOracleClobToString(String champ)
	throws JDBCWrapperException {
		CLOB clob;
		Reader read;

		try {
			int length;
			String res = new String();
			char[] buffer = new char[200000];

			clob = (CLOB) get_rset().getClob(champ);
			read = clob.getCharacterStream();

			while ((length = read.read(buffer)) != -1) {
				res += String.valueOf(buffer, 0, length);
			}
			read.close();

			return res;
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp CLOB",
					new Object[] { champ });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type CLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public String loadOracleClobToString(int index) throws JDBCWrapperException {
		CLOB clob;
		Reader read;

		try {
			int length;
			String res = new String();
			char[] buffer = new char[200000];

			clob = (CLOB) get_rset().getClob(index);
			read = clob.getCharacterStream();

			while ((length = read.read(buffer)) != -1) {
				res += String.valueOf(buffer, 0, length);
			}
			read.close();

			return res;
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp CLOB",
					new Object[] { new Integer(index) });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type CLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public InputStream loadOracleClobToStream(String champ)
	throws JDBCWrapperException {
		CLOB clob;

		try {
			clob = (CLOB) get_rset().getClob(champ);
			return clob.getAsciiStream();
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp CLOB",
					new Object[] { champ });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type CLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public InputStream loadOracleClobToStream(int index)
	throws JDBCWrapperException {
		CLOB clob;

		try {
			clob = (CLOB) get_rset().getClob(index);
			return clob.getAsciiStream();
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp CLOB",
					new Object[] { new Integer(index) });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type BLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public byte[] loadOracleBlobToByte(String champ)
	throws JDBCWrapperException {
		BLOB blob;

		try {
			blob = (BLOB) get_rset().getBlob(champ);
			return blob.getBytes(1, (int) blob.length());
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp", new Object[] { champ });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type BLOB, resultant d'une requte SQL
	 *
	 * @param index	index du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public byte[] loadOracleBlobToBye(int index) throws JDBCWrapperException {
		BLOB blob;

		try {
			blob = (BLOB) get_call().getBlob(index);
			return blob.getBytes(1, (int) blob.length());
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp",
					new Object[] { new Integer(index) });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type BLOB, rsultant d'une requte SQL
	 *
	 * @param champ nom du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public InputStream loadOracleBlobToStream(String champ)
	throws JDBCWrapperException {
		BLOB blob;

		try {
			blob = (BLOB) get_rset().getBlob(champ);
			return blob.getBinaryStream();
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp", new Object[] { champ });
			throw new JDBCWrapperException(message, e);
		}
	}

	/**
	 * Rcupration de la valeur d'un champ de type BLOB, resultant d'une requte SQL
	 *
	 * @param index	index du champ
	 * @return valeur du champ
	 * @exception JDBCWrapperException Si erreur lors de la lecture du resultset, rsultat de la requte SQL      
	 */
	public InputStream loadOracleBlobToStream(int index)
	throws JDBCWrapperException {
		BLOB blob;

		try {
			blob = (BLOB) get_call().getBlob(index);
			return blob.getBinaryStream();
		} catch (Exception e) {
			String message = MessageFormat.format(
					"Erreur.JDBCWrapper.LectureChamp",
					new Object[] { new Integer(index) });
			throw new JDBCWrapperException(message, e);
		}
	}
}