/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.utility;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import acube.framework.technical.Constants;

/**
 * Cette classe comprend un ensemble de mthode permettant de grer les dates aux formats Franais. 
 * 
 * @version 2.0
 * @memberof General
 */
public class DateFR {
    /**
     * <code>LONG_DATE_JJMMAAAA</code>: Taille d'une date de type JJ/MM/AAAA
     */
    private static final int LONG_DATE_JJMMAAAA = 10;

    /**
     * <code>LONG_DATE_MMAAAA</code>: Taille d'une date de type MM/AAAA
     */
    private static final int LONG_DATE_MMAAAA = 7;

    /**
     * <code>LONG_DATE_AAAA</code>: Taille d'une date de type AAAA
     */
    private static final int LONG_DATE_AAAA = 4;

    /**
     * <code>CAR_SEPARATION</code>: Emplacement du caractere de sparation
     */
    private static final int CAR_SEPARATION = 3;

	/**
	 * Convertit une date en chaine de caractre au format franais
	 * 
	 * @param date
	 *            Date  convertir
	 * @return Date franaise au format franais jj/mm/aaaa
	 */
	public static String toFRString(Date date) {
	    String sRetour = "";
	    if (date != null) {
	        Calendar cal = Calendar.getInstance();
	        cal.set(9999, 11, 31);
	        DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy");
	        sRetour = dateFormat.format(date);
	        String sCal = dateFormat.format(cal.getTime());
	        if (sCal.equals(sRetour)) {
	            sRetour = "";
	        } else {
	            sRetour = dateFormat.format(date);
	        }
	    }
	    return sRetour;
	}

	/**
	 * Convertit une date en chaine de caractre au format yyMMdd
	 * 
	 * @param date
	 *            Date  convertir
	 * @return String la date au bon format
	 */
	public static String toStandardString(Date date) {
	    String sRetour = "";
	    if (date != null) {
	        DateFormat dateFormat = new SimpleDateFormat("yyMMdd");
	        sRetour = dateFormat.format(date);
	    }
	    return sRetour;
	}

	/**
	 * Convertit une chaine de caractre reprsentant une date incomplete au
	 * format franais en date
	 * 
	 * @param sDate
	 *            Date franaise au format franais
	 * @return Date convertie
	 */
	public static Date fromIncompleteStringFR(String sDate) {
	    Date dRetour = null;
	    String sDateComplete = "";
	
	    if (sDate != null) {
	
	        int iLongDate = sDate.length();
	
	        switch (iLongDate) {
	            case LONG_DATE_JJMMAAAA:
	                sDateComplete = sDate;
	                break;
	            case LONG_DATE_MMAAAA:
	                sDateComplete = "01/" + sDate;
	                break;
	            case LONG_DATE_AAAA:
	                sDateComplete = "01/01/" + sDate;
	                break;
	            default:
	                sDateComplete = sDate;
	                break;
	        }
	
	        try {
	            SimpleDateFormat sdfInput = new SimpleDateFormat("dd/MM/yyyy");
	            dRetour = sdfInput.parse(sDateComplete);
	        } catch (ParseException p) {
	            dRetour = null;
	        }
	    }
	    return dRetour;
	}

	/**
	 * Determine un code suivant le format de la date passe en parametre
	 * 
	 * @param sDate
	 *            Date incomplete au format franais
	 * @return code : information complete de la date: 0 => JJ/MM/AAAA 1 =>
	 *         MM/AAAA 2 => AAAA
	 *  
	 */
	public static int type(String sDate) {
	    int iRetour = 0;
	
	    if (sDate != null) {
	
	        int iLongDate = sDate.length();
	
	        switch (iLongDate) {
	            case LONG_DATE_JJMMAAAA:
	                iRetour = Constants.DATE_INCOMPLETE_JJMMAAAA;
	                break;
	            case LONG_DATE_MMAAAA:
	                iRetour = Constants.DATE_INCOMPLETE_MMAAAA;
	                break;
	            case LONG_DATE_AAAA:
	                iRetour = Constants.DATE_INCOMPLETE_AAAA;
	                break;
	            default:
	                iRetour = Constants.DATE_INCOMPLETE_JJMMAAAA;
	                break;
	        }
	    }
	    return iRetour;
	}

	/**
	 * Convertit une date incomplete en chaine de caractre au format dpendant
	 * du code pass en argument.
	 * 
	 * @param date
	 *            Date  convertir
	 * @param code
	 *            information complete de la date: 0 => JJ/MM/AAAA 1 => MM/AAAA
	 *            2 => AAAA
	 * @param carIncomplet
	 *            Chaine  utiliser pour remplacer les parties incompletes
	 * @return Date au format franais (incomplet)
	 */
	public static String toFRString(Date date, int code,
	        String carIncomplet) {
	    String sRetour = "";
	    String sFormatDate = null;
	    String sPartieIncomplete = "";
	    char carSeparateur = '/';
	
	    // Determination du caractere de sparation
	    if (carIncomplet.length() == CAR_SEPARATION) {
	        carSeparateur = carIncomplet.charAt(CAR_SEPARATION - 1);
	    }
	
	    switch (code) {
	    
	        case Constants.DATE_INCOMPLETE_JJMMAAAA:
	            sFormatDate = "dd" + carSeparateur + "MM" + carSeparateur + "yyyy";
	            break;
	
	        case Constants.DATE_INCOMPLETE_MMAAAA:
	            sFormatDate = "MM" + carSeparateur + "yyyy";
	            sPartieIncomplete = carIncomplet;
	            break;
	
	        case Constants.DATE_INCOMPLETE_AAAA:
	            sFormatDate = "yyyy";
	            sPartieIncomplete = carIncomplet + carIncomplet;
	            break;
	
	        default:
	            sFormatDate = "dd" + carSeparateur + "MM" + carSeparateur + "yyyy";
	            break;
	    }
	    if (date != null) {
	        DateFormat dateFormat = new SimpleDateFormat(sFormatDate);
	        sRetour = sPartieIncomplete + dateFormat.format(date);
	    }
	    return sRetour;
	}

	/**
	 * Convertit une chaine de caractre reprsentant une date au format
	 * franais en date
	 * 
	 * @param sDate
	 *            Date franaise au format franais
	 * @return Date convertie
	 */
	public static Date fromStringFR(String sDate) {
	    Date dRetour = null;
	    if (sDate != null) {
	        try {
	            SimpleDateFormat sdfInput = new SimpleDateFormat("dd/MM/yyyy");
	            dRetour = sdfInput.parse(sDate);
	        } catch (ParseException p) {
	            dRetour = null;
	        }
	    }
	    return dRetour;
	}
	
	/**
     * Convertit une date en chaine de caractre au format franais exemple:
     * 27/10/1998 14:31:25
     * 
     * @param date
     *            Date  convertir
     * @return Date franaise au format franais
     * @author rigal
     */
    public static String dateHeureToGMTString(Date date) {
        String sRetour = "";
        TimeZone zoneGMT = TimeZone.getTimeZone("UTC");

        if (date != null) {
            DateFormat dateFormat = new SimpleDateFormat(
                    "EEE, d MMM yyyy HH:mm:ss");
            dateFormat.setTimeZone(zoneGMT);
            sRetour = dateFormat.format(date) + " GMT";
        }
        return sRetour;
    }
    

}
