/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.utility;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItem;
import org.apache.log4j.Logger;

import acube.framework.exception.FunctionalException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Label;
import acube.framework.technical.LabelManager;
import acube.framework.technical.log.BuildLogger;
import acube.framework.vo.ErrorVO;

/**
 * Cette classe comprend l'ensemble des mthodes permettant d'accder aux donnes transmises dans l'url 
 * que ce soit en mode GET ou POST. 
 * 
 * @version 2.0
 * @memberof General
 */
public class GetRequest {
	/**
	 * <code>m_logger</code>: Definition du logger
	 */
	private static Logger logger = BuildLogger.getLogger();

	/**
	 * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
	 * logger
	 */
	private static String THIS_CLASS = "[acube.framework.utility.GetRequest]";

	
	/**
	 * Retourne la valeur d'une cl contenu dans la Map de parametres.
	 * @param request
	 * @param cle
	 * @return une chane de caractre ou le fichier correspond  la cl recherch
	 */
	private static String getValue(HttpServletRequest request,String cle) throws FunctionalException, TechnicalException {
		Map params = (Map)request.getAttribute(acube.framework.technical.Constants.PARAMS_REQUEST);
		String valeur = null;
		Object oParam = params.get(cle);
		if (oParam == null)
			return null;
		if (oParam instanceof FileItem)
			valeur = ((FileItem)oParam).getString();
		else {
			String[] contenu = (String[])oParam;
			if (contenu.length != 1) {
				LabelManager paramLib = new LabelManager();
				paramLib.addParameter("0", cle);
				logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-07", paramLib));
				throw new FunctionalException(new ErrorVO("ERR-FCT-07", cle, "", true, false, false));
			} else
				valeur = contenu[0];
		}
		return valeur;
	}	
	
	/**
	 * Recuperation d'un attribut en requete sous forme de int. Rejet
	 * TechnicalException si absence de l'objet en requete ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return int
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static int getInt(HttpServletRequest request, String cle) throws FunctionalException, TechnicalException {
		return GetRequest.getInteger(request, cle).intValue();
	}

	/**
	 * Recuperation d'un attribut en requete sous forme de String. Rejet
	 * FunctionalException si absence de l'objet en requete.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return String
	 * @throws FunctionalException exception fonctionnelle
	 * @throws TechnicalException exception technique
	 */
	public static String getString(HttpServletRequest request, String cle) throws TechnicalException, FunctionalException {
		String valeur = null;
		
		valeur = getValue(request,cle);
		if (null == valeur) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-07", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-07", cle, "",
					true, false, false));
		}
		return valeur;
	}

	/**
	 * Recuperation d'un attribut en requete sous forme de String. Rejet
	 * FunctionalException si le parametre est une chaine vide en requete.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return String
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static String getNotEmptyString(HttpServletRequest request, String cle) throws FunctionalException, TechnicalException {
		String valeur = getString(request, cle);
		if ("".equals(valeur)) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-08", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-08", cle, "",
					true, false, false));
		}
		return valeur;
	}

	/**
	 * Recuperation d'un attribut en requete sous forme de Integer. Rejet
	 * FunctionalException si absence de l'objet en requete ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return Integer
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static Integer getInteger(HttpServletRequest request, String cle) throws FunctionalException, TechnicalException {
		String valeur = null;
		Integer valeurInteger = null;
		valeur = getValue(request,cle);
		if (null == valeur) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-07", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-07", cle, "",
					true, false, false));
		}
		try {
			valeurInteger = new Integer(valeur);
		} catch (NumberFormatException nfe) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-09", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-09", cle, "",
					true, false, false));
		}
		return valeurInteger;
	}

	/**
	 * Recuperation d'un attribut en requete sous forme de long. Rejet
	 * TechnicalException si absence de l'objet en requete ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return long
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static long getLong(HttpServletRequest request, String cle) throws FunctionalException, TechnicalException {
		String valeur = null;
		Long valeurLong = null;

		valeur = getValue(request,cle);		
		if (null == valeur) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-07", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-07", cle, "",
					true, false, false));
		}
		try {
			valeurLong = new Long(valeur);
		} catch (NumberFormatException nfe) {
			LabelManager paramLib = new LabelManager();
			paramLib.addParameter("0", cle);
			logger.error(THIS_CLASS + Label.getLabel("ERR-FCT-09", paramLib));
			throw new FunctionalException(new ErrorVO("ERR-FCT-09", cle, "",
					true, false, false));
		}
		return valeurLong.longValue();
	}

}
