/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.utility;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import acube.framework.exception.FunctionalException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Label;
import acube.framework.technical.LabelManager;
import acube.framework.technical.log.BuildLogger;
import acube.framework.vo.ErrorVO;

/**
 * Cette classe comprend l'ensemble des mthodes permettant d'accder aux donnes disponible dans la session
 * 
 * @version 2.0
 * @memberof General
 */
public class GetSession {
	/**
	 * <code>m_logger</code>: Definition du logger
	 */
	private static Logger logger = BuildLogger.getLogger();

	/**
	 * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
	 * logger
	 */
	private static String THIS_CLASS = "[acube.framework.utility.GetRequest]";

	/**
	 * Recuperation d'un attribut en session sous forme de int. Rejet
	 * FunctionalException si absence de l'objet en session ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return int
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static int getInt(HttpServletRequest request,
	        String cle) throws FunctionalException, TechnicalException {
	
	    return getInteger(request, cle).intValue();
	}

	/**
	 * Recuperation d'un attribut en session sous forme de Integer. Rejet
	 * TechnicalException si absence de l'objet en session ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return Integer
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static Integer getInteger(HttpServletRequest request,
	        String cle) throws FunctionalException, TechnicalException {
	    Integer valeurInteger = null;
	    try {
	        valeurInteger = (Integer) request.getSession().getAttribute(cle);
	    } catch (NullPointerException npe) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-10", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-10", cle, npe
	                .getMessage(), true, false, false));
	    } catch (NumberFormatException nfe) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-12", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-12", cle, nfe
	                .getMessage(), true, false, false));
	    }
	
	    return valeurInteger;
	}

	/**
	 * Recuperation d'un attribut en session sous forme de long. Rejet
	 * TechnicalException si absence de l'objet en requete ou mauvais format.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return long
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException exception fonctionnelle
	 */
	public static long getLong(HttpServletRequest request,
	        String cle) throws FunctionalException, TechnicalException {
	    Long valeurLong = null;
	    try {
	        valeurLong = (Long) request.getSession().getAttribute(cle);
	    } catch (NullPointerException npe) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-10", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-10", cle, npe
	                .getMessage(), true, false, false));
	    } catch (NumberFormatException nfe) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-12", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-12", cle, nfe
	                .getMessage(), true, false, false));
	    }
	
	    return valeurLong.longValue();
	}

	/**
	 * Recuperation d'un attribut en session sous forme de String. Rejet
	 * FunctionalException si absence de l'objet en session.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return String
	 * @throws FunctionalException exception fonctionnelle si absence du parametre en session
	 * @throws TechnicalException exception technique
	 */
	public static String getString(HttpServletRequest request,
	        String cle) throws TechnicalException, FunctionalException {
	    String valeur = null;
	    valeur = (String) request.getSession().getAttribute(cle);
	    if (null == valeur) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-10", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-10", cle, "",
	                true, false, false));
	    }
	    return valeur;
	}

	/**
	 * Recuperation d'un attribut en requete sous forme de String. Rejet
	 * FunctionalException si l'objet en session est chaine vide.
	 * 
	 * @param request
	 *            HttpServletRequest requete
	 * @param cle
	 *            String cle dont on veut avoir la valeur en session
	 * @return String
	 * @throws TechnicalException exception technique
	 * @throws FunctionalException si le parametre a une valeur vide en session
	 */
	public static String getNotEmptyString(HttpServletRequest request,
	        String cle) throws FunctionalException, TechnicalException {
		
	    String valeur = getString(request, cle);
	    if ("".equals(valeur)) {
	        LabelManager paramLib = new LabelManager();
	        paramLib.addParameter("0", cle);
	        logger.error(THIS_CLASS
	                + Label.getLabel("ERR-FCT-11", paramLib));
	        throw new FunctionalException(new ErrorVO("ERR-FCT-11", cle, "",
	                true, false, false));
	    }
	    return valeur;
	}

}
