/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.utility;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;


import org.apache.log4j.Logger;

import acube.framework.exception.ConfigurationException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Configuration;
import acube.framework.technical.Constants;
import acube.framework.technical.Label;
import acube.framework.technical.LabelManager;
import acube.framework.technical.log.BuildLogger;
import acube.framework.vo.ErrorVO;



/**
 * Cette classe comprend des mthodes utilitaires partages par l'ensemble du
 * projet
 * 
 * @version 1.0
 * @memberof General
 */

public class Tools {

    /**
     * <code>m_logger</code>: Definition du logger
     */
    private static Logger logger = BuildLogger.getLogger();

    /**
     * <code>THIS_CLASS</code>: Dfinition de la classe utilise par le
     * logger
     */
    private static String THIS_CLASS = "[acube.framework.utility.Tools]";
    
    /**
     * @param value
     *            valeur  convertir
     * @return Boolean
     */
    public static Boolean intToBoolean(int value) {
        Boolean bReturn;
        if (value == 0) {
            bReturn = Boolean.TRUE;
        } else {
            bReturn = Boolean.FALSE;
        }
        return bReturn;
    }

    /**
     * @return Nouveau timestamp
     */
    public static long getTimeStamp() {
        return (new Date()).getTime();
    }

    /**
     * Permet de crypter une chaine de caracteres (utilise notamment pour
     * crypter le mot de passe)
     * 
     * @param chaineAEncrypter
     *            String chaine de caractres  encrypter
     * @return String chaine de caractre encrypte
     * @throws TechnicalException
     *             exception technique
     */
    public static String encryptString(String chaineAEncrypter)
    	throws TechnicalException {

        // objet digester permettant d'effectuer l'encryptage
        MessageDigest diggester = null;
        // chaine de caractre resultat
        String chaineEncryptee = null;
        // tableau de bytes, resultat de l'encodage
        byte[] resultat;

        try {
            // creation d'un diggester
            diggester = MessageDigest
                    .getInstance(Constants.METHODE_ENCODAGE_MDP);

            // chargement de la chaine  encrypter dans le diggester
            diggester.update(chaineAEncrypter.getBytes());

            // encodage
            resultat = diggester.digest();
            chaineEncryptee = HexadecimalConverter.convert(resultat);

        } catch (NoSuchAlgorithmException e) {
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter("0", Constants.METHODE_ENCODAGE_MDP);
            logger.error(THIS_CLASS
                    + Label.getLabel("ERR-TEC-19", paramLib));
            throw new TechnicalException(new ErrorVO("ERR-TEC-19",
                    Constants.METHODE_ENCODAGE_MDP, e.getMessage(), true,
                    false, false));

        } catch (IllegalArgumentException e) {
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter("0", Constants.METHODE_ENCODAGE_MDP);
            logger.error(THIS_CLASS
                    + Label.getLabel("ERR-TEC-19", paramLib));
            throw new TechnicalException(new ErrorVO("ERR-TEC-19",
                    Constants.METHODE_ENCODAGE_MDP, e.getMessage(), true,
                    false, false));

        }

        return chaineEncryptee;
    }

    /**
     * Permet de recuperer la valeur correspondant a une cle donne dans un
     * fichier de properties. Surcouche permettant de renvoyer une
     * TechnicalException en cas de probleme.
     * 
     * @param conf
     *            Configuration l'objet representant le fichier de conf
     * @param nomFichier
     *            string le nom du fichier de conf (uniquement pour produire un
     *            log explicite)
     * @param cle
     *            String la cl dans le fichier de properties
     * @return String la valeur trouve dans le fichier de properties
     * @throws TechnicalException
     *             exception technique
     */
    public static String obtenirValeur(Configuration conf, String nomFichier,
            String cle) throws TechnicalException {
        String valeur = null;
        try {
            valeur = conf.getString(cle);
        } catch (ConfigurationException maere) {
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter("0", nomFichier);
            paramLib.addParameter("1", cle);
            logger.error(THIS_CLASS
                    + Label.getLabel("ERR-TEC-02", paramLib));
            throw new TechnicalException(new ErrorVO("ERR-TEC-02", nomFichier
                    + ";" + cle, maere.getMessage(), true, false, false));
        }
        if (valeur == null || valeur.equals("")) {
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter("0", nomFichier);
            paramLib.addParameter("1", cle);
            logger.error(THIS_CLASS
                    + Label.getLabel("ERR-TEC-02", paramLib));
            throw new TechnicalException(new ErrorVO("ERR-TEC-02", nomFichier
                    + ";" + cle, "", true, false, false));
        }

        return valeur;
    }
}