/*
Copyright ou  ou Copr. 
Ministre des Affaires Etrangres,
Aubay SA, (21/06/07)

Ce logiciel est un programme informatique servant de framework J2EE bas sur les normes J2EE
(DAO,JDBC,MVC de Struts) et les principes multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
Il offre la possibilit de gnrer facilement des flux XML mtier pour consulter ou mettre  jour des sources
de donnes et s'interfacer avec le client riche ACube.
Ce logiciel est rgi par la licence CeCILL soumise au droit franais et respectant les principes de diffusion
des logiciels libres. Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les conditions de
la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilit au code source et des droits de copie, de modification et de redistribution
accords par cette licence, il n'est offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
seule une responsabilit restreinte pse sur l'auteur du programme,  le titulaire des droits patrimoniaux et
les concdants successifs.

A cet gard  l'attention de l'utilisateur est attire sur les risques associs au chargement,   l'utilisation,
 la modification et/ou au dveloppement et  la reproduction du logiciel par l'utilisateur tant donn sa
spcificit de logiciel libre, qui peut le rendre complexe  manipuler et qui le rserve donc  des dveloppeurs
et des professionnels avertis possdant  des  connaissances informatiques approfondies.  Les utilisateurs sont
donc invits  charger  et  tester  l'adquation  du logiciel  leurs besoins dans des conditions permettant
d'assurer la scurit de leurs systmes et ou de leurs donnes et, plus gnralement,  l'utiliser et
l'exploiter dans les mmes conditions de scurit. 

Le fait que vous puissiez accder  cet en-tte signifie que vous avez pris connaissance de la licence CeCILL,
et que vous en avez accept les termes.
*/

package acube.framework.vo;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.FileNameMap;
import java.net.URLConnection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;

import acube.framework.exception.FileWrapperException;

/**
 * VO de prsentation d'un transfert de fichier
 * 
 * @version 2.5 
 * @memberof FileWrapper
 */ 
public class FileVO implements Serializable {
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	/**
	 * <code>fichier</code>: Nom complet du fichier
	 */
	private String fichier;
	
	/**
	 * <code>extention</code>: Extention du fichier
	 */
	private String extention;
	
	/**
	 * <code>contentType</code>: Content type correspondant au fichier
	 */
	private String contentType;
	
	/**
	 * <code>inline</code>: Indique si on souhaite ou non afficher directement le contenu du fichier.
	 * La demande d'affiche directe lors du tlchargement du fichier ne se fera que si le navigateur en a la pssoibilit (format connu pour celui-ci)
	 * La valeur par dfaut est false.
	 */
	private boolean inline=false;
	
	/**
	 * <code>fluxDonnees</code>: Flux de donnes pointant vers le fichier
	 */
	private InputStream fluxDonnees;
	
	/**
	 * <code>taille</code>: taille du fichier
	 */
	private int taille;
	
    /**
     * Le constructeur de la class FileVO
     */
    public FileVO() {
        super();
    }

    /**
     * Le constructeur de la class FileVO avec paramtres
     * @param action String nom de l'action appelante 
     */
    public FileVO(String fichier) {
        super();
        this.setFichier(fichier);
    }
    
    /**
     * Le constructeur de la class FileVO avec paramtres
     * @param request Request ou l'on veut rcuprer le fichier.
     * @version 2.5 
     * @date 03/10/2007
     */
    public FileVO(HttpServletRequest request) throws FileWrapperException {
        super();
		ServletRequestContext sReq = new ServletRequestContext(request);
		
		boolean isMultipart = ServletFileUpload.isMultipartContent(sReq);
		if(!isMultipart){			
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Ceci n'est pas une requte multipart");			
		}

		int nbFile=0;
		try{
			Map params = (Map)request.getAttribute(acube.framework.technical.Constants.PARAMS_REQUEST);
			Iterator itr = params.keySet().iterator();
			while(itr.hasNext()){
				String key = (String)itr.next();
				FileItem item = (FileItem) params.get(key);
				if (!item.isFormField()) {
					if (nbFile == 1)
						throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Dfinition de plusieurs fichiers impossible");
					nbFile++;
					String name = item.getName();
					int last = name.lastIndexOf("/");
					if (last==-1)
						last = name.lastIndexOf("\\");
					if (last==-1)
						last = name.lastIndexOf(":");
					
					name=name.substring(last+1);
					this.setFichier(name);
					this.setFluxDonnees(item.getInputStream());
				}
			}
		}catch (IOException ex){
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : "+ex.getMessage(),ex);									
		}
		if ( nbFile == 0)
			throw new FileWrapperException("Erreur.FileWrapper.uploadFile : Aucun champs fichier trouv");
    }
    
	/**
	 * @return Renvoie inline.
	 */
	public boolean getInline() {
		return this.inline;
	}
	/**
	 * @param inline boolen indiquant si le fichier doit tre transmis pour affichage direct.
	 */
	public void setInline(boolean inline) {
		this.inline = inline;
	}

	/**
	 * @return Renvoie contentType.
	 */
	public String getContentType() {
		return contentType;
	}
	/**
	 * @param contentType contentType  dfinir.
	 */
	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	/**
	 * @return Renvoie extention.
	 */
	public String getExtention() {
		return extention;
	}
	/**
	 * @param extention extention  dfinir.
	 */
	public void setExtention(String extention) {
		this.extention = extention;
	}
	/**
	 * @return Renvoie fichier.
	 */
	public String getFichier() {
		return fichier;
	}
	/**
	 * @param fichier fichier  dfinir.
	 */
	public void setFichier(String fichier) {
		this.fichier = fichier;
		
		String typeFichier = new String("multipart/binary") ;

		if (fichier != null) {
			FileNameMap map = URLConnection.getFileNameMap();
			typeFichier = map.getContentTypeFor(fichier);

			int indExt = fichier.lastIndexOf('.');
			String extention = fichier.substring(indExt+1);
			this.setExtention(extention);
		}
		this.setContentType(typeFichier);
	}
	
	/**
	 * @return Renvoie fluxDonnees.
	 */
	public InputStream getFluxDonnees() {
		return fluxDonnees;
	}
	/**
	 * @param fluxDonnees fluxDonnees  dfinir.
	 */
	public void setFluxDonnees(InputStream fluxDonnees) {
		this.fluxDonnees = fluxDonnees;
		try {
			if (fluxDonnees!=null)
				this.taille = fluxDonnees.available();
			else
				this.taille = 0;
		} catch (IOException a) {
			this.taille=0;
		}

	}

	/**
	 * @return Renvoie taille.
	 */
	public int getTaille() {
		return this.taille;
	}
	/**
	 * @param fluxDonnees fluxDonnees  dfinir.
	 */
	public void setTaille(int taille) {
		this.taille = taille;
	}
	
	/**
     * renvoie l'objet sous forme d'une String
     * @return String
     */
    public String toString() {
        return "[ fichier = " + this.getFichier() + ", extention = " + this.getExtention() + ", extention = " + this.getContentType() + " ]";
    }

}