/*
 * Copyright ou  ou Copr. EDS, (26/06/08)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.webcomp.action;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;
import java.util.TreeMap;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;
import org.apache.log4j.Logger;
import org.apache.struts2.interceptor.ApplicationAware;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;
import org.apache.struts2.util.ServletContextAware;
import org.jdom.Document;
import org.jdom.Element;

import acube.framework.exception.AuthentificationException;
import acube.framework.exception.ConfigurationException;
import acube.framework.exception.FunctionalException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Configuration;
import acube.framework.technical.ConfigurationManager;
import acube.framework.technical.Constants;
import acube.framework.technical.log.BuildLogger;
import acube.framework.technical.log.Chrono;
import acube.framework.technical.log.level.Performance;
import acube.framework.utility.Tools;

import com.opensymphony.xwork2.ActionSupport;

/**
 * <p>
 * Abstract class for LISE 2.x actions.
 * </p>
 * <p>
 * This class provide a compatibility layer between LISE 2 actions
 * based on Struts1.1 and Struts2 Actions.
 * </p>
 * <p>
 * With LISE 3.x ,the request flow is now handled by this class and an
 * interceptor stack (see package webcomp aand struts-acube.xml)
 * </p>
 * 
 */
abstract public class BaseAction extends ActionSupport implements
        ServletRequestAware, ServletResponseAware, ApplicationAware,
        ServletContextAware {

    /**
     * <code>contentTypes</code> the return content types
     */
    private static Map<Integer, String> contentTypes =
            new TreeMap<Integer, String>();

    /**
     * <code>modeFonctionnement</code> application's working mode
     */
    protected static String modeFonctionnement = null;

    /**
     * <code>configMode</code> configuration of application
     */
    protected Configuration configMode = null;

    /**
     * <code>context</code> the context
     */
    private Map<String, Object> context;

    /**
     * <code>logger</code> logger, visible for all classes
     */
    protected Logger logger;

    /**
     * <code>mecanismeAuthentification</code> flag indiquate if the
     * authentification running
     */
    protected String mecanismeAuthentification = null;

    /**
     * <code>request</code> the HTTP request
     */
    private HttpServletRequest request;

    /**
     * <code>response</code> the HTTP response
     */
    private HttpServletResponse response;

    /**
     * <code>root</code> the data return by the action
     */
    private BaseActionData root = new BaseActionData();

    /**
     * <code>servletContext</code> the servlet context
     */
    private ServletContext servletContext;

    /** Comment for <code>THIS_CLASS</code> nom de la classe courante */
    protected final String THIS_CLASS = "[" + this.getClass().getName() + "]";

    static {
        BaseAction.contentTypes.put(
            acube.framework.technical.Constants.TYPE_FLUX_XML, "text/xml");
        BaseAction.contentTypes.put(
            acube.framework.technical.Constants.TYPE_FLUX_PDF,
            "application/pdf");
        BaseAction.contentTypes.put(
            acube.framework.technical.Constants.TYPE_FLUX_HTML, "text/html");
        BaseAction.contentTypes.put(
            acube.framework.technical.Constants.TYPE_FLUX_MS_EXCEL,
            "application/ms-excel");
    }

    // //////////////////////////////////////////////////////////////////////////////

    /**
     * Recuperation de la configuration concernant le mode de
     * fonctionement de l'application: (mode Test Unitaire, mode Test
     * Integration, ...)
     * 
     * @return mode de fonctionnement
     */
    public static String getModeFonctionnement() {

        return BaseAction.modeFonctionnement;
    }

    /**
     * M&eacute;thode pour les requetes en multipart avec des datas
     * 
     * @param request
     *            HttpServletRequest
     * @throws FunctionalException
     *             exception d'authentification
     * @throws TechnicalException
     *             exception technique
     */
    protected void AnalyseRequest(
            HttpServletRequest request) throws TechnicalException,
        FunctionalException {

        ServletRequestContext sReq = new ServletRequestContext(
                request);
        if (ServletFileUpload.isMultipartContent(sReq)) {
            FileItemFactory factory = new DiskFileItemFactory();
            ServletFileUpload upload = new ServletFileUpload(
                    factory);
            List items = null;
            try {
                items = upload.parseRequest(request);
                Map<String, FileItem> params = new HashMap<String, FileItem>();
                Iterator itr = items.iterator();
                while (itr.hasNext()) {
                    FileItem item = (FileItem) itr.next();
                    params.put(
                        item.getFieldName(), item);
                }
                request.setAttribute(
                    acube.framework.technical.Constants.PARAMS_REQUEST, params);
            } catch (Exception e) {
                throw new TechnicalException(
                        "Erreur.BaseAction.execute : problme de parsing des parametres de la request");
            }
        } else {
            Map params = request.getParameterMap();
            request.setAttribute(
                acube.framework.technical.Constants.PARAMS_REQUEST, params);
        }
    }

    /** {@inheritDoc} */
    public String execute() throws Exception {

        this.logger = BuildLogger.getLogger(this.request);

        this.logger.info("using BaseAction Compatibility");

        String contentType = null;

        this.executeAtStart(
            this.request, this.response, null);

        contentType = BaseAction.contentTypes.get(this.getTypeOfFlux());
        if (contentType == null) {
            contentType = "text/xml";
        }

        // Demarrage du Timer pour la mesure de PERF
        Chrono chrono = new Chrono();
        chrono.start();

        this.verifierDroitAcces(
            this.request, this.servletContext, this.getClass().getName());

        // Gestion des Paramtres de la requte et plus
        // particulirement dans le cas d'une requte mutilpart
        this.AnalyseRequest(this.request);

        // Appel de la fonction de construction des objets qui seront
        // transmis  StrustCX
        this.response.setContentType(contentType);
        this.getRoot().setData(
            this.getBeansActions(this.request));

        chrono.stop();
        if (this.logger.isEnabledFor(Performance.TRACE)) {
            this.logger.log(
                Performance.TRACE, "Temps d'excution: " + chrono.time()
                        + " ms");
        }

        this.executeAtEnd(
            null, this.request, this.response, this.getRoot().getData(),
            new ArrayList<Serializable>(), this.getTypeOfFlux());

        return SUCCESS;
    }

    /**
     * Mthode appele en fin de traitement: charge la bonne liste de
     * bean au niveau de la servlet (pour la transmettre  strutscx)
     * et choisit la bonne feuille xsl
     * 
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param forward
     *            ActionForward
     * @param beans
     *            ArrayList
     * @param erreurs
     *            ArrayList
     * @param typeFluxAttendu
     *            String le type de flux attendu par le client (XML /
     *            PDF / ...)
     * @return ActionForward the ActionForward
     */
    protected String executeAtEnd(
            String forward, HttpServletRequest request,
            HttpServletResponse response, ArrayList<Serializable> beans,
            ArrayList<Serializable> erreurs, int typeFluxAttendu) {

        if (this.logger.isDebugEnabled()) {
            this.logger.debug(this.THIS_CLASS + "BASE_ACTION executeAtEnd");
        }

        request
                .removeAttribute(acube.framework.technical.Constants.PARAMS_REQUEST);

        Date datePassee = this.getCalandar();

        // gestion des erreurs
        this.setError(
            erreurs, request, response, datePassee, typeFluxAttendu);

        return (forward);
    }

    /**
     * Mthode appele au dbut de l'action, pour effectuer les
     * diffrentes initialisations
     * 
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param activite
     *            String feuille xsl  utiliser pour cette action
     * @return HttpSession session http
     * @throws TechnicalException
     *             exception technique
     */
    protected HttpSession executeAtStart(
            HttpServletRequest request, HttpServletResponse response,
            String activite) throws TechnicalException {

        // cration du logger
        this.logger = BuildLogger.getLogger(request);

        // chargement de la configuration sur le mode de
        // fonctionnement de l'application
        this.getModeFonctionnement(request);

        return null;
    }

    /**
     * This method return the key to find the XSLT Stylesheet to use
     * as view.
     * 
     * @return StrutsCX key (XSL_KEY)
     * @deprecated use struts.xml
     */
    protected abstract String getActivite();

    /**
     * @return String
     * @deprecated
     */
    public String getActiviteV1() {

        return this.getActivite();
    }

    /**
     * This method return data that will be sent to the view
     * 
     * @param request
     *            the HTTP request
     * @return List of Serializable Object
     */
    protected abstract ArrayList<Serializable> getBeansActions(
            HttpServletRequest request) throws Exception;

    /**
     * Fonction permettant de rcuprer la date du jour. Cette date
     * est utilis pour grer les directives de cache dans le header
     * HTTP.
     * 
     * @return la date de jour
     */
    protected Date getCalandar() {

        // !!!!!!!
        // BUG Internet Explorer: IE n'arrive pas  acceder  des
        // pages HTTPS
        // ayant un header HTTP "Cache-control: no-cache"
        final int diff = -1;

        TimeZone zoneLocale = TimeZone.getDefault();
        Calendar calendrier = Calendar.getInstance(zoneLocale);
        calendrier.setTimeZone(zoneLocale);

        // On enleve 1 an, 1 mois, 1 jour, 1heure, 1 min et 1 s
        calendrier.add(
            Calendar.YEAR, diff);
        calendrier.add(
            Calendar.MONTH, diff);
        calendrier.add(
            Calendar.DAY_OF_YEAR, diff);
        calendrier.add(
            Calendar.HOUR, diff);
        calendrier.add(
            Calendar.MINUTE, diff);
        calendrier.add(
            Calendar.SECOND, diff);
        return calendrier.getTime();
    }

    /**
     * Recuperation de la configuration concernant le mode de
     * fonctionement de l'application: (mode Test Unitaire, mode Test
     * Integration, ...)
     * 
     * @param request
     *            HttpServletRequest
     * @throws TechnicalException
     *             exception technique
     */
    protected void getModeFonctionnement(
            HttpServletRequest request) throws TechnicalException {

        try {
            // rcuperation de la configuration relative au mode de
            // fonctionnement
            this.configMode = ConfigurationManager.getInstance("tuning");
        } catch (ConfigurationException e) {
            // cas ou ce fichier est inacessible: on passe normale
            BaseAction.modeFonctionnement =
                    Constants.MODE_FONCTIONNEMENT_NORMAL;
            this.mecanismeAuthentification = "true";
        }

        // dans le cas o le fichier de configuration est accessible
        if ((null != this.configMode)) {
            // rcuperation du mode de fonctionnement
            BaseAction.modeFonctionnement =
                    Tools.obtenirValeur(
                        this.configMode, "tuning.properties",
                        Constants.MODE_FONCTIONNEMENT);
            // rcuperation du mode de fonctionnement pour le
            // mcanisme d'authentification
            this.mecanismeAuthentification =
                    Tools.obtenirValeur(
                        this.configMode, "tuning.properties",
                        Constants.MODE_AUTHENTIFICATION);
        }
    }

    /**
     * Trouve un lment dans un fichier de proprits StrutsCX
     * 
     * @param propertiesFileName
     *            nom du fichier de proprits StrutsCX
     * @param tagName
     *            nom de l'lment  chercher
     * @return Element
     * @exception ConfigurationException
     *                error
     */
    protected Element getProprietes(
            String propertiesFileName, String tagName)
        throws ConfigurationException {

        // rcuperation du fichier de proprits charg par strutscx
        Document proprietes = (Document) this.context.get(propertiesFileName);

        if (proprietes == null) {
            throw new ConfigurationException(
                    this.getClass().getName(), "Le fichier de configuration "
                            + propertiesFileName + " n'est pas accessible");
        }

        // rcuperation de l'element par son nom
        Element element = proprietes.getRootElement().getChild(
            tagName);
        if (element == null) {
            throw new ConfigurationException(
                    this.getClass().getName(), "<" + tagName
                            + "> manquant dans le fichier "
                            + propertiesFileName);
        }

        return element;
    }

    /**
     * @return the request
     */
    protected HttpServletRequest getRequest() {

        return this.request;
    }

    /**
     * @return the response
     */
    protected HttpServletResponse getResponse() {

        return this.response;
    }

    /**
     * @return the root BaseActionData
     */
    public BaseActionData getRoot() {

        return this.root;
    }

    /**
     * This method return the type of return of this action.
     * 
     * @return Type of result
     * @deprecated use struts.xml
     */
    protected abstract int getTypeOfFlux();

    /**
     * @return the type of Flux
     * @deprecated
     */
    public int getTypeOfFluxV1() {

        return this.getTypeOfFlux();
    }

    /** {@inheritDoc} */
    public void setApplication(
            Map context) {

        this.context = context;
    }

    /**
     * 
     * @param erreurs
     *            erreurs
     * @param request
     *            request
     * @param response
     *            response
     * @param datePassee
     *            datePassee
     * @param typeFluxAttendu
     *            typeFluxAttendu
     */
    protected void setError(
            ArrayList<Serializable> erreurs, HttpServletRequest request,
            HttpServletResponse response, Date datePassee, int typeFluxAttendu) {

    }

    /**
     * @param root
     *            the root BaseActionData
     */
    public void setRoot(
            BaseActionData root) {

        this.root = root;
    }

    /** {@inheritDoc} */
    public void setServletContext(
            ServletContext arg0) {

        this.servletContext = arg0;
    }

    /** {@inheritDoc} */
    public void setServletRequest(
            HttpServletRequest request) {

        this.request = request;
    }

    /** {@inheritDoc} */
    public void setServletResponse(
            HttpServletResponse response) {

        this.response = response;
    }

    /**
     * Contrle le droit d'accs d'un utilisateur  une action. Un
     * utilisateur authentifi doit possder au moins un rle dans la
     * liste des rles dfinie dans la partie AUTHORISATIONS et la
     * balise d'action correspondante du document variables_fr.xml.
     * Renvoie une AuthentificationException en cas de droits
     * insuffisants.
     * 
     * @param request
     *            HttpServletRequest requte HTTP en cours,
     *            initialise avec les rles de l'utilisateur.
     * @param context
     *            ServletContext Le contexte possdant un document
     *            variables_fr.xml valide
     * @param nomClasse
     *            String Le nom complet de la classe action.
     * @throws AuthentificationException
     *             Dans le cas d'un chec d'authentification
     * @throws TechnicalException
     *             exception technique
     */
    public void verifierDroitAcces(
            HttpServletRequest request, ServletContext context, String nomClasse)
        throws AuthentificationException, TechnicalException {

    }
}
