/*
 * Copyright ou  ou Copr. EDS, (26/06/08)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.webcomp.interceptor;

import acube.framework.exception.AuthentificationException;
import acube.framework.exception.FunctionalException;
import acube.framework.exception.GenericException;
import acube.framework.vo.ErrorVO;
import acube.framework.webcomp.action.BaseAction;

import com.opensymphony.xwork2.ActionInvocation;
import com.opensymphony.xwork2.ValidationAware;
import com.opensymphony.xwork2.interceptor.AbstractInterceptor;

/**
 * Use when an exception is thrown in a action. This will generate an
 * error Stream for the client.
 * 
 */
public class ExceptionInterceptor extends AbstractInterceptor {

    /**
     * <code>serialVersionUID</code> the serialVersionUID
     */
    private static final long serialVersionUID = -728713282893521116L;

    /** {@inheritDoc} */
    public String intercept(
            ActionInvocation invocation) throws Exception {

        try {
            return invocation.invoke();
        } catch (Exception ex) {
            Object action = invocation.getAction();
            if (action instanceof ValidationAware) {
                ((ValidationAware) action).addActionError(ex.getMessage());
            }
            // gestion ErrorVO
            if (action instanceof BaseAction) {
                if (ex instanceof FunctionalException
                        || ex instanceof AuthentificationException) {
                    // exceptions dont le message remonte entirement
                    //  l'utilisateur...
                    GenericException gex = (GenericException) ex;
                    ((BaseAction) action).getRoot().getData().add(
                        gex.getError());
                } else {
                    // exceptions techniques
                    // on trace...
                    this.generateReport(ex);
                    ErrorVO vo =
                            new ErrorVO(
                                    "ERR-TEC-00", "Erreur Technique", ex
                                            .getMessage(), true, false, false);
                    ((BaseAction) action).getRoot().getData().add(
                        vo);
                }
            } else {
                this.generateReport(ex);
            }
            return "error";
        }
    }

    /**
     * Generate a report in the standart output
     * 
     * @param ex
     *            the exception to report
     */
    void generateReport(
            Exception ex) {

        ex.printStackTrace();
    }

}
