/*
 * Copyright ou  ou Copr. EDS, (26/06/08)
 * 
 * Ce logiciel est un programme informatique servant de framework J2EE
 * bas sur les normes J2EE (DAO,JDBC,MVC de Struts) et les principes
 * multi-X (multi-couches, multi-canal, multi-formats, multi-langues).
 * Il offre la possibilit de gnrer facilement des flux XML mtier
 * pour consulter ou mettre  jour des sources de donnes et
 * s'interfacer avec le client riche ACube. Ce logiciel est rgi par
 * la licence CeCILL soumise au droit franais et respectant les
 * principes de diffusion des logiciels libres. Vous pouvez utiliser,
 * modifier et/ou redistribuer ce programme sous les conditions de la
 * licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * 
 * En contrepartie de l'accessibilit au code source et des droits de
 * copie, de modification et de redistribution accords par cette
 * licence, il n'est offert aux utilisateurs qu'une garantie limite.
 * Pour les mmes raisons, seule une responsabilit restreinte pse
 * sur l'auteur du programme, le titulaire des droits patrimoniaux et
 * les concdants successifs.
 * 
 * A cet gard l'attention de l'utilisateur est attire sur les
 * risques associs au chargement,  l'utilisation,  la modification
 * et/ou au dveloppement et  la reproduction du logiciel par
 * l'utilisateur tant donn sa spcificit de logiciel libre, qui
 * peut le rendre complexe  manipuler et qui le rserve donc  des
 * dveloppeurs et des professionnels avertis possdant des
 * connaissances informatiques approfondies. Les utilisateurs sont
 * donc invits  charger et tester l'adquation du logiciel  leurs
 * besoins dans des conditions permettant d'assurer la scurit de
 * leurs systmes et ou de leurs donnes et, plus gnralement, 
 * l'utiliser et l'exploiter dans les mmes conditions de scurit.
 * 
 * Le fait que vous puissiez accder  cet en-tte signifie que vous
 * avez pris connaissance de la licence CeCILL, et que vous en avez
 * accept les termes.
 */

package acube.framework.webcomp.interceptor;

import java.util.List;

import org.apache.log4j.Logger;
import org.apache.struts2.ServletActionContext;
import org.jdom.Document;
import org.jdom.Element;

import acube.framework.exception.AuthentificationException;
import acube.framework.exception.TechnicalException;
import acube.framework.technical.Label;
import acube.framework.technical.LabelManager;
import acube.framework.vo.ErrorVO;
import acube.framework.webcomp.technical.StrutsCXConstants;

import com.opensymphony.xwork2.ActionInvocation;
import com.opensymphony.xwork2.interceptor.AbstractInterceptor;

/**
 * Use to check the access right to an action in regard of user's
 * profil.
 * 
 */
public class SecurityInterceptor extends AbstractInterceptor {

    /**
     * <code>serialVersionUID</code> the serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    /**
     * <code>logger</code> the logger
     */
    private Logger logger = Logger.getLogger(SecurityInterceptor.class);

    /**
     * <code>msgError</code> the error message
     */
    private String msgError;

    /** {@inheritDoc} */
    public String intercept(
            ActionInvocation invocation) throws Exception {

        Class<?> classeInvoquee = invocation.getAction().getClass();
        String nomClasseInvoquee = classeInvoquee.getName();

        String user = ServletActionContext.getRequest().getRemoteUser();

        this.logger.info("Vrification du droit d'accs du user " + user
                + "  la classe " + nomClasseInvoquee);

        String ressources =
                StrutsCXConstants.RESSOURCEPROPERTIES_FILES
                        + "."
                        + ServletActionContext.getRequest().getLocale()
                                .getLanguage();

        // recuperation du fichier de proprietes charg par strutscx
        Document doc =
                (Document) ServletActionContext.getServletContext()
                        .getAttribute(
                            ressources);

        if (null == doc) {
            // erreur lors de la recuperation de la propriete: on leve
            // une
            // exception
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter(
                "0", ressources);
            this.logger.error(Label.getLabel(
                "ERR-TEC-08", paramLib));
            throw new TechnicalException(
                    new ErrorVO(
                            "ERR-TEC-08", ressources, "", true, false, false));
        }

        // recuperation de l'element "AUTHORISATIONS" de ce document
        Element elementAuthorisationsProprietes =
                doc.getRootElement().getChild(
                    "AUTHORISATIONS");
        if (null == elementAuthorisationsProprietes) {
            // erreur lors de la rcupration de l'lment
            // "AUTHORISATIONS"
            LabelManager paramLib = new LabelManager();
            paramLib.addParameter(
                "0", ressources);
            paramLib.addParameter(
                "1", "AUTHORISATIONS");
            this.logger.error(Label.getLabel(
                "ERR-TEC-09", paramLib));
            throw new TechnicalException(
                    new ErrorVO(
                            "ERR-TEC-09", ressources + ";" + "AUTHORISATIONS",
                            "", true, false, false));
        }

        boolean authorisationOK = false;
        // recuperation de l'element correspondant a la classe passe
        // en
        // parametre: nomClasse
        Element elementClasse =
                elementAuthorisationsProprietes.getChild(nomClasseInvoquee);
        if (null == elementClasse) {
            // pas de classe = on accepte
            authorisationOK = true;
        } else {

            // cration d'une liste des roles contenus sous cet
            // element
            List<Element> listeRoleProprietes = elementClasse.getChildren();
            if (null == listeRoleProprietes) {
                // erreur lors de la rcupration de la liste des
                // roles
                LabelManager paramLib = new LabelManager();
                paramLib.addParameter(
                    "0", ressources);
                paramLib.addParameter(
                    "1", nomClasseInvoquee);
                this.logger.error(Label.getLabel(
                    "ERR-TEC-09", paramLib));
                throw new TechnicalException(
                        new ErrorVO(
                                "ERR-TEC-09", ressources + ";"
                                        + nomClasseInvoquee, "", true, false,
                                false));
            }

            // balayage de cette liste de roles
            for (Element elementCourant : listeRoleProprietes) {
                String roleCourant = elementCourant.getTextTrim();

                // test de ce role courant avec le role de la session
                if (ServletActionContext.getRequest().isUserInRole(
                    roleCourant)) {
                    authorisationOK = true;
                    break;
                }
            }
        }

        this.logger.info("Rsultat de la vrification des droits d'accs: "
                + authorisationOK);

        if (authorisationOK) {
            return invocation.invoke();
        } else {
            throw new AuthentificationException(
                    new ErrorVO(
                            this.msgError, user + ";" + nomClasseInvoquee, "",
                            true, false, false));
        }

    }

    /**
     * Getter of the msgError
     * 
     * @return Returns the msgError.
     */
    public String getMsgError() {

        return this.msgError;
    }

    /**
     * Setter of the msgError
     * 
     * @param msgError
     *            The msgError to set.
     */
    public void setMsgError(
            String msgError) {

        this.msgError = msgError;
    }

}
